import java.io.*;
import java.awt.*;
import java.awt.image.*;
import java.sql.SQLException;
import java.util.Vector;

import uniba.gui.*;
import uniba.core.*;
import uniba.databanker.*;


/** 
 * This class has main method of WebClass.
 */
public class WebClass extends Frame {


  MakeDTDialog          makedtDialog;     
  MenuBar               mainMenuBar;
  CategoriesDialog      categoriesDialog;
  MakeClusterDialog     makeClDialog;
  MakeBayesianProbabilitiesDialog  makeBayesianProbabilitiesDialog;
  AboutDialog            aboutDialog;  
  ViewFeaturesDialog     viewFeaturesDialog;  
  ExtractFeaturesDialog  extractFeaturesDialog;
  BrowserDialog          browserDialog;
  InserimentoDocumentoTT inserimentoDocumentoTT;
  InserimentoCategoria   inserimentoCategoria = null;
  CancellazioneCategoria cancellazioneCategoria = null;  
  PageClassifyDialog    pageClassifyDialog;
  PageRepresentDialog   pageRepresentDialog;
  ViewCentroidsDialog            viewCentroidsDialog;
  CategoriesManager    cgm;
  TrainingPagesManager trpm;
  FeaturesManager      ftm; 
  RepresentManager     rpm;
  String utente = null;
  static Databanker db = null;
  ProgressWindow progressWindow = new ProgressWindow("Processing..."); 	
  TestingPagesManager  tspm;
  ViewBayesianProbabilitiesDialog  viewBayesianProbabilitiesDialog; 
  TestingPagesClassifyDialog     testingPagesClassifyDialog; 
  NaturalQueryDialog nlq; 
  NaturalLanguageQueryDialog nlqd;	
  ViewDocumentDialog viewDocumentDialog;
  DocumentDialog documentDialog;
  ControlDialog ctr;
  UserDialog userDialog;		
  DeleteUserDialog eud;
  MakeRejectThresholdForCategoriesDialog makeRejectThresholdForCategoriesDialog; 
  RejectPagesManager   rjpm;
  ViewRejectThresholdsDialog       viewRejectThresholdsDialog;
  ViewDTDialog          viewdtDialog;
  StopwordDialog stopDial;
  StopwordsDialog stopsDial;
  ChangeUserDialog cud;
  ModifyCategoryDialog mcd;	
  int mode = 0;

            
   

  /**
   * Istanzia tutti gli oggetti necessari al funzionamento e prepara
   * l'interfaccia utente. Il parametro <code>mode</code> permette
   * di visualizzare o meno lo standard output dei processi figli
   * creati da <code>WebClass</code> quando si classifica con gli
   * gli alberi di decisione. Se si desidera non visualizzare
   * l'output di tali processi basta scegliere 0. Per interi
   * maggiori di 0 si hanno vari modi di visualizzazione:
   * <ul><li>finestra ridotta ad icona</li><li>finestra ingrandita</li>
   * <li>etc.</li></ul>  
   */
  public WebClass(String utente, int mode) {
   	super("WebClass: A Web Mining Tool - User: " + utente);
	setBackground(new Color(0).gray);  
	resize(800, 500);
	this.utente = utente;
      this.mode = mode;
	try {
		setupMenuBar();
		cgm  = new CategoriesManager(db, utente);
		trpm = new TrainingPagesManager(db, utente, cgm);
		tspm = new TestingPagesManager(db, utente);
		rjpm = new RejectPagesManager(utente, db);
		ftm  = new FeaturesManager(cgm, trpm, db, utente);
	
            int repType = ((Integer)db.getConfiguration(utente).elementAt(4)).intValue();
	      rpm  = new RepresentManager(ftm, repType, db, utente);
            setupDialogs();
	} 
      catch (Exception e) {
            new StampaErrore(this, 
                             "WebClass", 
                             "Exception:\n"+e.getMessage());
            e.printStackTrace();
      }
	show();
  }           //End  WebClass


  /**
  * Richiama la funzione di <i>login</i>. Se i dati sono immessi, si avvia
  * <b>WebClass</b>, altrimenti si termina immediatamente.
  */
  public static void main(String args[]) {
	if (args.length != 2) {
		System.out.println("Sintassi: java WebClass nomeOrigineDati mode.");
		System.exit(0);
	}
	Vector elencoLog = null;
	
	try {
	   db = new Databanker(args[0]);
         int mode = new Integer(args[1]).intValue();
	   elencoLog = db.getLogin();
	   String utente = null;
         int time = 0;
         do {
            Login log = new Login(new Frame(), "WebClass - Login", elencoLog, db);
            log.setVisible(true);
	   	utente = log.getUser();
		if (utente == null) {
			new StampaErrore(new Frame(), "WebClass - Login", "Login e/o Password Errata");
                  time++;
            }
         } while ( (utente == null) & (time <= 2) );
         if ( (time > 2) & (utente == null) )
                System.exit(0);
			
	   MultiServer ms = new MultiServer(db, utente);
	   db.automInit(utente, true);
	   WebClass app = new WebClass(utente, mode);
	   app.move(0,0);
      } catch (Exception e) {
		    new StampaErrore(new Frame(), "WebClass", "Exception:\n" + e.getMessage());
		    e.printStackTrace();	
		    System.exit(0);
	}
  }     // End main 
 

 
  /** 
   * Event handler of the class.
   */
  public boolean handleEvent(Event evt) {
	if (evt.id==evt.WINDOW_DESTROY) {
        System.exit(0);
	  return true;
	}
	else
	if (evt.id==evt.ACTION_EVENT) {
	  if (evt.target instanceof MenuItem) {
		if (evt.arg.equals("New...")) {
			try {
				userDialog = new UserDialog(this, "New Usr", db);
			} catch (Exception e) {
	                   new StampaErrore(this, 
                         	"User Dialog", 
                              "Exception:\n" + e.getMessage());e.printStackTrace();
              	}
			return true;
		}
		else	
		if (evt.arg.equals("View configuration")) {
			try {
				userDialog = new UserDialog(this, utente, 
					"View configuration of " + utente, db);
				ftm.updateConfiguration();
				int repType = ((Integer)db.getConfiguration(utente).elementAt(4)).intValue();
				rpm.updateRappType(repType);
			} catch (Exception e) {
	                   new StampaErrore(this, 
                         	"User Dialog", 
                              "Exception:\n" + e.getMessage());e.printStackTrace();
              	}
			return true;
		}
		else	
		if (evt.arg.equals("Erase...")) {
			try {
				eud = new DeleteUserDialog(this, utente, db);
			} catch (Exception e) {
	                   new StampaErrore(this, 
                         	"Delete User Dialog", 
                              "Exception:\n" + e.getMessage());e.printStackTrace();
              	}
			return true;
		}
		else	
		if (evt.arg.equals("Change...")) {
			try {
				cud = new ChangeUserDialog(this, utente, db);
				String newUser = cud.getUser();
				if (newUser != null) {
					db.automInit(newUser, true);
					WebClass app = new WebClass(newUser, mode);
					setVisible(false);
					app.move(0,0);
				}
			} catch (Exception e) {
	                   new StampaErrore(this, 
                         	"Change User Dialog", 
                              "Exception:\n" + e.getMessage());e.printStackTrace();
              	}
			return true;
		}
		else	
		if (evt.arg.equals("Insert a Stopword")) {
			try {
				stopDial = new StopwordDialog(this, utente, db);
			} catch (Exception e) {
	                   new StampaErrore(this, 
                         	"Insert a Stopword", 
                              "Exception:\n" + e.getMessage());e.printStackTrace();
              	}
			return true;
		}
		else
		if (evt.arg.equals("Load Stopword")) {
			try {
				stopsDial = new StopwordsDialog(this, utente, db);
			} catch (Exception e) {
	                   new StampaErrore(this, 
                         	"Load Stopword", 
                              "Exception:\n" + e.getMessage());e.printStackTrace();
              	} finally {
				setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
			return true;
		}
		else
		if (evt.arg.equals("Quit")) {
		  System.exit(0);
		  return true;
		}
		else
		if (evt.arg.equals("World Wide Web")) {
              try {
                  browserDialog = 
                      new BrowserDialog(this, 
                                        db.getConfiguration(utente), 
                                        db.getStopword(utente), db);
                  browserDialog.show();
              } catch (Exception e) {
                   new StampaErrore(this, "Browser Dialog", "Exception:\n" + e.getMessage());
              }
		  return true;
		}
		else
		if (evt.arg.equals("Insert Training\\Testing\\Reject")) {
                  try {
                      inserimentoDocumentoTT = 
                             new InserimentoDocumentoTT(this, 
                                                        db.getCategories(utente));  
                      if (inserimentoDocumentoTT.getCategory() != null) {
			       setCursor(new Cursor(Cursor.WAIT_CURSOR));
				 new Thread_1(this, inserimentoDocumentoTT, db, utente, trpm, tspm, rjpm);
			    }
 			} catch (Exception e) {
                          new StampaErrore(this, 
                                           "getCategories", 
                                           "Exception:\n"+e.getMessage());
							 e.printStackTrace();  
                  }
                  return true;
		}
		else
            if (evt.arg.equals("Delete Category")) {
                  try {
                      cancellazioneCategoria = 
                                new CancellazioneCategoria(this, 
                                                           db.getCategories(utente));
                      if (cancellazioneCategoria.getCategory() != null) {
			           setCursor(new Cursor(Cursor.WAIT_CURSOR));
                             new Thread_2(this, cancellazioneCategoria, 
                                          db,   utente, trpm, tspm, cgm);
                      }
                  } catch (Exception e) {
                            new StampaErrore(this, 
                                             "inserisciDocumentoTT",
                                             "Exception:\n"+e.getMessage());
                  }
			return true;
            }
            else
            if (evt.arg.equals("Modify Category")) {
                  try {
                      mcd = new ModifyCategoryDialog(this, utente, db.getCategories(utente), 				db, cgm, trpm, tspm);
                  } catch (Exception e) {
                            new StampaErrore(this, 
                                             "inserisciDocumentoTT",
                                             "Exception:\n"+e.getMessage());
                  }
			return true;
            }
            else
		if (evt.arg.equals("Delete Documents")) {
			try {
				viewDocumentDialog = new ViewDocumentDialog(this, db.getAllDocuments(utente, null), db, utente);
                        if  (viewDocumentDialog.getUrlDoc() != null) {
					setCursor(new Cursor(Cursor.WAIT_CURSOR));
					new Thread_5(this, viewDocumentDialog, db, cgm, trpm, tspm, rjpm); 
				}
			} catch (Exception e) {
				new StampaErrore(this, 
                                         "View Documents",
                                         "Exception:\n"+e.getMessage());
			}
			return true;
		}
            else
		if (evt.arg.equals("View All Documents")) {
			try {
				documentDialog = new DocumentDialog(this, db.getAllDocuments(utente, null), db, trpm, tspm, rjpm, utente);
                        //if  (viewDocumentDialog.getUrlDoc() != null) {
				//	setCursor(new Cursor(Cursor.WAIT_CURSOR));
				//	new Thread_5(this, viewDocumentDialog, db, cgm, trpm, tspm, rjpm); 
				//}
			} catch (Exception e) {
				new StampaErrore(this, 
                                         "View Documents",
                                         "Exception:\n"+e.getMessage());
			}
			return true;
		}
            else
            if (evt.arg.equals("Insert Category")) {
                  inserimentoCategoria = new InserimentoCategoria(this);  
			if (inserimentoCategoria.getCategory() != null) {
                       setCursor(new Cursor(Cursor.WAIT_CURSOR));
                       new Thread_3(this, inserimentoCategoria, 
                                    db,   utente, cgm);
                  }
                  return true;
            }
            else if (evt.arg.equals("Represent a page")) {
                try {
                    pageRepresentDialog = new PageRepresentDialog(this,
                                                                  ftm,
                                                                  rpm,
                                                                  db.getConfiguration(utente), 
                                                                  db.getStopword(utente), db);
                    pageRepresentDialog.show(); 
                } catch (Exception e) {
                     new StampaErrore(this, 
                                      "pageRepresentDialog", 
                                      "Exception:\n"+e.getMessage());
                     }
  
		   return true;
		}
            else if (evt.arg.equals("Enphasis Construction")) {
                try {
					setCursor(new Cursor(Cursor.WAIT_CURSOR));
                	db.setEnfRepresentation(utente);
                	System.out.println("fine Costruzione");
					setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					Exception ex = new Exception("Costruzione enfasi eseguita correttamente");
					throw ex;
                } catch (Exception e) {
                	 e.printStackTrace();
                	 new StampaErrore(this, 
                                      "Enphasis Construction", 
                                      "Exception:\n"+e.getMessage());
                     }
  
		   return true;
		}
				
		else
		if (evt.arg.equals("View current features")) {
              viewFeaturesDialog = new ViewFeaturesDialog(this, ftm);
		  viewFeaturesDialog.setupFeaturesList();
		  viewFeaturesDialog.show();
		  return true;
		} 
		else
		if (evt.arg.equals("Extract new features")) {
              extractFeaturesDialog = new ExtractFeaturesDialog(this, 
										    ftm,
										    cgm,
										    progressWindow,
        									    db,
										    utente);
		  return true;
		} 
		else
		if(evt.arg.equals("Configuration files...")) {
			//ConfigDialog configDialog = new ConfigDialog(this);
			//configDialog.show();
			return true;
		} else
            if ( evt.arg.equals("Simple Matching") ) {
                 nlq = new NaturalQueryDialog(this, "Simple Matching", db, utente, rpm);
            } else
		if ( evt.arg.equals("Dice's Coefficient") ) {
                 nlq = new NaturalQueryDialog(this, "Dice's Coefficient", db, utente, rpm);
            }
        else
        if ( evt.arg.equals("Idf") ) {
                 nlq = new NaturalQueryDialog(this, "Idf", db, utente, rpm);
            }
		else
		if (evt.arg.equals("Network Configuration...")) {
			//ConfigNetDialog configNetDialog = new ConfigNetDialog(this);
			return true;
		} 
		else
		if (evt.arg.equals("Pages representation...")) {
		  //preferenceDialog.show();
		  return true;
		} 
		else
		if (evt.arg.equals("Make centroids")) {
			try {
	              makeClDialog = new MakeClusterDialog(this, cgm, ftm, trpm, utente, db);
			  makeClDialog.show();
			}catch(Exception e) {
                            new StampaErrore(this, 
                                             "Represent training set",
                                             "Exception:\n"+e.getMessage());}
		  return true;
		}
		else
		if (evt.arg.equals("View centroids")) {
                  viewCentroidsDialog = new ViewCentroidsDialog(this, ftm, cgm, utente, db);
			viewCentroidsDialog.show();
			return true;
		} 
		else
		if(evt.arg.equals("Make decision tree")) {
			makedtDialog = new MakeDTDialog(this, utente, db, cgm, mode);
			makedtDialog.setVisible(true);
			return true;
		}
		else
		if (evt.arg.equals("Make Bayesian Probabilities")) {
                  makeBayesianProbabilitiesDialog=new MakeBayesianProbabilitiesDialog(this,
														  cgm,
														  ftm,
														  trpm,
														  utente,
														  db);
		      makeBayesianProbabilitiesDialog.show();
			return true;
		}
		else
		if (evt.arg.equals("View Bayesian Probabilities")) {
                  viewBayesianProbabilitiesDialog = new ViewBayesianProbabilitiesDialog(this,
														  utente,
														  db);
		      return true;
		} 
		else
            if(evt.arg.equals("Make Reject Thresholds for Categories")) {
			makeRejectThresholdForCategoriesDialog =new MakeRejectThresholdForCategoriesDialog(this,
																     cgm,
																     trpm,
																     rjpm,
																     progressWindow,
																     db, utente);
		      makeRejectThresholdForCategoriesDialog.setVisible(true);
			return true; 
		}	
		else
		if (evt.arg.equals("View Reject Thresholds for Categories")) {
			viewRejectThresholdsDialog = new ViewRejectThresholdsDialog(this, utente, db);
			viewRejectThresholdsDialog.setVisible(true);
			return true;
		} 
	    else	
		if (evt.arg.equals("View decision tree")) {
		  viewdtDialog = new ViewDTDialog(this, ftm, utente); 
		  viewdtDialog.loadDecisionTree();
              viewdtDialog.show(); 
		  return true;
		}
		else
		if (evt.arg.equals("View Categories")) {
              categoriesDialog = new CategoriesDialog(this, cgm, trpm, tspm, db, utente);
		  return true;
		}
		else
		if (evt.arg.equals("Page")) {
			try {
				pageClassifyDialog = new PageClassifyDialog
					(this, cgm, rpm, trpm, ftm, utente, db, mode);
			} catch (Exception ex){
                        new StampaErrore(this, "pageClassifyDialog",
                        "Exception:\n"+ex.getMessage());}
			return true;
		}else
		if (evt.arg.equals("QTFxWIDF")) {
			try {
				nlqd = new NaturalLanguageQueryDialog(this, utente, db);
			} catch (Exception ex){
                      	new StampaErrore(this, "NaturalLanguageQueryDialog",
                        "Exception:\n"+ex.getMessage());ex.printStackTrace();}
			return true;
		}
		else
		if (evt.arg.equals("Control")) {
			try {
				ctr = new ControlDialog(this, utente, db);
			} catch (Exception ex){
                      	new StampaErrore(this, "ControlDialog",
                        "Exception:\n"+ex.getMessage());ex.printStackTrace();}
			return true;
		}
		else
		if (evt.arg.equals("All testing pages")) {
			try {
                	testingPagesClassifyDialog = new TestingPagesClassifyDialog(this,
                    cgm, tspm, trpm, ftm, utente, db, mode);
			} catch(Exception ex) {
                        new StampaErrore(this, "TestingPagesClassifyDialog",
                        "Exception:\n"+ex.getMessage());}
		  return true;
		}	else
		if (evt.arg.equals("About")) {
                  aboutDialog = new AboutDialog(this);
			return true;
		}
	  }
	}
	return false;
  }   
   

 
 


 
   /*
    * Sets the dialog bar.
    */
 void setupDialogs() throws SQLException {
	Toolkit t=getToolkit();
	 int scw=t.getScreenSize().width;
	 int sch=t.getScreenSize().height;
	 int w;
	 int h;
	
	
/*
	preferenceDialog = new PreferenceDialog(this, rpm);
	
		
	
	makedtDialog = new MakeDTDialog(this);
	 w=makedtDialog.size().width;
	 h=makedtDialog.size().height;
	 makedtDialog.move((scw/2)-(w/2),(sch/2)-(h/2));
	
	 w=viewdtDialog.size().width;
	 h=viewdtDialog.size().height;
	 viewdtDialog.move((scw/2)-(w/2),(sch/2)-(h/2)); 
	pageClassifyDialog = new PageClassifyDialog(this,cgm,rpm,trpm,ftm);
	 w=pageClassifyDialog.size().width;
	 h=pageClassifyDialog.size().height;
	 pageClassifyDialog.move((scw/2)-(w/2),(sch/2)-(h/2)); 
	testingPagesClassifyDialog = new TestingPagesClassifyDialog(this,cgm,tspm,rpm,trpm,ftm); 
	
   configDialog = new ConfigDialog(this); 
			 
	// statusPanel = new StatusPanel(); 
	// add("Center",statusPanel);             */
  }

  
 /*  
  * Sets the menu bar.
  */
  void setupMenuBar () throws SQLException  {
	mainMenuBar = new MenuBar();
	  Menu userMenu = new Menu("User");
		MenuItem newUserMenu = new MenuItem("New...");
		MenuItem eraseUserMenu = new MenuItem("Erase...");
		MenuItem viewConfig = new MenuItem("View configuration");
		MenuItem changeUserMenu = new MenuItem("Change...");
		userMenu.add(newUserMenu);
		userMenu.add(viewConfig);
		userMenu.add(changeUserMenu);
		userMenu.add(eraseUserMenu);
		userMenu.addSeparator();
		Menu stopword = new Menu("Stopword");
		userMenu.add(stopword);	
		userMenu.addSeparator();
			MenuItem loadStopword = new MenuItem("Load Stopword");
			stopword.add(loadStopword);
			MenuItem insStopword = new MenuItem("Insert a Stopword");
			stopword.add(insStopword);
		Menu categoria = new Menu("Categories");
            	MenuItem inserisciCategoria = new MenuItem("Insert Category");
	            categoria.add(inserisciCategoria);
	            MenuItem cancellaCategoria = new MenuItem("Delete Category");
	            categoria.add(cancellaCategoria);
	            MenuItem modificaCategoria = new MenuItem("Modify Category");
	            categoria.add(modificaCategoria);
			MenuItem viewCategories = new MenuItem("View Categories");
			categoria.add(viewCategories);

		userMenu.add(categoria);
			Menu documento = new Menu("Documents");
		      MenuItem insDocumentoTT = new MenuItem("Insert Training\\Testing\\Reject");
			MenuItem delDoc = new MenuItem("Delete Documents");
			documento.add(insDocumentoTT);
			documento.add(delDoc);
			MenuItem viewDoc = new MenuItem("View All Documents");
			documento.add(viewDoc);

		userMenu.add(documento);
            userMenu.addSeparator();

		Menu retrivalMenu = new Menu("Retrieval");
		Menu function = new Menu("Serial Search");
            Menu bool = new Menu("Boolean Search");
		retrivalMenu.add(function);
            retrivalMenu.add(bool);
            MenuItem NatLanQuery = new MenuItem("QTFxWIDF");
		function.add(NatLanQuery);
		MenuItem localMatch = new MenuItem("Simple Matching");
		MenuItem globalMatch = new MenuItem("Dice's Coefficient");
		MenuItem idf = new MenuItem("Idf");
            bool.add(localMatch);
		function.add(globalMatch);
		function.add(idf);
		userMenu.add(retrivalMenu);
		userMenu.addSeparator();

			MenuItem controllo = new MenuItem("Control");
			 
            userMenu.add(controllo);
		userMenu.addSeparator();
		MenuItem quit = new MenuItem("Quit");
		userMenu.add(quit);
		
		
            

	  Menu browseMenu = new Menu("Browse");
		MenuItem browseMenuWWW = new MenuItem("World Wide Web");
		browseMenu.add(browseMenuWWW);
	  Menu featuresMenu = new Menu("Features");
		MenuItem featuresMenuView = new MenuItem("View current features");
		MenuItem featuresMenuExtract = new MenuItem("Extract new features");
		MenuItem featuresMenuRepresentAPage = new MenuItem("Represent a page");
		MenuItem featuresMenuEnphasis = new MenuItem("Enphasis Construction");
		featuresMenu.add(featuresMenuExtract);
		featuresMenu.add(featuresMenuView);
		featuresMenu.addSeparator();
		
		featuresMenu.add(featuresMenuRepresentAPage);
        featuresMenu.add(featuresMenuEnphasis);    
            
	  Menu learnMenu = new  Menu("Learn");
		MenuItem learnMenuMakeDT = new MenuItem("Make decision tree");
		MenuItem learnMenuViewDT = new MenuItem("View decision tree");
		MenuItem learnMenuMakeCluster = new MenuItem("Make centroids");
		MenuItem learnMenuViewCluster = new MenuItem("View centroids");
		MenuItem learnMenuMakeBayesianProbabilities=new MenuItem("Make Bayesian Probabilities");
            MenuItem learnMenuViewBayesianProbabilities=new MenuItem("View Bayesian Probabilities");
		MenuItem learnMenuMakeRejectThresholdForCategories=new MenuItem("Make Reject Thresholds for Categories"); 
		MenuItem learnMenuViewRejectThresholds=new MenuItem("View Reject Thresholds for Categories");
		
		learnMenu.add(learnMenuMakeDT);
		learnMenu.add(learnMenuViewDT);
		learnMenu.addSeparator();
		learnMenu.add(learnMenuMakeCluster);
		learnMenu.add(learnMenuViewCluster); 
            learnMenu.addSeparator(); 
	      learnMenu.add(learnMenuMakeBayesianProbabilities);
		learnMenu.add(learnMenuViewBayesianProbabilities); 
            learnMenu.addSeparator(); 
		learnMenu.add(learnMenuMakeRejectThresholdForCategories);  
		learnMenu.add(learnMenuViewRejectThresholds);
	  Menu classifyMenu = new  Menu("Classify");  
		MenuItem classifyMenuPage = new MenuItem("Page");
		MenuItem classifyMenuBookmark = new MenuItem("Bookmark");
		classifyMenuBookmark.disable();
		MenuItem classifyMenuAltavista = new MenuItem("Altavista search result");
		classifyMenuAltavista.disable();
		MenuItem classifyMenuTestPages = new MenuItem("All testing pages");
		classifyMenu.add(classifyMenuPage);
		classifyMenu.add(classifyMenuBookmark);
		classifyMenu.add(classifyMenuAltavista);
		classifyMenu.add(classifyMenuTestPages);
	  
	           
                  
            
	  	

		
	  
	  Menu helpMenu = new  Menu("Help");    
		MenuItem helpMenuIndex = new MenuItem("Index");
		helpMenuIndex.disable();
		MenuItem helpMenuAbout = new MenuItem("About");
		helpMenu.add(helpMenuIndex);
		helpMenu.add(helpMenuAbout);
	
	mainMenuBar.add(userMenu);
	mainMenuBar.add(browseMenu);  
	mainMenuBar.add(featuresMenu);
	mainMenuBar.add(learnMenu);
	mainMenuBar.add(classifyMenu);
	mainMenuBar.add(helpMenu);
	setMenuBar(mainMenuBar);
  }  
}
