package uniba.core;

import java.util.*;


/**
 * Questa classe contiene tutte le ancore di un documento. Per ogni ancora si memorizzano
 * <ul><li>l'attributo <i>href</i></li><li>le parole che compaiono all'interno dell'ancora</li>
 * <li>gli indirizzi delle immagini che compaiono all'interno dell'ancora</li></ul>
 * I metodi <b>newLink</b> e <b>addElement</b> permettono di popolare l'oggetto <i>Ancore</i>,
 * mentre gli altri metodi servono per recuperare i valori. Ecco un esempio di recupero valori:
 * <br><br><code>HtmlPage hp = new HtmlPage(file, db, utente);<br><br>Ancora a = hp.getLinkA();
 * <br><br>while (!a.endOfLink) {<blockquote>System.out.println("Href = " + a.getHref());<br>
 * <br>while (!a.endOfElements)<blockquote>System.out.println("  Elem. = "+a.nextElement());
 * </blockquote> a.nextLink();</blockquote> }</code><br><br>
 */
public class Ancore {

	public boolean endOfLink = false;
	public boolean endOfElements = false;
	private int index = 0;		
      private Vector inizioFrasi = new Vector();
      private Vector frasi = new Vector();

	public Ancore() {}

	/**
	 * Questo metodo viene utilizzato per inserire una nuova ancora.
       */
	public void newLink(String href) {
		inizioFrasi.addElement(new Integer(frasi.size()));
		frasi.addElement( SQL.correggiApice(href.toLowerCase()) );
	}

	/**
       * Una volta inserita una nuova ancora, questo metodo inserisce le immagini 
       * e le parole associate all'ancora. Non sono ammessi elementi duplicati.
       * (Se un elemento  gi stato inserito, non viene inserito nuovamente e
       * pertanto compare una sola volta nella struttura.
       */
	public void addElement(String elem) {
            if ( !frasi.contains(elem.toLowerCase()) )
			frasi.addElement( SQL.correggiApice(elem.toLowerCase()) );
      }

   
	/**
       * Passa alla prossima ancora
       */
	public void nextLink() {
		endOfElements = false;
	}


      /**
       * Restituisce il prossimo elemento contenuto nell'ancora. Esso pu essere
       * una parola oppure l'indirizzo di una immagine.
       */
	public String nextElement() {
		if ( !inizioFrasi.contains(new Integer(index)) ) {
			if (index < frasi.size()) {
				index = index + 1;
			      return (String)frasi.elementAt(index-1);
			} else {
				endOfLink = true;
				endOfElements = true;
				return null;
			}
		} else {
			endOfElements = true;
			return null;
		}
	}


	/**
       * Restituisce il valore dell'attributo <i>HREF</i> della'ancora
       */
	public String getHref() {
		if (index < frasi.size()) {  
			index = index + 1;
			return (String)frasi.elementAt(index-1);
		} else {
			endOfLink = true;
			endOfElements = true;
			return null;
		}
	}

	
}			 
		
 