/**
 * Questo package racchiude tutte le classi originariamente nominate <i>xxxManager</i>.
 */
package uniba.core;

import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import java.sql.SQLException;
import uniba.databanker.*;


/**
 * Mantiene informazioni sulle categorie
 */
public class CategoriesManager {

  private Vector categories = new Vector();
  public String utente = null;
  private Databanker db = null;
  
  /**
   * Legge le categorie di un utente dal database
   */
  public CategoriesManager(Databanker db, String utente)
    throws SQLException {
	
      categories = db.getCategories(utente);
	categories.addElement("UNKNOWN"); 
      this.utente = utente;
      this.db = db;
  }
  
  /**
   * Ricarica le categorie quando esse cambiano. Quando si effettua una qualsiasi
   * operazione che modifica l'insieme delle categorie, si deve richiamare questo
   * metodo per notificare all'oggetto <b>CategoriesManager</b> tale cambiamento
   */
  public void updateCategory() throws SQLException {
      categories.removeAllElements();
	categories = db.getCategories(utente);
      categories.addElement("UNKNOWN"); 
  }	
		

  /**
   * Restituisce l'indice di <i>category</i> nel vettore <i>categories</i>
   */
  public int categoryIndex(String category) {
	return categories.indexOf(category);
  }

  /**
   * Restituisce il nome della i-esima categoria
   */
  public String categoryName(int i) {
	return (String) categories.elementAt(i);
  }

  /**
   * Vero se <i>category</i> esiste in <i>categories</i>
   */
  public boolean existCategory(String category) {
	return categories.contains(category); 
  }

  /**
   * Restituisce le categorie, compreso UNKNOWN.
   */
  public Vector getCategories(){
      return categories;
  }

  /**
   * Restituisce le categorie, compreso UNKNOWN.
   */
  public Enumeration listCategories() {
	return categories.elements();
  }

  /**
   * Restituisce le categorie, escluso UNKNOWN.
   */
  public Enumeration listTrainingCategories() {
	Vector categoriesToReturn = new Vector();
	String category = "";
	Enumeration ecat=categories.elements();
  
	while (ecat.hasMoreElements()) {
	  category = (String) ecat.nextElement();
	  if ( !category.equals("UNKNOWN"))
		 categoriesToReturn.addElement(category);
	}   
	return categoriesToReturn.elements();
  }

  /**
   * Restituisce il numero di categorie
   */
  public int totalCategories() {
         // "Unknown" category is not included
        return categories.size()-1; 
  }

} 
