package uniba.core;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.sql.SQLException;

import uniba.core.*;
import uniba.databanker.*;

/**
 * Implementa vari metodi di classificazione.
 */
public class Classifier {

	CategoriesManager cgm;
	TrainingPagesManager trpm;
      Databanker db;
	String utente;

	public Classifier(CategoriesManager cgm, TrainingPagesManager trpm, String utente,Databanker db) {
		this.cgm  = cgm;
		this.trpm = trpm;
	    	this.db   = db;
		this.utente = utente;

	}
//------------------------------------------------------------------------------------------
       
/**
* Effetua la classificazione della pagina HTML <i>Page</i><br>secondo il metodo
* dei <i>centroidi</i>
*/

public String classifyCentroidPage(double[] rappresentazione)
		 throws SQLException, OperationException {
	double[] rap = rappresentazione;
	return getClusteringResult(rap);
}

//------------------------------------------------------------------------------------------
   
/**
* Effetua la classificazione della pagina HTML <i>Page</i><br>secondo il metodo
* <i>Nearest Neighbor</i> con fattore pari a <i>k</i>.
*/

public String classifyKNNPage(double[] rappresentazione)
		 throws SQLException, OperationException{
	double[] rap = rappresentazione;
	int kFactor = 0;
	Vector config = db.getConfiguration(utente);
	kFactor = ((Integer)config.elementAt(7)).intValue();
	return getKNNResult(kFactor,rap);
}  

//------------------------------------------------------------------------------------------

/**
* Effettua la classificazione della pagina HTML <i>page</i><br>secondo il metodo
*<i>Nearest Neighbor</i>.
*/

public String classifyNNPage(double[] rappresentazione)
		throws SQLException, OperationException{
	double[] rap = rappresentazione;
	return getNNResult(rap);
} 

//--------------------------------------------------------------------------------------------

/**
* Effettua la classificazione della pagina HTML <i>page</i><br>secondo il metodo
*<i>Naive Bayesian</i>.
*/
	
public String classifyNaiveBayesianPage(double[] rappresentazione)
		throws SQLException,OperationException{
	double[] rap = rappresentazione;
	return getNaiveBayesianResult(rap);
}

//-----------------------------------------------------------------------------------------

/**
* Restituisce il risultato della classificazione mediante centroidi.
*/

String getClusteringResult(double features[]) throws SQLException, OperationException{
	String result;
	double tVector[] = new double[cgm.totalCategories()];
	double maxDist = 0;
	int maxCategory = 0;
	String res = "";
	String categoryName;
	for (int i = 0;i < cgm.totalCategories(); i++) {
		categoryName = cgm.categoryName(i);
	      tVector[i] = db.getThreshold(utente, categoryName);
		double value[] = db.getCentroid(utente, categoryName);
		double d = Vettori.distanza(value,features);
		if (d > maxDist) {
			maxDist = d; maxCategory = i;}
	}
	if( maxDist <= (double) tVector[maxCategory]/100) 
		result = cgm.categoryName(cgm.totalCategories());   
	else
		result = cgm.categoryName(maxCategory);
	return result;
}

//---------------------------------------------------------------------

/** 
* Restituisce il risultato della classificazione mediante Nearest Neighbor<br>
* con fattore pari a k.
*/  
	
String getKNNResult(int kFactor, double features[]) 
		throws SQLException, OperationException {
	String result = "";
	int currentCategory = 0;
	Vector kNNVector = new Vector();
	int idDoc = 0;
	String urlPage;
	String categoryName;
	Enumeration lp = trpm.listPages();
	while (lp.hasMoreElements()) {
		StringTokenizer catPage = new StringTokenizer((String)lp.nextElement());
		idDoc = (new Integer(catPage.nextToken())).intValue();
		categoryName = catPage.nextToken();
		currentCategory = cgm.categoryIndex(categoryName);
		double value[] = db.getRepresentedPage(idDoc, utente);		
		double newDist = Vettori.distanza(value, features);
		System.out.println(newDist + " " + currentCategory);	
		kNNVector.addElement(new kNNVectorElement(newDist, currentCategory));
		if(kNNVector.size() >= 2*kFactor) {
	   		kNNVector = quicksort(kNNVector);
	   		kNNVector.setSize((trpm.getNumber() > kFactor) ? kFactor 
						: trpm.getNumber());
		}
	} 
	kNNVector = quicksort(kNNVector);
	kNNVector.setSize((trpm.getNumber() > kFactor) ? kFactor : trpm.getNumber());
	Enumeration num = kNNVector.elements();
	result = kNNEvalFunction(kNNVector);
	return result;                  
} // end getKNNResult 

//----------------------------------------------------------------------


/** 
* Restituisce il risultato della classificazione mediante<br>
* il metodo Naive Bayesian.
* @author Marta Caputi 1999
*/

String getNaiveBayesianResult(double features[])
		throws SQLException, OperationException{
	Vector probVector=new Vector();
	String result="";
	double numerat=0;
	double denominat=0;
//	double pDiC=1;
//	pDiC /= cgm.totalCategories();
	int[] docPerCat = trpm.getTotPageInCategory();
	int totDoc = trpm.getNumber();	
	double tVector[] = new double[cgm.totalCategories()];
	for (int i = 0;i < cgm.totalCategories();i++) 
		tVector[i] = db.getThreshold(utente, cgm.categoryName(i));
	String line;
	for(int t=0;t<cgm.totalCategories();t++){
		double p1=0;
		double p2=1;
		numerat=0;
		int i = 0;
		double[] prob = db.getProbabilities(utente, cgm.categoryName(t));
		while (i < prob.length) {
			p1 = Math.pow((prob[i]),(features[i]/(double)10));
			p2 *= p1;
//			System.out.println("p1: "+p1+" p2: "+p2+" prob: "+prob[i]+" fet: "+ 
//				(features[i]/(double)1000));
			i++;
		}
		double pDiC = (double)docPerCat[t]/(double)totDoc;	
//System.out.println("pDiC: " + pDiC + " p2: " + p2); 		
		numerat = pDiC * p2;
//System.out.println("numerat: " + numerat);
		Double num = new Double(numerat);
		probVector.addElement(num);
		denominat += numerat;
	}
	if(denominat==0) denominat=1;		 
	for(int g=0;g<probVector.size();g++){ 
		Double pr=new Double(((((Double)probVector.elementAt(g)).doubleValue())/denominat));
		probVector.setElementAt(pr,g);
	}
	// in questo modo probVector contiene le P(C|Doc) per ogni C
	// ricava la categoria relativa alla probabilit pi alta
	double max= ((Double)probVector.elementAt(0)).doubleValue();
	int ind=0;
	for(int n=1;n<probVector.size();n++){
		if(max<((Double)probVector.elementAt(n)).doubleValue()){
			max=((Double)(probVector.elementAt(n))).doubleValue();
			ind=n;
		}
	}
	if (max <= (tVector[ind]/100))
		result=cgm.categoryName(cgm.totalCategories());
	else
		result=cgm.categoryName(ind);
	System.out.println("Distanza da " + result + " :   " + max);
	return result;
}// end getNaiveBayesianResult	


//----------------------------------------------------------------

/** 
* Restituiosce il risultato della classificazione mediante<br>
* il metodo Nearest Neighbor
*/   

String getNNResult(double features[])throws SQLException, OperationException {
	String result;
	String categoryName;
	int currentCategory = 0;
	double maxDist = 0;
	int maxCategory = 0;
	int idDoc = 0;
	String urlPage;
	double tVector[] = new double[cgm.totalCategories()];
	for (int i = 0;i < cgm.totalCategories();i++) 
		tVector[i] = db.getThreshold(utente, cgm.categoryName(i));
	Enumeration lp = trpm.listPages();
	while (lp.hasMoreElements()) {
		StringTokenizer catPage = new StringTokenizer((String)lp.nextElement());
		idDoc = (new Integer(catPage.nextToken())).intValue();
		categoryName = catPage.nextToken();
		currentCategory = cgm.categoryIndex(categoryName);
		double value[] = db.getRepresentedPage(idDoc, utente);
		double newDist = Vettori.distanza(value,features);
		if(newDist > maxDist) {
			maxDist = newDist;
			maxCategory = currentCategory;
		}
	}
	if( maxDist <= (double) tVector[maxCategory]/100) 
		result = cgm.categoryName(cgm.totalCategories());   
	else
		result = cgm.categoryName(maxCategory); 
	return result;
} // end getNNResult()

//--------------------------------------------------------------------



/** Evaluate the kNN function.
* This version gives enphasis to the rank of a value in the kNNVector.
*/

public String kNNEvalFunction(Vector vect)throws SQLException, OperationException {
	// I'll need a copy of vect later in lessThanThreshold()
	Vector originalVect = new Vector(vect.size());
	for(int i=0; i < vect.size(); i++) {
		kNNVectorElement e = new kNNVectorElement( 
			((kNNVectorElement)(vect.elementAt(i))).getSimilarityValue(),
			((kNNVectorElement)(vect.elementAt(i))).getCategoryCode() );
		originalVect.addElement(e);
	}
	// Emphasize the values of similarity.
	for(int i=0; i < vect.size(); i++) {
		kNNVectorElement elem = (kNNVectorElement)vect.elementAt(i);
		elem.setData( elem.getSimilarityValue()*(vect.size()-i), 
		elem.getCategoryCode() );
		vect.setElementAt( elem , i );
	}
	// Sum all the values of each category and sort them.
	Vector categoriesRank = new Vector(cgm.totalCategories());
	for(int j=0; j < cgm.totalCategories(); j++) {
		kNNVectorElement tempValue = new kNNVectorElement();
		for(int i=0; i < vect.size(); i++) {
			kNNVectorElement elem = (kNNVectorElement)vect.elementAt(i);
			if(elem.getCategoryCode() == j) {
				tempValue.setData( 
				tempValue.getSimilarityValue()+elem.getSimilarityValue(), 
					elem.getCategoryCode() );
			}
		}
		categoriesRank.addElement(tempValue);
	}
	// Sort the final vector
	categoriesRank = quicksort(categoriesRank);
	// When first category is '99' or 
	// similarity is less than threshold then Unknown
	if( (((kNNVectorElement)(categoriesRank.elementAt(0)))
			.getCategoryCode() == 99) | 
		lessThanThreshold( ((kNNVectorElement)(categoriesRank.elementAt(0)))
		.getCategoryCode(), originalVect)  ) 
		return cgm.categoryName(cgm.totalCategories()); 
	else 
		// Return category name for 1st (bigger) value in 'categoriesRank'.
		return cgm.categoryName(
			((kNNVectorElement)(categoriesRank.elementAt(0)))
			.getCategoryCode() );
} // end kNNEvalFunction      

//--------------------------------------------------------------------

/** Tell if the similarity value for 'theCategory' is 
* less than classifier threshold.
*/

boolean lessThanThreshold(int theCategory, Vector original)
		throws SQLException, OperationException {
	double similarity = 0;
	double tVector[] = new double[cgm.totalCategories()];
	for (int i = 0;i < cgm.totalCategories();i++) {
		tVector[i] = db.getThreshold(utente, cgm.categoryName(i));
	}
	// get the first (max) similarity value for 'theCategory'
	for(int j = 0;j < original.size(); j++) {
		kNNVectorElement kve = (kNNVectorElement)original.elementAt(j);
		if( kve.getCategoryCode() == theCategory ) {
			similarity = kve.getSimilarityValue();
			break;   // exit if you got it
		}
	}   
	// is it less than classifier threshold ?
	if (theCategory == 99){
		return true;
	} else
	if ( similarity <= (double)(tVector[theCategory]/100))
		return true;
	else
		return false;
}  // end lessThanThreshold      

//--------------------------------------------------------------------

   /** 'Quicksort' a Vector made of kNNVectorElement. 
	  Recursive version - C.A.R.Hoare
   */
   Vector quicksort(Vector v) {
	  sort(v, 0, v.size()-1);
	  return v;
   }   

//------------------------------------------------------------------
   /** Sorting recursive method.
   */ 
   void sort(Vector v, int s, int d) {
	  int i = s;
	  int j = d;
	  kNNVectorElement temp;
	  kNNVectorElement pivot = (kNNVectorElement)v.elementAt((s+d) / 2);
	  
	  try {
		 do {
			while( ((kNNVectorElement)v.elementAt(i)).greaterThan(pivot) ) i++;
			while( pivot.greaterThan( (kNNVectorElement)v.elementAt(j) ) ) j--;
			if(i <= j) {
			   temp = (kNNVectorElement)v.elementAt(i);
			   v.setElementAt(v.elementAt(j), i);
			   v.setElementAt(temp, j);
			   i++;
			   j--;
			}
		 } while(i <= j); 
		 if(s < j) sort(v, s, j);
		 if(i < d) sort(v, i, d);
	  } catch(ArrayIndexOutOfBoundsException e) {
		 e.printStackTrace();
	  }
   }   

//-----------------------------------------------------

}
