package uniba.core;



/** 
 * Confusion matrix for experimental results of WebClass classification.
 * @author Giorgio Vespucci 1998
 */


 public class ConfusionMatrix {
   int matrix[][];
   CategoriesManager cgm;
	  
   /** Construct the matrix.
   */
   public ConfusionMatrix(CategoriesManager cgmfo) {
	  cgm = cgmfo;
	  matrix = new int[cgm.totalCategories()+1][cgm.totalCategories()+1];
	  zeroIt();
   }   
   /** Get the F-score measure of each category involved in classification.
   */
   public String allFScore() {
	  String outString = "";
	  double sum = 0.0;
	  for(int i=0; i < cgm.totalCategories(); i++) {  
		 outString += "F-score for " + cgm.categoryName(i) + " " +
			   fScore(cgm.categoryName(i)) + "% - \n";
		 sum += fScore(cgm.categoryName(i));
	  }
	  outString += "F-score (Average) " + (sum / cgm.totalCategories()) + "%";
	  return "\n" + outString + "\n";
   }   
   /** Get the precision measure of each category involved in classification.
   */
   public String allPrecision() {
	  String outString = "";
	  double sum = 0.0;
	  for(int i=0; i < cgm.totalCategories(); i++) {  
		 outString += "Precision for " + cgm.categoryName(i) + " " +
			   precision(cgm.categoryName(i)) + "% -\n";
		 sum += precision(cgm.categoryName(i));
	  }
	  outString += "Precision (Average) " + (sum / cgm.totalCategories()) +"%";
	  return "\n" + outString + "\n";
   }   
   /** Get the recall measure of each category involved in classification.
   */
   public String allRecall() {
	  String outString = "";
	  double sum = 0.0;
	  for(int i=0; i < cgm.totalCategories(); i++) {  
		 outString += "Recall for " + cgm.categoryName(i) + " " +
			   recall(cgm.categoryName(i)) + "% - \n";
		 sum += recall(cgm.categoryName(i));
	  }
	  outString += "Recall (Average) " + (sum / cgm.totalCategories()) + "%";
	  return "\n" + outString + "\n";
   }   
   /** Get the F-score measure of classification for given 'category'.
   */
   private double fScore(String category) {
	  if(precision(category)==0 | recall(category)==0)
		 return 0;
	  else
		 return (2*precision(category)*recall(category)) / 
			(precision(category)+recall(category));
   }   
   /** Get the precision measure of classification for given 'category'.
   */
   private double precision(String category) {
	  int sum = 0;
	  for(int i=0; i < cgm.totalCategories()+1; i++) {  
		 sum += matrix[i][cgm.categoryIndex(category)];
	  }
	  if(sum==0)
		 return 0;
	  else
		 return (double)(matrix[cgm.categoryIndex(category)]
							   [cgm.categoryIndex(category)])/ sum * 100;
   }   
   /** Get the recall measure of classification for given 'category'.
   */
   private double recall(String category) {
	  int sum = 0;
	  for(int j=0; j < cgm.totalCategories()+1; j++) {  
		 sum += matrix[cgm.categoryIndex(category)][j];
	  }
	  if(sum==0)
		 return 0;
	  else
		 return (double)(matrix[cgm.categoryIndex(category)]
							   [cgm.categoryIndex(category)])/ sum * 100;
   }   
   /** A string representation of this object.
   */
   public String toString() {
	  String outString = "";
	  String spazi = "                               ";
	  String temp;
	  outString = spazi.substring(0,15);
 	  for(int i=0; i < cgm.totalCategories()+1; i++) {
		 temp = cgm.categoryName(i) + spazi;
		 outString += temp.substring(0,15);
	  } 
	  outString += "\n";	
	  for(int i=0; i < cgm.totalCategories()+1; i++) {
		 temp = cgm.categoryName(i) + spazi;
		 outString += temp.substring(0,15);
		 for(int j=0; j < cgm.totalCategories()+1; j++) {
			temp = matrix[i][j] + spazi;
			outString += temp.substring(0,15);
		 }
		 outString += "\n";
	  }
	  return outString;   
   }   
   /** Write values in matrix at right place.
   */
   public void write(String belongsTo, String classifiedAs) {
	  matrix[cgm.categoryIndex(belongsTo)][cgm.categoryIndex(classifiedAs)]++;
   }   
   /** Reset the matrix; useful later for summarization.
   */
   private void zeroIt() {
	  for(int i=0; i < cgm.totalCategories()+1; i++) {
		 for(int j=0; j < cgm.totalCategories()+1; j++) {
			matrix[i][j] = 0;
		 }
	  }
   }   
} // end class