package uniba.core;

import uniba.databanker.*;
import uniba.gui.*;


import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Cursor;
import java.util.Vector;
import java.util.StringTokenizer;
import java.io.*;

/**
 * Esegue la classificazione di tutte le pagine di testing
 * usando gli alberi di decisione costruite dal modulo
 * OC1. Poich questa classe usa la tecnologia J/Direct della
 * Microsoft, necessita di essere compilata con un compilatore
 * Microsoft per java.
 */
public class DTClassifier extends Thread {

	private String utente = null;
	private Databanker db = null;
      private Dialog parent = null;
	private String FILEPRUNEDTREE = null;
  	private String FILEUNPRUNEDTREE = null;
  	private String FILELOG = null; 		
  	private String FILETESTING = null;
	private String FILECLASS = null;
      private TestingPagesClassifyDialog p = null;
      private CategoriesManager cgm = null;
      private int mode = 0;


	public DTClassifier(Dialog parent, String utente, Databanker db, CategoriesManager cgm, int mode) {
		super();
	      this.utente = utente;
	FILEPRUNEDTREE = ".\\DT\\" + utente + "\\training.dt";
  	FILEUNPRUNEDTREE = ".\\DT\\" + utente + "\\training.unp";
  	FILELOG = ".\\DT\\" + utente + "\\oc1.log"; 		
  	FILETESTING = ".\\DT\\" + utente + "\\testing";
	FILECLASS = ".\\DT\\" + utente + "\\testing.classified";
		this.db = db;
		this.parent = parent;
		this.cgm = cgm;
            this.p = (TestingPagesClassifyDialog)parent;
            this.mode = mode;
	}

	/**
       * Corpo del thread
       */
	public void run() {
		boolean avanti = false;
		try {
			FileReader fr = new FileReader(FILEPRUNEDTREE);
			fr.close();
			avanti = true;
		} catch (Exception fnfe) {
			p.resultText.setText("File of Decision Tree Not Found"); 
			parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			p.resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		if (avanti) {
			int okPage = 0;
            	int totPage = 0;
			ConfusionMatrix confMatr = new ConfusionMatrix(cgm);
            
			//Elimina vecchi file
			java.io.File testing = new java.io.File(FILETESTING);
            	java.io.File oc1log = new java.io.File(FILELOG);
            	java.io.File fileclass = new java.io.File(FILECLASS);
			testing.delete();
			oc1log.delete();
            	fileclass.delete();
		
			try {
				Vector pagine = new Vector();
				pagine = db.getTestingPages(utente);
				totPage = pagine.size();
				Result result = new Result();
				FileWriter fw = new FileWriter(FILETESTING, false);
				for(int i=0; i<totPage; i++) {
                    StringTokenizer st = new StringTokenizer((String)pagine.elementAt(i));
					int idDoc = new Integer(st.nextToken()).intValue();
					String categoria = st.nextToken();
					String urlDoc = st.nextToken();
					
					result.addActual(categoria, urlDoc);

					double rep[] = db.getRepresentedPage(idDoc, utente);
					for (int k=0; k<rep.length; k++) {
						if (k == rep.length - 1)
							fw.write(rep[k] + "");
						else  
							fw.write(rep[k] + ", ");
					}
					fw.write("\n");
					
				}
				fw.close();
			
				//Classifica
				try {
					new MkTree(".\\dt\\" + utente + "\\class.bat", mode);
				} catch (UnsatisfiedLinkError ule) {
					parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					p.resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					new StampaErrore(new Frame(), 
                                      	     "Classify by Decision Tree", 
                                      	     "Impossibile continuare:\n" +
                                               "per utilizzare questa funzione, compilare\n" +
                                               "ed eseguire WebClass con i tools\n" +
                                               "della Microsoft per Java.");
					return;
				}
				boolean fine = false;

				//attende che mktree crei il file di log
				while (!fine) {
					try {
						FileReader fr = new FileReader(FILELOG);
						fine = true;
						fr.close();
					} catch (FileNotFoundException e) {}
				}
				result = getClassificationResult(result);
				String ok;
				for (int j=0; j<result.size(); j++) {
					String assigned = result.getAssigned(j);
					String actual = result.getActual(j);
					String urlDoc = result.getDoc(j);
					if (assigned.equals(actual)) {
						ok = "True";
						okPage++;
					} else 
						ok = "False";
					if (actual.equalsIgnoreCase("Reject"))
						confMatr.write("UNKNOWN", assigned);
					else
						confMatr.write(actual, assigned);

					int h = urlDoc.lastIndexOf("/");
                  		String temp = urlDoc.substring(h+1, urlDoc.length()) + " of '" + actual + 				"'                                " ;
					//p.resultText.setText(p.resultText.getText() +"\n"+ temp.substring(0,30));
				
					temp = "classified as '" + assigned + "'                             ";
					//p.resultText.setText(p.resultText.getText()+temp.substring(0,30)+"'"+ok+"'");

				} 

				double accuracy = (double)(100*okPage)/(double)totPage;
				p.resultText.setText(p.resultText.getText() + "\n\n\n" + confMatr.toString() + "\n");
 		 		String finalReport = "\n\nMicro-average (accuracy): " + accuracy + "%" +
				confMatr.allPrecision()+confMatr.allRecall()+confMatr.allFScore();
		 		p.resultText.setText(p.resultText.getText() + finalReport);

			} catch (Exception e) {
				new StampaErrore(new Frame(),
                                         "Classify by Decision tree",
			      	     	     "Exception:\n" + e.getMessage());
                  	e.printStackTrace();
			} 
			parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			p.resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		} // end if (avanti)
  	      
	} // end run




	/*
	 * Legge dal file .cla le classi attribuite agli esempi
       * di testing da mktree.
	 */
	private Result getClassificationResult(Result r) throws Exception {

		FileReader fr = new FileReader(FILECLASS);
		int c = 0;
            StringBuffer buff = new StringBuffer();
		do {
			c = fr.read();
			buff.append((char)c);
		} while (c != -1);
		String source = buff.toString();

		int inizio = 0;
		for (int i=0; i<r.size(); i++) {
			int j = source.indexOf("\n", inizio);
			int k = j-1;
			char current = source.charAt(k);
			while (current != '\t') { 
				k--;
				current = source.charAt(k);
			}
				
			Integer catNum = new Integer((source.substring(k, j)).trim());
			r.addAssigned(cgm.categoryName(catNum.intValue()-1));
			inizio = j+1;
		}
			
		return r;
  	}   

} // end class
 


			