package uniba.core;

import java.io.*;
import java.util.*;
import java.net.*;
import java.io.*;


import uniba.databanker.*;


/**
 * Questa classe recupera il documento HTML e ne estrae le parole. Le parole estratte
 * hanno le seguenti caratteristiche:<ul><li>lunghezza compresa fra <b>minLengthThreshold</b>
 * e <b>maxLengthThreshold</b></li><li>il primo carattere non  un numero</li><li>
 * non  una stopword</li><li>ridotta alla radice</li></ul>
 */
public class HtmlPage {

  private Databanker db = null;
  private DataInputStream inStream;
  private String background = null;
  private Ancore ancore = new Ancore();
  private Vector linkIMG = new Vector();
  private URL theUrl = null;
  private int minLengthThreshold;
  private int maxLengthThreshold;
  private String proxyServer;
  private String socksServer;	
  private Vector stopwords = new Vector();
  private URL       url;      // URL della pagina HTML (http://...  oppure file://...).
  private String    source;   // Sorgente HTML.
  private String    title;    // Titolo della pagina.
  private Vector    words;    // Vettore delle parole (eventualmente HTML-tagged).
					  // che costituiscono la pagina. 

					  
  private int       stato;    // >0 se la pagina non  aperta correttamente; 
  
    
  /**
   * Recupera il documento HTML da disco o da Internet, ne estrae le parole,
   * il titolo e le frasi.
   */	   
  public HtmlPage(String urlName, Vector conf, Vector stopword, Databanker db)
     throws MalformedURLException, FileNotFoundException, UnknownHostException,
     IOException, Exception  {

      String protocol;
	String fileName;
	protocol = urlName.substring(0,4);
      theUrl = new URL(urlName);
	String line;
	GetURL urlSite;
	String PROXY = "";
	String SOCKS = "";
	
	minLengthThreshold = ((Integer)conf.elementAt(0)).intValue();
      maxLengthThreshold = ((Integer)conf.elementAt(1)).intValue();
    
      stopwords = stopword;
      proxyServer = (String)conf.elementAt(2);
      socksServer = (String)conf.elementAt(3);
      this.db = db;
      
	
	if (protocol.equalsIgnoreCase("HTTP")) {
       
	  /* Apre la connessione http con la URL indicata come parametro. */

          if(! (proxyServer.equals(PROXY)) ) {
			urlSite = new GetURL(urlName, proxyServer);
          } else if (!(socksServer.equals(SOCKS)) ) {
			urlSite = new GetURL(urlName, SOCKS , socksServer);
          } else {
			urlSite = new GetURL(urlName);
          }


	  BufferedReader received = new BufferedReader(new InputStreamReader(urlSite.connect())); 
	  int charcode, second, third, fourth; 
	  boolean found=false;
	  line="";
	  source="";
	  charcode = received.read();

	  //Take off the html received streem the HTTP header
	  while (received.ready()&&!found) {
	   if (charcode!=13) charcode = received.read();
			else {
				  second = received.read();
				  third = received.read();
				  fourth = received.read();
				  if ((second==10)&&(third==13)&&(fourth==10)) found=true;
					else charcode = received.read();
				 }
	  }  
	 //Load the HTML source
	 while ((line = received.readLine())!=null) {
	   source=source+line+"\n";
	  }  
	  urlSite.finalize();
	  //if (source.length()>32768) {source=source.substring(0,32768);}
	  //           Viene creato un nuovo InputStream (in memoria) con il contenuto della
	  //           stringa source, resettabile e rileggibile un qualsivoglia numero di volte.
		StringBufferInputStream SBinStream = new StringBufferInputStream(source);
		words=extractWords(SBinStream);
		findTitle();
            stato=0;


	}
	else if (protocol.equalsIgnoreCase("FILE")) {
	  fileName=urlName.substring(7, urlName.length());  
	  
          
		/* Dal file aperto riceve come flusso di input il sorgente */
		/* L'InputStream viene letto (quindi consumato) e copiato 
		   nella stringa source; */
		inStream = new DataInputStream(new FileInputStream(new java.io.File(fileName)));  
		source=extractSource(inStream);
		/* se il sorgente  piu lungo di 32Kb viene troncato */
		//if (source.length()>32768) {source=source.substring(0,32768);}
		/* Viene creato un nuovo InputStream (in memoria) con il contenuto della
		   stringa source, resettabile e rileggibile un qualsivoglia numero di volte. */
		StringBufferInputStream SBinStream = new StringBufferInputStream(source);
		words=extractWords(SBinStream);
		findTitle();
            stato=0;
	}
	else {
	 
	}

  }

  
  
  /**
   * Trasforma il flusso di input contenente il sorgente HTML in una stringa. 
   */
  private String extractSource (InputStream inStream) throws Exception {
	String source="";
        
	  DataInputStream dataInStream= new DataInputStream(inStream);
	  String line;
	  while ((line=dataInStream.readLine())!=null) {
		source=source+line+"\n";
	  }  
	return source;
  }  


  /**
   * Estrae dal sorgente HTML tutte le parole che lo compongono e le restituisce
   * sotto forma di vettore di "HtmlWord".
   */
  private Vector extractWords (InputStream HtmlSource) throws IOException {


	boolean nowTitle =false;
	int     nowHeader = 0; 
	boolean nowBold = false;
	boolean nowItalic = false;
	boolean nowUnderline = false;
	boolean nowStrong = false;
	boolean nowEmphasize = false;
	boolean nowCite = false;
	boolean nowBlockQuote = false;
	boolean nowLink = false;
	boolean nowListItem = false;
      boolean indirizzoValido = false;
	HtmlTokenizer HT = new HtmlTokenizer (HtmlSource, theUrl);
	StemmerPorter ST = new StemmerPorter();
	Vector wordList = new Vector();
	boolean eof = false;
	do {
	  int token = HT.nextToken();
	  switch (token) {
		case -1: { eof = true; }
		break;
		case 0: { }
		break;
		case 1: {   // Tag
		  if (HT.getTag().getName().equalsIgnoreCase("TITLE")) {
			nowTitle=HT.getTag().isStart(); }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("B")) { 
			nowBold=HT.getTag().isStart(); }
		  else 
			if (HT.getTag().getName().equalsIgnoreCase("I")) { 
			nowItalic=HT.getTag().isStart(); }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("U")) { 
			nowUnderline=HT.getTag().isStart(); }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("H1")) { 
			if (HT.getTag().isStart()) nowHeader=1; else nowHeader=0; }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("H2")) { 
			if (HT.getTag().isStart()) nowHeader=2; else nowHeader=0; } 
		  else
		  if (HT.getTag().getName().equalsIgnoreCase("H3")) { 
			if (HT.getTag().isStart()) nowHeader=3; else nowHeader=0; } 
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("H4")) { 
			if (HT.getTag().isStart()) nowHeader=4; else nowHeader=0; }
		  else
		  if (HT.getTag().getName().equalsIgnoreCase("STRONG")) { 
			nowStrong=HT.getTag().isStart(); }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("EM")) { 
		   nowEmphasize=HT.getTag().isStart(); }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("CITE")) { 
			nowCite=HT.getTag().isStart(); }
		  else 
		  if (HT.getTag().getName().equalsIgnoreCase("BLOCKQUOTE")) { 
			nowBlockQuote=HT.getTag().isStart(); }
		  else  
		  if (HT.getTag().getName().equalsIgnoreCase("A")) { 
			nowLink=HT.getTag().isStart();
                  if (HT.getTag().getUrl() != null) {
			    indirizzoValido = true;
                      ancore.newLink(HT.getTag().getUrl()); 
                  } else
                      indirizzoValido = false;
              }
              else
		  if (HT.getTag().getName().equalsIgnoreCase("IMG")) { 
			if (HT.getTag().getUrl() != null) 
			    if (nowLink & indirizzoValido) {
					ancore.addElement(HT.getTag().getUrl());	
			    } else if (!linkIMG.contains(HT.getTag().getUrl().toLowerCase()))  	
                      		linkIMG.addElement(HT.getTag().getUrl().toLowerCase());
		  }   
		  else
		  if (HT.getTag().getName().equalsIgnoreCase("BODY")) { 
			if ( HT.getTag().isStart() )
			     background = HT.getTag().getUrl(); 
              }
		   		
		}
		break;
		case 2: { // Numero
		}
		break;
		case 3: { // Carattere 
		}
		break;    
		case 4: { // Special Char 
		}
		break;
		case 5: {  // Parola
		  if (  (HT.getWord().length() > minLengthThreshold ) && 
                    (HT.getWord().length() < maxLengthThreshold ) &&
                    !((HT.getWord().charAt(0)<58) && (HT.getWord().charAt(0)>=48)) &&
			  (!isStopword(HT.getWord().toLowerCase())) 
                  ) {	
			ST.toChar(HT.getWord());
			ST.stem(0);
                  HtmlWord word = new HtmlWord(ST.toString());
			if (word.getText().length() > minLengthThreshold) {
				if (nowHeader>0)   word.addAttribute("H"+nowHeader+" ");
				if (nowTitle)      word.addAttribute("TI ");
				if (nowBold)       word.addAttribute("BO ");
				if (nowItalic)     word.addAttribute("IT ");
				if (nowUnderline)  word.addAttribute("UN ");
				if (nowStrong)     word.addAttribute("ST ");
				if (nowEmphasize)  word.addAttribute("EM ");
				if (nowCite)       word.addAttribute("CI ");
				if (nowBlockQuote) word.addAttribute("QT ");
				if (nowLink) { 
					word.addAttribute("LK ");
					if (indirizzoValido) {	
                        		ancore.addElement(word.getText()); 
					}
				}
				if (nowListItem)   word.addAttribute("LI ");
				wordList = aggiungi(wordList, word);
			}
		  } 
		}
		break;
		case 6: { // Quotation 
			//System.out.println("QUOTATION " + HT.getQuotation());
		}
		break;
	  } 
	} while (!eof); 
	return wordList;
  }  

  

  /**
   * Calcola l'occorrenza di un token
   */
  private Vector aggiungi(Vector parole, HtmlWord word) {
	String testo = word.getText().toLowerCase();
      boolean trovato = false;
      int i=0;
	while ( (!trovato) && (i<parole.size()) ) {
		HtmlWord elem = (HtmlWord)parole.elementAt(i);
		String testo2 = elem.getText().toLowerCase();
		if (testo2.equals(testo)) {
                  if (elem.getAttributes() != null) {
                       StringTokenizer st = new StringTokenizer(elem.getAttributes());
                       while (st.hasMoreTokens())  
                  		word.addAttribute(st.nextToken()+" ");
                  } 
			word.setOcc(elem.getOcc()+1);
                  word.setText( word.getText().toLowerCase() ); 
			parole.setElementAt(word, i);
                  trovato = true;
		} else
			i++;
	}
	if (!trovato) {
		word.setOcc(1);
            word.setText( word.getText().toLowerCase() );
		parole.addElement(word);
	}
	return parole;
   }		
			 	



  /**
   * Ricava il titolo della pagina HTML cercando nel vettore di parole tutte
   * quelle "taggate" con l'attributo "TI" (cio "title") 
   */
  private void findTitle() {
	HtmlWord currentWord;
	int i;
	int pos;
	String currentAtt;      
	title="";
	for (i=0; i<words.size(); i++) {
	  currentWord = (HtmlWord) words.elementAt(i);
	  currentAtt = currentWord.getAttributes();
	  if (currentAtt != null) {
		pos=currentAtt.indexOf("TI"); 
		if (pos!=-1) {
		  title=title+currentWord.getText()+" ";
		}  
	  }
	}
	if (title.equals(""))
		title = "No Title";
	title = title.trim();
  }

  
  /**
   * Restituisce il numero totale di parole estratte.
   */
  public int getNumWords() {
      double tot = 0;
	for (int i=0; i<words.size(); i++) {
		HtmlWord w = (HtmlWord)words.elementAt(i);
            tot = tot + w.getOcc();
      }
	return new Double(tot).intValue();
  }


  /**
   * Restituisce il sorgente HTML in formato String
   */
  public String getSource() {
	return source;
  }  

  

  /**
   * Restituisce il titolo della pagina HTML
   */
  public String getTitle() {
	return title;
  }  

  /**
   * Restituisce l'URL della pagina
   */
  public URL getUrl() {
	return theUrl;
  }  

  /**
   * Restituisce l'n-esima parola estratta.
   */ 
  public HtmlWord getWord(int n) {
	if ((n>0) && (n<words.size())) 
	  return (HtmlWord)words.elementAt(n);
	else 
	  return null;
  }

  /**
   * Restituisce tutte le parole estratte. Si noti che gli elementi della lista
   * restituita sono di tipo <b>HtmlWord</b>.
   */
  public Enumeration getWords() throws Exception {
	Vector source = calculateTF(words);
      return source.elements();
  }  

  private boolean isStopword(String word) {
	int i = 0;
	while (i<stopwords.size()) {
		String element = (String)stopwords.elementAt(i);
		if ( word.equals(element) )
			return true;
            else
               i++;
      } 
      return false;
  }	

  /**
   * Restituisce le ancore con relative immagini e testo associati.
   */		
  public Ancore getLinkA() {
	return ancore;
  }

  
  /**
   * Restituisce la lista dei link. In questo caso viene considerato il tag
   * <b>IMG</b>. La lista non contiene duplicati.
   * Ogni elemento della lista  di tipo <b>String</b>.
   */
  public Enumeration getLinkIMG() {

      return linkIMG.elements();
  }

  /**
   * Restituisce l'immagine di background. Se l'immagine non esiste allora 
   * restituisce <i>null</i>.
   */	
  public String getBackgroundImage() {
	return background;
  }

  /**
   * Restituisce il sorgente HTML sotto forma di <b>DataInputStream</b>.
   */
   public DataInputStream getDataInputStream() {
		return inStream;
   }

   
  

  /**
   * Calcola la TF partendo dalle occorrenze memorizzate
   */
  private Vector calculateTF(Vector source) throws Exception {
  		Hashtable ht = db.getEnphasis();
  		//--------------------------------------------------------------
  		Enumeration chiavi = ht.keys();
  		while (chiavi.hasMoreElements()) {
  			String c = (String)chiavi.nextElement();
  			System.out.println(c + "   " +  
  			                   ((Integer)ht.get(c)).intValue() );
  		}
  		//--------------------------------------------------------------
		double tot = 0;
            for (int i=0; i<source.size(); i++) {
                  HtmlWord ww = (HtmlWord)source.elementAt(i);
			tot = tot + ww.getOcc();
            } 
		for (int i=0; i<source.size(); i++) {
                  // Calcola la TF
			HtmlWord w = (HtmlWord)source.elementAt(i); 
			double occ = w.getOcc();
			double TF = (1000*occ/tot);
                  			
		      //calcola l'enfasi additiva e moltiplicativa
                  String att = w.getAttributes();
                  double peso = 0;
                  if (att != null) {
				StringTokenizer stk = new StringTokenizer(att);
                        while (stk.hasMoreTokens()) {
					String token = stk.nextToken();
					if (token.equalsIgnoreCase("LK"))
						peso = peso + ((Integer)ht.get("LK")).intValue();
					else if (token.equalsIgnoreCase("QT"))
						peso = peso + ((Integer)ht.get("QT")).intValue();
                              else if (token.equalsIgnoreCase("EM"))
						peso = peso + ((Integer)ht.get("EM")).intValue();
					else if (token.equalsIgnoreCase("ST"))
						peso = peso + ((Integer)ht.get("ST")).intValue();
					else if (token.equalsIgnoreCase("H4"))
						peso = peso + ((Integer)ht.get("H4")).intValue();
					else if (token.equalsIgnoreCase("H3"))
						peso = peso + ((Integer)ht.get("H3")).intValue();
					else if (token.equalsIgnoreCase("H2"))
						peso = peso + ((Integer)ht.get("H2")).intValue();
					else if (token.equalsIgnoreCase("H1"))
						peso = peso + ((Integer)ht.get("H1")).intValue();
					else if (token.equalsIgnoreCase("TI"))
						peso = peso + ((Integer)ht.get("TI")).intValue();
				}
			}
                  w.setTF(TF);
                  w.setHtmlWeight( peso );
                  w.setEnfasiAdditiva( TF + peso );
			w.setEnfasiMoltiplicativa( TF * (peso + 1) );  
                  source.setElementAt(w, i);
				
		}
		return source;
  } 	
     
  	

 

   
        
}
