package uniba.core;

import java.io.*;
import java.net.*;
import java.util.*;




/**
 * HtmlTokenizer implementa una classe per estrarre i token di un sorgente HTML:
 * parole, numeri, punteggiatura, caratteri speciali, e tags HTML. Viene associato
 * un tipo (int) ad ogni token: <ul><li>-1 fine flusso</li>
 * <li>0 errore</li> <li>1 Tag HTML</li><li>2 numero</li> <li>3 carattere</li>
 * <li>4 "&code;"</li><li>5 parola</li><li>6 stringa fra virgolette</li></ul>
 */
public class HtmlTokenizer extends StreamTokenizer {

 // Tabella dei Tags del linguaggio HTML. 
  private static String TagKeyWord [] =
  {"HTML","BODY","HEAD","TITLE",
   "LINK","IMG","A",
   "P","BR","HR",
   "H1","H2","H3","H4","H5","H6",
   "OL","UL","LI","DL","DT","DD","MENU","DIR",
   "B","I","U",
   "EM","STRONG","CITE","VAR","TT",
   "BLOCKQUOTE", "OBJECT", "SCRIPT", "INPUT",
   "AREA", "BASE", "APPLET", "FORM"
  };
  
 // Tabella dei caratteri speciali HTML, identificati da &xxx, dove xxx  una
 // stringa di identificazione. 
  private static String SpecialCharKeyWord [] =
   {"amp","gt","lt","nbsp"
   };
   
 // Caratteri di controllo del HTML.  
  private static char OrdinaryChar [] =
   { '/', '<', '>', '!', '&', ';' }; 

 private int        HtmlType; /* -1 fine flusso 
						  0 errore 
						  1 tag html 
						  2 numero
						  3 carattere
						  4 "&code;" 
						  5 parola
						  6 stringa
					   */
 private HtmlTag currentTag;
 private double  currentNumber;                     
 private char    currentChar; 
 private String  currentSpecialChar;
 private String  currentWord;
 private String  currentQuotation;
 private URL indirizzo = null;
 private URL base = null; 
	
 /**
  * Il Costruttore riceve in ingresso un InputStream contenente il sorgente HTML. 
  */
 public HtmlTokenizer (InputStream HtmlSource, URL ind) throws
   MalformedURLException {
  super (HtmlSource);
  indirizzo = ind;
  String temp=null;
  if (indirizzo.getPort() != -1)
       temp = indirizzo.getProtocol() + "://" + indirizzo.getHost() +
                     indirizzo.getPort() + getPath();
  else
	 temp = indirizzo.getProtocol() + "://" + indirizzo.getHost() +
                     getPath();

 
  base = new URL(temp);
          
  whitespaceChars(0,255);
  ordinaryChars('a','z');
  ordinaryChars('A','Z');
  ordinaryChars('0','9');
  int i=0;
  while (i<OrdinaryChar.length) {ordinaryChar(OrdinaryChar[i]);i++;} 
  wordChars('a','z');
  wordChars('A','Z');
  wordChars('0','9');
  quoteChar('"');
  // parseNumbers();
 } 

       
            

 /**
  * Se dopo aver ottenuto da <code>nextToken()</code> il tipo <b>3</b> si vuole 
  * il carattere stesso, allora si invoca <code>getChar()</code>.
  */
 public char getChar() {
  if (HtmlType==3) return currentChar; else return 0;
 } 

 /**
  * Se dopo aver ottenuto da <code>nextToken()</code> il tipo <b>2</b> si vuole 
  * il numero stesso, allora si invoca <code>getNumber()</code>.
  */
 public double getNumber() {
  if (HtmlType==2) return currentNumber; else return 0;
 } 

 /**
  * Se dopo aver ottenuto da <code>nextToken()</code> il tipo <b>6</b> si vuole 
  * la stringa stessa, allora si invoca <code>getQuotation()</code>.
  */
 public String getQuotation() {
  if (HtmlType==6) return currentQuotation; else return null;
 } 

 /**
  * Se dopo aver ottenuto da <code>nextToken()</code> il tipo <b>4</b> si vuole 
  * il carattere speciale stesso, allora si invoca <code>getSpecialChar()</code>.
  */
 public String getSpecialChar() {
  if (HtmlType==4) return currentSpecialChar; else return null;
 } 

 /*
  * Ricava il path del documento di input.
  */
  private String getPath() {
  	String file = indirizzo.getFile();
	int i = file.length();
	while (file.charAt(i-1) != '/')
          i = i-1;
      return file.substring(0, i);
  }
  
 /**
  * Se dopo aver ottenuto da <code>nextToken()</code> il tipo <b>1</b> si vuole 
  * il tag stesso, allora si invoca <code>getTag()</code>.
  */
 public HtmlTag getTag() {
  if (HtmlType==1) return currentTag; else return null;
 } 

 /**
  * Se dopo aver ottenuto da <code>nextToken()</code> il tipo <b>5</b> si vuole 
  * la parola stessa, allora si invoca <code>getWord()</code>.
  */
 public String getWord() {
  if (HtmlType==5) return currentWord; else return null;
 } 


 private boolean isSpecialChar(String s) {
  boolean found = false;
  int i=0;
  while ((i<SpecialCharKeyWord.length) && (!found)) {
   found=s.equalsIgnoreCase(SpecialCharKeyWord[i]);
   i++;
  }
  return found;
 } 


 private boolean isTag(String s) {
  boolean found = false;
  int i=0;
  while ((i<TagKeyWord.length) && (!found)) {
   found=s.equalsIgnoreCase(TagKeyWord[i]);
   i++;
  }
  return found;
 } 

 /** 
  * Ogni volta che il metodo "nextToken" viene chiamato restituisce il tipo (int)
  * del successivo token HTML, disponibile sull'InputStream.
  */
 public int nextToken() throws IOException {
  boolean StartTag;
  int t1=super.nextToken();
  HtmlType=0;
  switch (t1) {
   // Fine Flusso.
   case -1: { HtmlType=-1; }
   break;
   // Tag...
   case '<': {
	StartTag=true;
	int t2;
	t2=super.nextToken();
	if (t2=='!') { // ...di commento.
	 int t3;
	 do { t3=super.nextToken(); } while (!((t3=='>') || (t3==-1)));  
	}
	else { 
	 if (t2=='/') { // ...di chiusura (cio </"nometag">.
	  StartTag=false; 
	  t2=super.nextToken();
	 } 
	 if ((t2==-3) && isTag(sval)) { // ...noto del linguaggio HTML.
	  HtmlType=1;
	  currentTag=new HtmlTag(sval, StartTag);
	 }
	 int t3;
	 do { 
	  	t3=super.nextToken();
		if (File.estensioneNota(sval)) {
            	try { 					// trova url assoluti
				URL theUrl = new URL(sval);
				currentTag.setUrl(sval);
			} catch (MalformedURLException e) {
				try { 				// trova url relativi
                 			URL host = new URL(base, sval);
                 			currentTag.setUrl(sval); 
            		} catch (MalformedURLException me){}
            	}
		}
	 } while (!((t3=='>') || (t3==-1)));  
	}
   }  
   break;
   // Numero.
   case -2: {
	HtmlType=2;
	currentNumber=nval;
	}
   break;
   // Carattere
   case ',':
   case '.':
   case ';': { 
	HtmlType=3; 
	currentChar=((char) t1);
   }
   break;
   // Carattere speciale.
   case '&':{
	HtmlType=0;   
	int t2;
	t2=super.nextToken();
	if ((t2==-3) && isSpecialChar(sval)) {
	 currentSpecialChar=sval;
	 int t3;
	 t3=super.nextToken();
	 if (t3==';') HtmlType=4; 
	} 
   }
   break;
   // Parola.
   case -3: {
	HtmlType=5;
	currentWord=sval;
   }
   break;
   //Quotation (cio stringa tra virgolette).
   case '"':{
	HtmlType=6;
	currentQuotation=sval;
   }
	  
   // Errore.
   default : { HtmlType=0; } 

  }  
  return HtmlType;
 } 
      
}
