package uniba.core;

import java.io.*;
import java.util.*;
import java.net.*;
import java.io.*;

/**
 * Tokenizza la query posta dall'utente
 */
public class QueryTokenizer {

	Vector ris = new Vector();
	Vector conf = new Vector();
      Vector stopwords = null; 

	public QueryTokenizer(String query, Vector conf, Vector stopword) throws IOException {

            this.stopwords = stopword;
		this.conf = conf;
            int minLengthThreshold = ((Integer)conf.elementAt(0)).intValue();
      	int maxLengthThreshold = ((Integer)conf.elementAt(1)).intValue();

		StreamTokenizer sttk = new StreamTokenizer( new StringReader(query) );
		sttk.ordinaryChar('/');
		sttk.ordinaryChar('\'');
      	sttk.ordinaryChar('.'); 
		int tipo = 0;

		StemmerPorter ST = new StemmerPorter();
	      while ((tipo = sttk.nextToken()) != sttk.TT_EOF) {
			if (tipo == sttk.TT_WORD) {
				if (  (sttk.sval.length() > minLengthThreshold ) && 
                              (sttk.sval.length() < maxLengthThreshold ) &&
                              !((sttk.sval.charAt(0)<58) && (sttk.sval.charAt(0)>=48)) &&
			            (!isStopword(sttk.sval.toLowerCase())) 
                            ) {	

				        ST.toChar(sttk.sval);
				        ST.stem(0);
                  	        HtmlWord word = new HtmlWord(ST.toString());
					  if (word.getText().length() > minLengthThreshold) 
						ris = aggiungi(ris, word);
				}
            	}
			else if (tipo == 34) {
				StringTokenizer sk = new StringTokenizer(sttk.sval);
				while (sk.hasMoreTokens()) {
                        	String s = sk.nextToken();
                              if (  (s.length() > minLengthThreshold ) && 
                                    (s.length() < maxLengthThreshold ) &&
                                    !((s.charAt(0)<58) && (s.charAt(0)>=48)) &&
			                  (!isStopword(s.toLowerCase())) 
                                  ) {	

                        	          ST.toChar( s );
			      	          ST.stem(0);
                        	          HtmlWord w = new HtmlWord(ST.toString()); 
					          ris = aggiungi(ris, w);
					}
                  	}
			}
		}
	}


  /**
   * Calcola l'occorrenza di un token
   */
  private Vector aggiungi(Vector parole, HtmlWord word) {
	String testo = word.getText().toLowerCase();
      boolean trovato = false;
      int i=0;
	while ( (!trovato) && (i<parole.size()) ) {
		HtmlWord elem = (HtmlWord)parole.elementAt(i);
		String testo2 = elem.getText().toLowerCase();
		if (testo2.equals(testo)) {
                  if (elem.getAttributes() != null) {
                       StringTokenizer st = new StringTokenizer(elem.getAttributes());
                       while (st.hasMoreTokens())  
                  		word.addAttribute(st.nextToken()+" ");
                  } 
			word.setOcc(elem.getOcc()+1);
                  word.setText( word.getText().toLowerCase() ); 
			parole.setElementAt(word, i);
                  trovato = true;
		} else
			i++;
	}
	if (!trovato) {
		word.setOcc(1);
            word.setText( word.getText().toLowerCase() );
		parole.addElement(word);
	}
	return parole;
   }	

  /**
   * Calcola la TF partendo dalle occorrenze memorizzate
   */
  private Vector calculateTF(Vector source) {
		double tot = source.size();
		for (int i=0; i<source.size(); i++) {
                  // Calcola la TF
			HtmlWord w = (HtmlWord)source.elementAt(i); 
			double occ = w.getOcc();
			double TF = (1000*occ/tot);
                  			
		      //calcola l'enfasi additiva e moltiplicativa
                  String att = w.getAttributes();
                  double peso = 0;
                  if (att != null) {
				StringTokenizer stk = new StringTokenizer(att);
                        while (stk.hasMoreTokens()) {
					String token = stk.nextToken();
					if (token.equalsIgnoreCase("LK"))
						peso = peso + 1;
					else if (token.equalsIgnoreCase("QT"))
						peso = peso + 2;
                              else if (token.equalsIgnoreCase("EM"))
						peso = peso + 3;
					else if (token.equalsIgnoreCase("ST"))
						peso = peso + 4;
					else if (token.equalsIgnoreCase("H4"))
						peso = peso + 5;
					else if (token.equalsIgnoreCase("H3"))
						peso = peso + 6;
					else if (token.equalsIgnoreCase("H2"))
						peso = peso + 7;
					else if (token.equalsIgnoreCase("H1"))
						peso = peso + 8;
					else if (token.equalsIgnoreCase("TI"))
						peso = peso + 10;
				}
			}
                  w.setTF(TF);
                  w.setEnfasiAdditiva( TF + peso );
			w.setEnfasiMoltiplicativa( TF * (peso + 1) );  
                  source.setElementAt(w, i);
				
		}
		return source;
  } 

  /**
   * Restituisce la query tokenizzata; gli elementi
   * del vettore sono di tipo <b>HtmlWord</b>.
   */
  public Vector getTokenQuery() {
	return calculateTF(ris);
  }

  /**
   * Restituisce il numero totale di parole estratte.
   */
  public int getNumWords() {
      double tot = 0;
	for (int i=0; i<ris.size(); i++) {
		HtmlWord w = (HtmlWord)ris.elementAt(i);
            tot = tot + w.getOcc();
      }
	return new Double(tot).intValue();
  }
	

 private boolean isStopword(String word) {
	int i = 0;
	while (i<stopwords.size()) {
		String element = (String)stopwords.elementAt(i);
		if ( word.equals(element) )
			return true;
            else
               i++;
      } 
      return false;
  }	
	
}