package uniba.core;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.*;
import java.sql.*;

import uniba.databanker.*;


/** 
 * Questa classe permette di rappresentare un documento non ancora
 * memorizzato nel database. Ci  utile in fase di classificazione 
 * di un nuovo documento.
 */
public class RepresentManager {
  
  private FeaturesManager ftm ;
  private Vector features;
  private Vector occurredFeatures;
  private int rappresentationType;
  private double freq[];
  private double htmlWeight[];
  private double totWeight[];
  private String tags[];
  private Databanker db = null;
  private String utente = null;

  // E se il numero delle features  >256 ?
  // Dovrebbe essere int[ftm.getTotalFeatures()] ma qui non  visibile...(GV)
  
	  
  public RepresentManager(FeaturesManager ftm, int rappType, Databanker db, String utente) {
	  this.ftm  = ftm;
	  this.db = db;
	  this.utente = utente;
	  rappresentationType = rappType;	
  }   

  /**
   * Questo metodo si deve invocare quando cambia il tipo di
   * rappresentazione.
   */
   public void updateRappType(int type) {
       rappresentationType = type;	    
       System.out.println("Tipo di rappresentazione aggiornata.");
   }
  
  /**
   * Restituisce la rappresentazione vettoriale della pagina corrente.
   */		
  public synchronized double[] getCurrentRepresentation() {
	return totWeight;
  }  

  /**
   * Gli elementi del vettore restituito sono del seguente tipo:<br>
   * <i>feature</i> + <i>frequenza</i> + <i>peso tag html</i> + 
   * <i>peso totale</i> + <i>contesto</i>. 
   */
  public synchronized Enumeration listFeatures() {
	 return occurredFeatures.elements();
  }  

   /** 
    *	Represent a page according to features vector.
    * @author Giulio DeLuise 1997
    */
  public synchronized void represent(Enumeration wordInPage, int totWord) {
      String word = "";
	String spazi="                              ";
	String itemOccurredFeature;
      occurredFeatures = new Vector();
	HtmlWord hword;

	/*
       * Elenca le parole della pagina da rappresentare e controlla quali di esse
	 *  sono presenti tra le features 
       */
      
	features = ftm.getFeatures();
	totWeight  = new double[ftm.getTotalFeatures()];
      htmlWeight  = new double[ftm.getTotalFeatures()];
      freq = new double[ftm.getTotalFeatures()];
      tags = new String[ftm.getTotalFeatures()]; 

	for (int j=0; j<ftm.getTotalFeatures(); j++) {
            freq[j] = 0;
	  	totWeight[j]  = 0;
		tags[j] = "";
		htmlWeight[j] = 0;
	}

	Enumeration e = wordInPage;
	while (e.hasMoreElements()) {
		hword = (HtmlWord) e.nextElement();
            int j = contains(features, hword.getText());
            if ( j != -1 ) {
			freq[j] = hword.getTF();
                  htmlWeight[j] = hword.getHtmlWeight();
		      if (rappresentationType == 0)
		 		totWeight[j] = hword.getTF();
			else if(rappresentationType == 1) 
				totWeight[j] = hword.getEnfasiAdditiva();
			else if(rappresentationType == 2) 
				totWeight[j] = hword.getEnfasiMoltiplicativa();
		 	if (hword.getAttributes() != null) 
		   		tags[j] = hword.getAttributes();
	  	}
	}
	/*
       * prepara la stringa per il report
       */
	for (int i=0; i<ftm.getTotalFeatures(); i++) {
            Feature f =  (Feature)features.elementAt(i);
		word = f.getFeature();
		Double Freq = new Double(freq[i]);
		Double HtmlWeight = new Double(htmlWeight[i]);
		Double TotWeight =  new Double(totWeight[i]);
		itemOccurredFeature = word +
						   spazi.substring(1,22-word.length())+
						   spazi.substring(1,22-Freq.toString().length())+
						   Freq.toString()+"              "+
						   spazi.substring(1,22-HtmlWeight.toString().length())+
						   HtmlWeight.toString()+"              "+
						   spazi.substring(1,22-TotWeight.toString().length())+
						   TotWeight.toString()+"               "+
						   tags[i];
	   occurredFeatures.addElement(itemOccurredFeature);
	}
  }



  




  private int contains(Vector v, String s) {
	int i = 0;
      boolean presente = false;
	while (i < v.size() && !presente) {
            Feature f = (Feature)v.elementAt(i);
		String fea = f.getFeature();
		if (fea.equalsIgnoreCase(s)) 
			presente = true;
		else
            	i++;
	}
	if (presente)
		return i;
	else
		return -1;
  }	
  
  


} // end class