package uniba.core;

import java.util.*;
import java.io.*;
import uniba.databanker.*;
import java.sql.SQLException;

/**
 * Mantiene informazioni sulle pagine di training.
 */
public class TrainingPagesManager {
  
  private CategoriesManager cgm = null;
  private Vector trainingPages;
  private int count = 0;
  private String utente = null;
  private Databanker db = null;
  
  public TrainingPagesManager(Databanker db, String utente, CategoriesManager cgm)
    throws SQLException{
        this.cgm = cgm;
        this.utente = utente;
        this.db = db; 
	  trainingPages = new Vector();
        trainingPages = db.getTrainingPages(utente);
	  count = trainingPages.size();	
  }

  /**
   * Quando una qualsiasi operazione modifica l'insieme delle pagine di training
   * si deve chiamare questa procedura per notificare all'oggetto <b>TrainingPagesManager</b>
   * tale cambiamento.
   */
  public void updateTrainingPages() throws SQLException {
      trainingPages.removeAllElements();
	trainingPages = db.getTrainingPages(utente);
      count = trainingPages.size();
  }

 
  /**
   * Restituisce il numero di pagine di training.
   */
  public int getNumber() {
	  return count;
  }

  /**
   * Per ogni categoria, restituisce il numero di pagine di training appartenenti
   * alla categoria data. Il vettore restituito  in corrispondenza con il vettore 
   * di categorie prodotto da <b>CategoriesManager</b>.
   */
  public int[] getTotPageInCategory() {
    Enumeration epag;
	int currentCategory;
	String extractedCategoryName;
	int totPageInCategory[] = new int[cgm.totalCategories()];
	for (int i=0;i<cgm.totalCategories();i++){totPageInCategory[i] = 0; }
	  epag = listPages();
	  while (epag.hasMoreElements()){
	    String Page = (String) epag.nextElement();
	    StringTokenizer st = new StringTokenizer(Page);
		String adress = st.nextToken();
            extractedCategoryName  = st.nextToken();
	    currentCategory = cgm.categoryIndex(extractedCategoryName);
        totPageInCategory[currentCategory]++;
	 }
     return totPageInCategory;
  }

  /**
   * Restituisce la lista della pagine di training con relativa categoria.<br>
   * Gli elementi del vettore restituito sono del seguente tipo:<br>
   * <code>id pagina</code> + <code>categoria pagina</code> + <code>url pagina</code>.
   */
  public Enumeration listPages() {
	return trainingPages.elements();
  }

}
