package uniba.databanker;

import java.sql.*;
import java.util.*;

import uniba.core.*;
import uniba.gui.*;


/**
 * Contiene tutti i metodi per l'accesso alle tavole del database.
 * Si noti che le feature e tutti i vettori di pesi ad esse collegate
 * sono restituite in ordine alfabetico. Questo  necessario per fare
 * in modo che ogni feature (coordinata) mantenga sempre la stessa
 * posizione nello spazio vettoriale. Se non si rispetta tale ordine
 * le coordinate dello spazio vettoriale cambiano rendendo vani i
 * risultati di classificazione e di interrogazione.
 */
public class Databanker {

    private Connection con;
    public final String MAXTF = "MaxTF";	
    public final String AMI = "Ami";	
    public final String ODDS = "Odds";
    private final int blocco = 20;	
	    

    /**
     * Controlla la corretta successione delle operazioni
     * da parte di un utente.
     */	 		

    public Automa automa;
    private boolean enable;	
    
    /**
     * Stabilisce la connessione con la fonte dati associata al database
     * per WebClass.
     */ 

    public Databanker(String fonte) throws ClassNotFoundException,
                                   SQLException {

		String Microsoft = "com.ms.jdbc.odbc.JdbcOdbcDriver";
            String Sun = "sun.jdbc.odbc.JdbcOdbcDriver";
 		 
            try {
            	Class.forName(Microsoft); 
            	con = DriverManager.getConnection("jdbc:odbc:" + fonte, "", ""); 
			System.out.println("Microsoft");
            }
            catch (ClassNotFoundException cn) {
                  Class.forName(Sun); 
            	con = DriverManager.getConnection("jdbc:odbc:" + fonte, "", "");
			System.out.println("Sun");
            }
    }

    /**
     * Restituisce la lista delle stopword
     */
	public synchronized Vector getStopword(String utente) throws SQLException {
			
      	Vector ris = new Vector();
		Statement stmt = null;
		ResultSet rs = null;
		try {
			stmt = con.createStatement();
            	String query = "SELECT parolaNonValida FROM stopword " +
                                 "WHERE login = '" + utente + "'";
            	rs = stmt.executeQuery(query); 
            	while (rs.next()) 
		    		ris.addElement(rs.getString(1));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			
			try {
				if (rs != null)
					rs.close();
				if (stmt != null)
            			stmt.close();
			} catch (SQLException sqe) {
				throw sqe;
			}
            }
            
	}

    
    /** 
    * Restituisce il vettore centroide della categoria e l'utente
    * specificato. Il vettore  ordinato in base al campo feature; ci
    *  necessario per lasciare immutata la posizione di ogni feature (coordinata)
    * nello spazio vettoriale.
    */
    public synchronized double[] getCentroid(String utente, String categoria)
                    throws SQLException, OperationException {

	Statement stmt = null;
	ResultSet rs = null;
	try {	
        	String query = "SELECT valoreCentroide FROM classificazione WHERE "+
            		   "login = '"+utente+"' and categoria = '"+categoria+"' " +
                           "ORDER BY token ASC";
        	stmt = con.createStatement();
        	rs = stmt.executeQuery(query);
        	Vector v = new Vector();
        	while (rs.next()) {
            	v.addElement(new Double(rs.getDouble("valoreCentroide")));}
        	double value[] = new double[v.size()];
        	for (int i = 0;i < value.length;i++) {
            	value[i] = (double) ((Double) v.elementAt(i)).doubleValue();}
		return value;
	} catch (SQLException se) {
		throw se;
	} finally {
        	try {
			if (rs != null)
				rs.close();
			if (stmt != null)
            		stmt.close();
		} catch (SQLException sqe) {
			throw sqe;
		}
	}
        
    }

    
    /**
    *  
    *	Restituisce il vettore delle probabilit bayesiane.
    * Il vettore  ordinato in base al campo feature; ci
    *  necessario per lasciare immutata la posizione di ogni feature (coordinata)
    * nello spazio vettoriale.
    */

     	public synchronized double[] getProbabilities(String utente, String categoria)
			throws SQLException, OperationException {

		Statement stmt = null;
		ResultSet rs = null;
		try {
			Vector result = new Vector();
			String query = "SELECT probabilitaBayes FROM classificazione WHERE "+
					   "login = '" + utente + "' and categoria = '" + categoria + "' " +
                  		   "ORDER BY token ASC";
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
	      	Vector v = new Vector();
        		while (rs.next()) {
            		v.addElement(new Double(rs.getDouble(1)));}
        		double value[] = new double[v.size()];
        		for (int i = 0;i < value.length;i++) {
            		value[i] = (double) ((Double) v.elementAt(i)).doubleValue();}
			return value;
		} catch (SQLException se) {
			throw se;
		} finally {
        		try {
				if (rs != null)
					rs.close();
				if (stmt != null)
            			stmt.close();
			} catch (SQLException sqe) {
				throw sqe;
			}

		}	
        	
    	}
 	


    /**
    * Restituisce le soglie di threshold per ogni utente e categoria
    * specificati.
    */
    public synchronized double getThreshold(String utente, String categoria)
                throws SQLException, OperationException {

	Statement stmt = null;
      ResultSet rs = null; 
	try {
	  String query = "SELECT * FROM categoria WHERE login = '"+utente+"'"+
	  		     " AND nomeCategoria = '"+categoria+"'";
        stmt = con.createStatement();
        rs = stmt.executeQuery(query);
	  double value = 0;	
        if (rs.next())
        	value = (double)rs.getDouble("threshold");
	  return value;
	} catch (SQLException se) {
		throw se;
	} finally {
		try {
			if (rs != null)
				rs.close();
			if (stmt != null)
            		stmt.close();
		} catch (SQLException sqe) {
			throw sqe;
		}
	}
        
    }


    /**
     * Inserisce un nuovo documento di training. Vengono popolate le seguenti 
     * tavole:<ul><li>documento</li><li>appartenenza</li><li>composizione</li>
     * <li>token</li><li>testoLink</li><li>immagineLink</li><li>immagine</li>
     * <li>zona</li><li>composizioneLocale</li></ul>
     */ 
    public synchronized void inserisciDocumentoTT(String utente, String categoria,
                   HtmlPage documento, String tipo)
                   throws SQLException, OperationException, Exception  {


		ResultSet rs = null;
		ResultSet rs2 = null;
		Statement doc = null;
		Statement id = null;
		Statement app = null;
		Statement img = null;
		Statement lnk = null;
		PreparedStatement tok1 = null;
		PreparedStatement stmt = null;
		PreparedStatement tok2 = null;
		PreparedStatement tok3 = null;
		
		try {

            	con.setAutoCommit(false);
			try {
				if (tipo.equalsIgnoreCase("TRAINING")) 
					automa.Transizione("atp", categoria);
				else if (tipo.equalsIgnoreCase("TESTING"))
					automa.Transizione("asp", categoria);
			} catch (OperationException oe) {
				throw oe;
			}
	               
            	doc = con.createStatement(); 
            	doc.executeUpdate("INSERT INTO documento (URL, immagineBackground, titolo, testo) " 
            	+ "VALUES ('" + 
            	documento.getUrl().toString() + "', '"+documento.getBackgroundImage()+
			"', '" + documento.getTitle().replace(' ','_') + "', '" + SQL.correggiApice(documento.getSource())+ "')");

            	id = con.createStatement();
           	 	rs = id.executeQuery("SELECT MAX(idDocumento) FROM documento");
            	int idDocumento = 0;
			if (rs.next())
				idDocumento = rs.getInt(1);
		
            	app = con.createStatement();
            	app.executeUpdate("INSERT INTO appartenenza (login, nomeCategoria, idDocumento, tipo) "
            	+ "VALUES ('" + utente + "', '" + categoria + "', " + idDocumento + ", '" + tipo + "')");  
                 
            	img = con.createStatement();
            	Enumeration immagini = documento.getLinkIMG();
            	while (immagini.hasMoreElements())  {
		   		String urlImg = (String)immagini.nextElement();
		   		img.executeUpdate("INSERT INTO immagine (idDocumento, URL) "+
               		"VALUES (" + idDocumento + ", '" + urlImg + "')");
			}


			lnk = con.createStatement();
            	Ancore link = documento.getLinkA();
            	while (!link.endOfLink) { 
                    	String url = link.getHref();
                    	while (!link.endOfElements) {
                          	String elem = link.nextElement(); 
				  	if (elem != null) {	    
                          		if (uniba.core.File.estensioneNota(elem)) {
						 	lnk.executeUpdate("INSERT INTO immagineLink (idDocumento, "+
                                     	" URL, immagine) VALUES (" + idDocumento + ", '" + url + "', '" + elem + "')");
				  		} else {
							lnk.executeUpdate("INSERT INTO testoLink (idDocumento, "+
                                    	" URL, parola) VALUES (" + idDocumento + ", '" + url + "', '" + elem+"')");
				  		}
				  	}
                    	}
                    	link.nextLink();
            	} 


			tok1 = con.prepareStatement("INSERT INTO token (token) VALUES (?)");
            	stmt = con.prepareStatement("SELECT token FROM token WHERE token = ?");
            	rs2 = null;
	      	Enumeration token = documento.getWords();
            	while (token.hasMoreElements()) {
                   	HtmlWord hw = (HtmlWord)token.nextElement();
                   	stmt.setString(1, hw.getText());
                   	rs2 = stmt.executeQuery();
                   	if (!rs2.next()) {
					tok1.setString(1, hw.getText());
                        	tok1.executeUpdate();
                   	} 
			 	rs2.close();
            	}

            	tok2 = con.prepareStatement("INSERT INTO composizione (idDocumento, token, TF, occorrenza) VALUES (?, ?, ?, ?)");
            	token = documento.getWords(); 
            	while (token.hasMoreElements()) {
            		HtmlWord hw = (HtmlWord)token.nextElement(); 
				tok2.setInt(1, idDocumento);
                  	tok2.setString(2, hw.getText());
                  	tok2.setDouble(3, hw.getTF()); 
                  	tok2.setDouble(4, hw.getOcc()); 
                  	//tok2.setDouble(5, hw.getEnfasiAdditiva()); 
                  	//tok2.setDouble(6, hw.getEnfasiMoltiplicativa());
                  	tok2.executeUpdate();
             	} 

             	tok3 = con.prepareStatement("INSERT INTO contesto (token, idDocumento, tag) VALUES (?, ?, ?)");
             	token = documento.getWords();
             	while (token.hasMoreElements()) {
                  	HtmlWord hw = (HtmlWord)token.nextElement(); 
				if (hw.getAttributes() != null) {	
			      	StringTokenizer st = new StringTokenizer(hw.getAttributes());
			      	while (st.hasMoreTokens()) {
                        		String att = st.nextToken(); 
				    		tok3.setString(1, hw.getText());
                            		tok3.setInt(2, idDocumento);
                            		tok3.setString(3, att);
				    		tok3.executeUpdate();
                        	}
                  	} 
             	} 

			

            	
      
            	con.commit();
           
		} catch (SQLException e) {
			
			try {
				con.rollback();
				System.out.println("Database Rolled Back");
			} catch (SQLException ee) {
				System.out.println("Rollback Impossible: " + ee.getMessage());
				throw ee;
			}
			throw e;
		} finally {
			
			try {
				con.setAutoCommit(true);
				if (rs != null)
            			rs.close();
				//if (rs2 != null)
            	//		rs2.close();
				if (stmt != null)
            			stmt.close();
				if (id != null)
            			id.close();
				if (app != null)
            			app.close();
				if (doc != null)
            			doc.close(); 
				if (img != null)
            			img.close();
				if (tok1 != null)
            			tok1.close();
				if (tok2 != null)
					tok2.close();
				if (tok3 != null)
            			tok3.close();
				if (lnk != null)
            			lnk.close();
			} catch (SQLException se) {
				throw se;
			}
		}
       }

	 /**
	  * Determina il metodo	con il quale sono state estratte le feature
	  * correnti.
	  */
	 public synchronized String getFeatureMode(String utente) throws SQLException {

		PreparedStatement ps1 = null;
		ResultSet rs1 = null;
		
		try {
			con.setAutoCommit(false);
			ps1 = con.prepareStatement("SELECT modoEstrazione FROM configurazione WHERE login = ?");
			String ris = null;
			ps1.setString(1, utente);
			rs1 = ps1.executeQuery();
			if (rs1.next()) {
				ris = rs1.getString(1);
			}
			con.commit();
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally { 
			try {
				con.setAutoCommit(true);
				if (rs1 != null)
					rs1.close();
				if (ps1 != null)
					ps1.close();
			} catch (SQLException sqe) {
				throw sqe;
			}
		}
		
	}
		

      

       /**
        * Restituisce le feature. Il vettore  ordinato in base al campo feature; ci
        *  necessario per lasciare immutata la posizione di ogni feature (coordinata)
        * nello spazio vettoriale.
        */
       public synchronized Vector getFeature(String utente)
          throws SQLException {
		
		Statement stmt = null;
		ResultSet rs = null;
		try {
		 	Vector ris = new Vector();
		 	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 			"SELECT token, categoria, valore FROM feature " +
                 			"WHERE login = '" + utente + "' " +
                 			"ORDER BY token ASC");
             	while (rs.next()) 
		     		ris.addElement( new Feature(rs.getString(1), rs.getString(2), rs.getDouble(3)) );
			
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
             	try {
				if (rs != null)
					rs.close();
				if (stmt != null)
            			stmt.close();
			} catch (SQLException sqe) {
				throw sqe;
			}
		}
	 }
	 
	 
	 

       /**
        * Inserisce le feature
        */
       public synchronized void putFeature(String utente, Vector feature, String operazione, String mode) throws
         SQLException, OperationException {

		PreparedStatement stmt = null;	
            Statement stmt2 = null;
		try {
             	con.setAutoCommit(false);
             	stmt = con.prepareStatement(
                   	"INSERT INTO feature (login, token, categoria, valore) VALUES (?, ?, ?, ?)"); 
                                    	
             	Enumeration item = feature.elements();
             	while (item.hasMoreElements()) {
                  	Feature f = (Feature)item.nextElement();
                  	stmt.setString(1, utente);
                  	stmt.setString(2, f.getFeature());
                  	stmt.setString(3, f.getCategoria());
				stmt.setDouble(4, f.getScore());
                  	stmt.executeUpdate();
             	}

			stmt2 = con.createStatement();
			stmt2.executeUpdate("UPDATE configurazione SET modoEstrazione = '" + mode + "' WHERE login = '" + utente + "'");

             	con.commit();
		} catch (SQLException se) {
			try {
				con.rollback();
			} catch (SQLException sqle) {
				throw sqle;
			}
			throw se;
		} finally {
			try {
             		con.setAutoCommit(true);
				if (stmt != null)
             			stmt.close();
				if (stmt2 != null)
             			stmt2.close();
			} catch (SQLException sqe) {
				throw sqe;
			}
		}
          
        }


        /**
         * Restituisce i token di un documento
         */
        public synchronized Enumeration getWords(int idPage)
           throws SQLException {
		
		Statement stmt = null;
            ResultSet rs = null;
		PreparedStatement ps = null;
		ResultSet rs2 = null;
		try {
			con.setAutoCommit(false);
			Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT token, TF, occorrenza FROM composizione " +
                 		"WHERE idDocumento = " + idPage);	
             	while (rs.next()) {
                   	HtmlWord hw = new HtmlWord(rs.getString(1));
                   	hw.setTF( rs.getDouble(2) );
                   	hw.setOcc( rs.getInt(3) );   
                  	ris.addElement(hw);
             	}
             	ps = con.prepareStatement("SELECT tag FROM contesto " +
                                                         "WHERE idDocumento = ? AND " +
                                                         "token = ?");
             	rs2 = null;
		 	for (int i=0; i<ris.size(); i++) {
				HtmlWord w = (HtmlWord)ris.elementAt(i);
				ps.setInt(1, idPage);
                  	ps.setString(2, w.getText());
                  	rs2 = ps.executeQuery();
                  	String tag = "";
				while (rs2.next()) 
					w.addAttribute( rs2.getString(1) + " " );
                  	ris.setElementAt(w, i);
				rs2.close();
		 	}
		 	con.commit();
			return ris.elements();
		} catch (SQLException se) {
			throw se;
		} finally {
		 	con.setAutoCommit(true);
			if (rs != null)
				rs.close();
			if (ps != null)
		 		ps.close();
			//if (rs2 != null)
			//	rs2.close();
			if (stmt != null)
             		stmt.close();
		}
        }
        
        
        /**
         * Restituisce le pagine di training di un utente.
         * Gli elementi del vettore restituito sono stringhe del tipo
         * <code>idDocumento</code> + <code>categoriaDocumento</code> + <code>urlDocumento</code>
         */
        public synchronized Vector getTrainingPages(String utente) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT appartenenza.idDocumento, appartenenza.nomeCategoria, documento.URL FROM appartenenza INNER JOIN documento ON (appartenenza.idDocumento = documento.idDocumento ) " +
                 		"WHERE login = '" + utente + "' AND tipo = " + "'TRAINING'");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1) + " " + rs.getString(2) + " " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

 /**
         * Restituisce tutte le pagine di un utente.
         * Gli elementi del vettore restituito sono stringhe del tipo
         * <code>idDocumento</code> + <code>categoriaDocumento</code> + <code>urlDocumento</code>
         */
        public synchronized Vector getAllDocuments(String utente) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT appartenenza.idDocumento, appartenenza.nomeCategoria, documento.URL FROM appartenenza INNER JOIN documento ON (appartenenza.idDocumento = documento.idDocumento ) " +
                 		"WHERE login = '" + utente + "'");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1) + " " + rs.getString(2) + " " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

        /**
         * Restituisce le pagine di training di una categoria dell'utente.
         * Gli elementi del vettore restituito sono stringhe del tipo
         * <code>idDocumento</code> + <code>categoriaDocumento</code> + <code>urlDocumento</code>
         */
        public synchronized Vector getTrainingPages(String utente, String categoria) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT appartenenza.idDocumento, appartenenza.nomeCategoria, documento.URL FROM appartenenza INNER JOIN documento ON (appartenenza.idDocumento = documento.idDocumento ) " +
                 		"WHERE login = '" + utente + "' AND tipo = " + "'TRAINING' AND "+
		     		"nomeCategoria = '" + categoria + "'");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1)+ " " + rs.getString(2) + " " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

	  /**
         * Restituisce le pagine di testing di un utente.
         * Gli elementi del vettore restituito sono stringhe del tipo
         * <code>idDocumento</code> + <code>categoria documento</code> + <code>urlDocumento</code>
         */
        public synchronized Vector getTestingPages(String utente) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null; 
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT appartenenza.idDocumento, appartenenza.nomeCategoria, " + 
				"documento.URL FROM appartenenza INNER JOIN documento ON " + 
				"(appartenenza.idDocumento = documento.idDocumento ) " +
                 		"WHERE login = '" + utente + "' AND tipo = " + "'TESTING'");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1)+ " " + rs.getString(2) + " " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }
	
	  /**
         * Restituisce le pagine di testing di un utente ordinate per id.
         * Gli elementi del vettore restituito sono stringhe del tipo
         * <code>idDocumento</code> + <code>categoria documento</code> + <code>urlDocumento</code>
         */
        public synchronized Vector getSortedTestingPages(String utente) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null; 
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT appartenenza.idDocumento, appartenenza.nomeCategoria, " + 
				"documento.URL FROM appartenenza INNER JOIN documento ON " + 
				"(appartenenza.idDocumento = documento.idDocumento ) " +
                 		"WHERE login = '" + utente + "' AND tipo = " + "'TESTING'" + 
				"ORDER BY appartenenza.idDocumento ASC");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1)+ " " + rs.getString(2) + " " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

	  /**
         * Restituisce le pagine di testing di un utente ordinate per id.
         * Gli elementi del vettore restituito sono di tipo <code>int</code>
         */
        public synchronized Vector getSortedTestingPages(String utente, int id1, int id2) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null; 
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT idDocumento FROM appartenenza " + 
                 		"WHERE login = '" + utente + "' AND tipo = " + "'TESTING' " + 
				"AND idDocumento >= " + id1 + " AND idDocumento <= " + id2 +
				" ORDER BY idDocumento"); 
             	while (rs.next())
                 		ris.addElement(new Integer(rs.getInt(1)));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

        /**
         * Restituisce le pagine di testing di una categoria dell'utente.
         * Gli elementi del vettore restituito sono stringhe del tipo
         * <code>idDocumento</code> + <code>categoriaDocumento</code> + <code>urlDocumento</code>
         */
        public synchronized Vector getTestingPages(String utente, String categoria) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT appartenenza.idDocumento, appartenenza.nomeCategoria, documento.URL FROM appartenenza INNER JOIN documento ON (appartenenza.idDocumento = documento.idDocumento ) " +
                 		"WHERE login = '" + utente + "' AND tipo = " + "'TESTING' AND "+
		     		"nomeCategoria = '" + categoria + "'");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1)+ " " + rs.getString(2) + " " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

        /**
         * Restituisce le categorie di un utente
         */
        public synchronized Vector getCategories(String utente) throws
           SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 		"SELECT nomeCategoria FROM categoria " +
                 		"WHERE login = '" + utente + "' AND nomeCategoria <> 'Reject'");
             	while (rs.next())
                 		ris.addElement(rs.getString(1));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }


        /**
         * Restituisce un vettore con i parametri di configurazione. Si noti quanto segue:<ul> 
         * <li>elemento <b>0</b> del vettore = <b>minLenghtThreshold</b></li>
         * <li>elemento <b>1</b> del vettore = <b>maxLenghtThreshold</b></li>
         * <li>elemento <b>2</b> del vettore = <b>not-used</b></li>
         * <li>elemento <b>3</b> del vettore = <b>not-used</b></li>
         * <li>elemento <b>4</b> del vettore = <b>representationType</b></li>  
         * <li>elemento <b>5</b> del vettore = <b>categoryDictionaryMaxSize</b></li>
         * <li>elemento <b>6</b> del vettore = <b>maxWordsPerCategoryAsFeatures</b></li>
         * <li>elemento <b>7</b> del vettore = <b>kFactor</b></li> 
	   * <li>elemento <b>8</b> del vettore = <b>itemToShow</b></li> 
         * <li>elemento <b>9</b> del vettore = <b>not-used</b></li> 
         * <li>elemento <b>10</b> del vettore = <b>not-used</b></li>
	   * <li>elemento <b>11</b> del vettore = <b>not-used</b></li>	
	   * <li>elemento <b>12</b> del vettore = <b>localhost</b></li>
         * <li>elemento <b>13</b> del vettore = <b>porta</b></li>		
         * </ul>   
         */
        public synchronized Vector getConfiguration(String utente) throws  SQLException {

		 Statement stmt = null;
		 ResultSet rs = null;
             try {
		 	Vector ris = new Vector();
             	String temp = null;
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                		"SELECT minLenghtThreshold, maxLenghtThreshold, "+
                		"representationType, " +
                		"categoryDictionaryMaxSize, maxWordsPerCategoryAsFeatures, " +
                		"kFactor, itemToShow, " +
		    		"localhost, porta FROM configurazione "+
                		"WHERE login = '" + utente + "'");
             	while (rs.next()) {
                		ris.addElement(new Integer(rs.getInt("minLenghtThreshold")));
                		ris.addElement(new Integer(rs.getInt("maxLenghtThreshold")));

                		ris.addElement("");	//questi due non servono
                		ris.addElement("");
                		

                		ris.addElement(new Integer(rs.getInt("representationType"))); 

		    		ris.addElement(new Integer(rs.getInt("categoryDictionaryMaxSize")));

		    		ris.addElement(new Integer(rs.getInt("maxWordsPerCategoryAsFeatures")));

		    		ris.addElement(new Integer(rs.getInt("kFactor")));			

		    		ris.addElement(new Integer(rs.getInt("itemToShow")));	

		    		ris.addElement(new Integer(0));  //non serve	

		    		ris.addElement(new Double(0));  //non serve	

		    		ris.addElement(new Double(0));  // non serve	

		    		ris.addElement(rs.getString("localhost"));	

		    		ris.addElement(rs.getString("porta"));	
			}
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
        }

	  /**
         * Cancella una categoria di un utente. Le tavole dalle quali sono eliminati
         * i record sono le seguenti:<ul><li>categoria</li><li>appartenenza</li>
         * <li>classificazione</li></ul> Prima di cancellare una categoria si
         * devono cancellare tutti i documenti della categoria.
         */
	  public synchronized void deleteCategory(String categoria, String utente) throws SQLException,
			OperationException {
		
		PreparedStatement delCat = null;
		try {
			con.setAutoCommit(false);
			automa.Transizione("rc", categoria);
            	delCat = con.prepareStatement("DELETE * FROM categoria " +
                                                            "WHERE login = ? AND nomeCategoria = ?");
			delCat.setString(1, utente);
			delCat.setString(2, categoria);
			delCat.executeUpdate();
			con.commit();
		} catch (SQLException sqe) {
			con.rollback();
			throw sqe;
		} finally {
			con.setAutoCommit(true);
			if (delCat != null)
				delCat.close();
		}
		
	   }	

        /**
         * Cancella una pagina. Le tavole dalle quali sono eliminati
         * i record sono le seguenti:<ul><li>appartenenza</li><li>documento</li>
         * <li>composizione</li><li>contesto</li><li>composizioneLocale</li>
         * <li>zona</li><li>token</li><li>immagine</li><li>immagineLink</li>
         * <li>testoLink</li></ul>
         */
	  public synchronized void deleteDocument(int idPage ) throws SQLException,
			OperationException {
		
		Statement stmt = null;
		ResultSet r = null;
		PreparedStatement delDoc = null;
		PreparedStatement single = null;
		PreparedStatement selTok = null;
		PreparedStatement delTok = null;
		ResultSet rs = null;
		ResultSet rs2 = null;
		try {
			con.setAutoCommit(false);
			stmt = con.createStatement();
			r = stmt.executeQuery("SELECT nomeCategoria, tipo " +
							  "FROM appartenenza " +
							  "WHERE idDocumento = " + idPage );
			if (r.next()) {
				String categoria = r.getString(1);
				String tipo = r.getString(2);
				if (!r.wasNull()){	
					if (tipo.equalsIgnoreCase("TRAINING")) {
						automa.Transizione("rtp",categoria);}
					else 
					if (tipo.equalsIgnoreCase("TESTING")){
						automa.Transizione("rsp",categoria);}
				}
			}
		
			System.out.println("Controllo Automa eseguito correttamente");
			delDoc = con.prepareStatement("DELETE * FROM documento " +
                                                            "WHERE idDocumento = ?");
                                                           
            	single = con.prepareStatement("SELECT * FROM composizione "+
										"WHERE idDocumento <> ? AND token = ?");

			selTok = con.prepareStatement("SELECT token FROM composizione "+
										"WHERE idDocumento = ?");

			delTok = con.prepareStatement("DELETE * FROM token " +
                                                            "WHERE token = ?");

			rs = null;
            	rs2 = null;				//seleziona i token del documento
			selTok.setInt(1, idPage); 
            	rs2 = selTok.executeQuery();
            	while (rs2.next()) {				//controlla se altri documenti
				String token = rs2.getString(1);    //hanno lo stesso token
				single.setInt(1, idPage);
				single.setString(2, token); 
				rs =  single.executeQuery();
				if (!rs.next()) {				//NO: il token viene cancellato
					delTok.setString(1, token);
					delTok.executeUpdate();
				}
				rs.close();
			}

			//infine cancella il documento
			delDoc.setInt(1, idPage);
			delDoc.executeUpdate();
                  con.commit();					//commit

		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			if (r != null)
				r.close();
			if (rs2 != null)					//pulizia
				rs2.close();
			//if (rs != null)					
			//	rs.close();
			if (single != null)
				single.close();
			if (delDoc != null)
				delDoc.close();
			if (delTok != null)
            		delTok.close();
			if (selTok != null)
				selTok.close();
			if (stmt != null)
				stmt.close();
		}
	  }	


	   /**
          * Restituisce la lista delle login presenti nella base di dati
          */       
	   public synchronized Vector getLogin() throws SQLException {
		
		Statement log = null;
		ResultSet rs = null;
		try {
			Vector ris = new Vector();
			log = con.createStatement();
			rs = log.executeQuery("SELECT login FROM utente");
			while (rs.next())
				ris.addElement(rs.getString(1));	 	 
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (log != null)
				log.close();
		}	
		
	   }

	    /**
           * Restituisce la lista delle password presenti nella base di dati
           */       
	   public synchronized Vector getPassword() throws SQLException {
		
		Statement log = null;
		ResultSet rs = null;
		try {
			Vector ris = new Vector();
			log = con.createStatement();
			rs = log.executeQuery("SELECT password FROM utente");
			while (rs.next())
				ris.addElement(rs.getString(1));	 	 
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (log != null)
				log.close();
		}
		
	   }
	
	   

	  
	

        /**
         * Inserisce la categoria di un utente
         */
	  public synchronized void insertCategory(String category, String utente) throws SQLException,
			OperationException {

		Statement cat = null;
		try {
			con.setAutoCommit(false);
			automa.Transizione("ac", category);
			cat = con.createStatement();
			cat.executeUpdate("INSERT INTO categoria (login, nomeCategoria) VALUES ('" +
                             utente + "', '" + category + "')");
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			if (cat != null)
				cat.close();
		}
	  }	


	  /**
         * Restituisce il testo di una pagina HTML, null se la pagina
         * non esiste.
         */
        public synchronized String getText(int idPage ) 
                     throws SQLException 	{

		Statement text = null;
            ResultSet rs = null;
		try {
			text = con.createStatement();
            	rs = text.executeQuery("SELECT testo FROM documento " +
                                             "WHERE idDocumento = " + idPage  );
            	String result = null; 
            	if (rs.next()) {
                 		result = rs.getString(1);
                 	}
			return result;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
            		rs.close();
			if (text != null)
            		text.close();
		}
            
        } 

	  /**
         * Restituisce il numero di parole di una pagina
         */
        public synchronized int getNumWords(int idPage ) throws SQLException {

		Statement count = null;
            ResultSet rs = null;
		try {
			count = con.createStatement();
            	rs = count.executeQuery("SELECT occorrenza FROM composizione " +
                                              "WHERE idDocumento = " + idPage );
            	int ris = 0;
            	while (rs.next())
            		ris = ris + rs.getInt(1);
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
            		rs.close();
			if (count != null)
            		count.close();
		}
            
         } 
         
	/**
         * Restituisce il numero di keywords associate al documento
         */
        public synchronized int getNumWords2(int idPage ) throws SQLException {

		Statement count = null;
            ResultSet rs = null;
		try {
			count = con.createStatement();
            rs = count.executeQuery("SELECT COUNT(token) FROM composizione " +
                                    "WHERE idDocumento = " + idPage );
            int ris = 0;
            if (rs.next())
            	ris = rs.getInt(1);
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
            		rs.close();
			if (count != null)
            		count.close();
		}
            
         }   
                             
	/**
	* Recupero dello stato della precedente sessione e inizio di un nuova 
	* sessione di lavoro per l'utente specificato.
	*/
	public synchronized void automInit(String utente, boolean enable) 
		throws SQLException {

		this.enable = enable;
		automa = new Automa(con, utente, enable);
	}		
    	



	/**
	* Restituisce l'automa che opera sull'oggetto databanker.
	*/
      public synchronized Automa getAutoma() {
		return automa;
	}


      

      /**
       * Inserisce il vettore centroide di una data categoria
       */
      private synchronized void putCentroid(String utente, 
 				       String categoria, 
                               String feature[], 
                               double centroidi[])
                   	throws SQLException, OperationException {

		Statement st = null;
		ResultSet rs = null;
		PreparedStatement cen = null;
		try {
			con.setAutoCommit(false);
			String query = "SELECT * FROM classificazione WHERE login = '"+utente+"' AND"+
					   " categoria = '"+categoria+"'";
			st = con.createStatement();
			rs = st.executeQuery(query);
			if (rs.next()){
				cen = con.prepareStatement("UPDATE classificazione " +
					"SET valoreCentroide = ? WHERE login = '"+utente+"' AND "+
					"categoria = '"+categoria+"' AND token = ?");
				for (int i = 0; i < feature.length; i++) {
                 			cen.setDouble(1, centroidi[i]);
					cen.setString(2,feature[i]);
                  		cen.executeUpdate();}
					cen.close();
			} else {
            		cen = con.prepareStatement("INSERT INTO classificazione" + 
                                  "(login, categoria, token, valoreCentroide) " +
                                  "VALUES (?, ?, ?, ?)");
				for (int i = 0; i < feature.length; i++) {
					cen.setString(1, utente);
            	      	cen.setString(2, categoria);
                  		cen.setString(3, feature[i]);
	                  	cen.setDouble(4, centroidi[i]);
      	            	cen.executeUpdate();
            		}
				cen.close();
			}
            	con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
            	con.setAutoCommit(true);
			if (rs != null)
				rs.close();
			if (st != null)
				st.close();
			//if (cen != null)
			//	cen.close();
		}
   }

      /**
       * Cancella le feature di un utente. 
       */
     public synchronized void deleteFeature(String utente) throws SQLException {
		
		PreparedStatement delFea = null;
		PreparedStatement delClass = null;
            PreparedStatement upConf = null;
		try {
			con.setAutoCommit(false);		
	     		delFea = con.prepareStatement("DELETE * FROM feature " + 
							      "WHERE login = ?");
			delClass = con.prepareStatement("DELETE * FROM classificazione"+
							        " WHERE login = ?");
			upConf = con.prepareStatement("UPDATE configurazione SET modoEstrazione = ? " +
                                                "WHERE login = ?");       
			delFea.setString(1, utente);
			delClass.setString(1, utente);
            	delFea.executeUpdate();
			delClass.executeUpdate();
                  upConf.setString(1, "NONE");
                  upConf.setString(2, utente);
                  upConf.executeUpdate(); 
            	con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
            	con.setAutoCommit(true);
			if (delFea != null)
	      		delFea.close();
			if (delClass != null)
				delClass.close();
			if (upConf != null)
				upConf.close();
		}
     } 


     /**
      * Inserisce i vettori delle probabilit bayesiane nel database
      */		
     private synchronized void putBayesianProbabilities(String utente, Vector prob) 
			throws SQLException, OperationException {

		Statement st = null;
		ResultSet rs = null;
		PreparedStatement p = null;
		try {
			String query = "SELECT * FROM classificazione WHERE login = '"+utente+"'";
			st = con.createStatement();
			rs = st.executeQuery(query);
			if (rs.next()){
				p = con.prepareStatement("UPDATE classificazione " +
					"SET probabilitaBayes = ? WHERE login = '"+utente+"' AND "+
					"categoria = ? AND token = ?");
				for (int i = 0;i < prob.size();i++){
	                  	p.setDouble(1, ((TermBayesianProbability)prob.elementAt(i)).getProb());
	                  	p.setString(2, ((TermBayesianProbability)prob.elementAt(i)).getCat());
	                  	p.setString(3, ((TermBayesianProbability)prob.elementAt(i)).getFea());
						p.executeUpdate();}
				p.close();
			} else {
	            	p = con.prepareStatement("INSERT INTO classificazione " +
				 "(login, categoria, token, probabilitaBayes) "+"VALUES (?, ?, ?, ?)");
				for (int i = 0;i < prob.size();i++){
						p.setString(1, utente);
	                  	p.setString(2, ((TermBayesianProbability)prob.elementAt(i)).getCat()); 
	           			p.setString(3, ((TermBayesianProbability)prob.elementAt(i)).getFea());
	                  	p.setDouble(4, ((TermBayesianProbability)prob.elementAt(i)).getProb());
	                  	p.executeUpdate();}
				p.close();
			}
            } catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
            	if (rs != null)
				rs.close();
			if (st != null)
				st.close();
			//if (p != null)
			//	p.close();
		}
      } 


      /**
       * Restituisce le probabilit bayesiane Il vettore  ordinato in base al campo feature; ci
       *  necessario per lasciare immutata la posizione di ogni feature (coordinata)
       * nello spazio vettoriale.
       */
      public synchronized Vector getBayesianProbabilities(String utente) throws SQLException {		
     	
		Statement prob = null;
		ResultSet rs = null;
		try { 	
         		Vector ris = new Vector();
         		prob = con.createStatement();
	   		rs = prob.executeQuery("SELECT token, probabilitaBayes, categoria FROM " +
                                          "classificazione WHERE login = '" + utente + "' " +
                                          "ORDER BY token ASC");
         		while (rs.next())
             	ris.addElement(rs.getString(1) + "  " + rs.getDouble(2) + "  " + rs.getString(3));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
         			rs.close();
			if (prob != null)
         			prob.close();
		}
         
      }

     /**
      * Cambia il tipo di una pagina di training e\o testing
      */
     public synchronized void updateType(String utente, String cat, int idPage, String newType) throws SQLException {	

		Statement app = null;
		try {
			con.setAutoCommit(false);
			app = con.createStatement();
            	app.executeUpdate("UPDATE appartenenza SET tipo = '" + newType + "' WHERE " +
                              "idDocumento = " + idPage + 
                              " AND login = '" + utente + "' " +
                              "AND nomeCategoria = '" + cat + "'");
			con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			if (app != null)
            		app.close();
		}

     }  
	/**
	* Restituisce la rappresentazione della pagine specificata espressa<BR>
	* mediante un vattore di numeri reali. Il vettore  ordinato in base al campo feature; ci
      *  necessario per lasciare immutata la posizione di ogni feature (coordinata)
      * nello spazio vettoriale.
  	*/

	public synchronized double[] getRepresentedPage(int idDoc, String utente) throws SQLException {
		

		CallableStatement rep = null;
		ResultSet rs2 = null;
		Vector config = new Vector();
		config = getConfiguration(utente);
		int repType = ((Integer)config.elementAt(4)).intValue();
		try {
			switch (repType) {
				case 0:
					rep = con.prepareCall("{ call rappresentazioneGlobale " + idDoc + 
                                        ", " + utente + "}");
                    break;
				case 1:
					rep = con.prepareCall("{ call rappresentazioneGlobaleAdd " + idDoc + 
                                        ", " + utente + "}");
					break;                                        
				case 2:
					rep = con.prepareCall("{ call rappresentazioneGlobaleMol " + idDoc + 
                                        ", " + utente + "}");
					break;                                        
        	}
			rs2 = rep.executeQuery();
            	Vector v = new Vector();
            	while (rs2.next())
                		v.addElement(new Double(rs2.getDouble(2)));
			double value[] = new double[v.size()];
        		for (int i = 0;i < value.length;i++) {
            		value[i] = (double) ((Double) v.elementAt(i)).doubleValue();}
			return value;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs2 != null)
        			rs2.close();
			if (rep != null)
        			rep.close();
		}
        	
    	}
    	
	
      /**
       * Restituisce tutti i documenti di una data categoria.
       * Gli elementi del vettore restituito sono del tipo
       * <code>idDocumento</code> + <code>urlDocumento</code>.
       * Se il parametro <i>categoria</i>  <b>null</b> allora
       * vengono restituiti tutti i documenti dell'utente a prescindere
       * dalla categoria
       */
      public synchronized Vector getAllDocuments(String utente, String categoria) throws SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
            	Vector ris = new Vector();
			stmt = con.createStatement();
            	if (categoria == null)
            		rs = stmt.executeQuery("SELECT appartenenza.idDocumento, documento.URL FROM appartenenza INNER JOIN documento ON (appartenenza.idDocumento = documento.idDocumento) WHERE " +
                                         "login = '" + utente + "' ORDER BY appartenenza.idDocumento");
			else
				rs = stmt.executeQuery("SELECT appartenenza.idDocumento, documento.URL FROM appartenenza INNER JOIN documento ON (appartenenza.idDocumento = documento.idDocumento) WHERE " +
                                         "login = '" + utente + "' AND nomeCategoria = '" + categoria + "'  ORDER BY appartenenza.idDocumento");
		
			while (rs.next()) 
                 		ris.addElement(rs.getInt(1) + " " + rs.getString(2));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
            		stmt.close();
		}
            
       }


	
	
	
	/**
       * Vero se esistono le feature per <i>utente</i>
       */
      public synchronized boolean existFeature(String utente) throws SQLException {

		Statement  st = null;
		ResultSet rs = null;
		try {
			st = con.createStatement();
			rs = st.executeQuery("SELECT * FROM feature WHERE " +
							 "login = '" + utente + "'");
           	 	boolean ris;
			ris = rs.next();
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (st != null)
            		st.close();
		}
	}

     /**
      * Chiude la connessione con il database.
      */
     public synchronized void close() throws SQLException {
		con.close();
     } 

     /**
      * Restituisce la lista di pagine che non sono
      * ne di testing ne di training ne di reject.
      * Il vettore restituito contiene stringhe del tipo
      * <code>idDoc</code> + <code>urlDoc</code>.
      */
     public synchronized Vector getOrdinaryPages(String utente) throws SQLException {
		

		Statement st = null;
	      ResultSet rs = null;
		try {
            	Vector ris = new Vector();
			st = con.createStatement();
	      	rs = st.executeQuery(  "SELECT appartenenza.idDocumento, documento.URL FROM appartenenza INNER JOIN documento ON " +
                                         "(appartenenza.idDocumento = documento.idDocumento) " + 
						     "WHERE login = '" + utente + "' AND " +
                                         "tipo <> 'TRAINING' AND tipo <> 'TESTING' AND tipo <> 'REJECT'" 
                                      );	
            	while (rs.next())
            		ris.addElement(rs.getInt(1) + " " + rs.getString(2));
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (st != null)
            		st.close();
		}
            
	}


	/**
	 * Restituisce il dizionario di <i>category</i> per <i>utente</i>
       */
	public synchronized Enumeration listDictionary(String utente, String category) throws SQLException {
		
		Statement st = null;
		ResultSet rs = null;
		try {
			Vector ris = new Vector();
			st = con.createStatement();
			rs = st.executeQuery("SELECT token, valore FROM feature WHERE " +
						       "login = '" + utente + "' AND " +
							 "categoria = '" + category + "' " +
						       "ORDER BY token ASC");
            	while (rs.next())
				ris.addElement(rs.getString(1) + "  " + rs.getDouble(2) );
			
			return ris.elements();
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (st != null)
				st.close();
		}

	}


	/**
	* Restituisce l'insieme delle query che l'utente specificato ha immagazzinato.
	*/
	public synchronized Enumeration getQueryNames(String utente) throws SQLException{
		
		Statement stmt = null;
        	ResultSet rs = null;
		try {
			Vector allQuery = new Vector();
	      	String query = "SELECT nomeQuery FROM query WHERE login = '" + utente + "' ORDER BY nomeQuery ASC";
			String riga;
			stmt = con.createStatement();
        		rs = stmt.executeQuery(query);
			while (rs.next()) {
				riga = rs.getString(1);
				allQuery.addElement(riga);
			}
			return allQuery.elements();
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
	  			rs.close();
			if (stmt != null)
        			stmt.close();
		}
        	
	}


	/**
	* Restituisce l'insieme delle query relative a <li>categoria</li><br>che l'utente
	* specificato ha immagazzinato.
	*/
	public synchronized Enumeration getQueryNames(String utente, String categoria) 
							throws SQLException{
		Statement stmt = null;
        	ResultSet rs = null;
		try {
			Vector allQuery = new Vector();
	      	String query = "SELECT query FROM attinenza WHERE ((categoria = '" + 					categoria + "') AND (login = '" + utente + "'))";
			String riga;
			stmt = con.createStatement();
        		rs = stmt.executeQuery(query);
			while (rs.next()) {
				riga = rs.getString("nomeQuery");
				allQuery.addElement(riga);
			}
			return allQuery.elements();
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
	  			rs.close();
			if (stmt != null)
        			stmt.close();
		}
        	
	}

	/**
	* Restituisce il testo della query specificata.
	*/
	public synchronized String getQueryText(String utente, String nomeQuery) throws SQLException{
		
		Statement stmt = null;
        	ResultSet rs = null;
		try {
	      	String result;
			String query = "SELECT testoQuery FROM query WHERE login = '" + utente + 
					   "' AND nomeQuery = '" + nomeQuery + "'";
		
	      	stmt = con.createStatement();
        		rs = stmt.executeQuery(query);
			rs.next();
			result = rs.getString(1);
			return result;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
	  			rs.close();
			if (stmt != null)
        			stmt.close();
		}
        	
	}


	/**
	* Elimina la query specificata.
	*/
	public synchronized void eraseQuery(String utente, String nomeQuery) 
					throws SQLException {
	

		PreparedStatement ps = null;	
		try {
			con.setAutoCommit(false);
			String query = "DELETE * FROM query WHERE login = '" + utente +
				"' and nomeQuery = '" + nomeQuery+"'";
			ps = con.prepareStatement(query);
			ps.executeUpdate();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			if (ps != null)
				ps.close();
		}
	}


	/**
	* Aggiorna il testo della query con <li>testoQuery</li><br>
	* se <li>nomeQuery</li> e gi presente altrimenti la query<br>
	* viene memorizzata con <li>nomeQuery</li> e <li>testoQuery</li>.
	*/
	public synchronized void setQuery(String utente, String nomeQuery, String testoQuery, 		Vector tokens, Vector categories) throws SQLException {

		PreparedStatement ps1 = null;
		PreparedStatement ps2 = null;
		PreparedStatement ps3 = null;
		PreparedStatement ps4 = null;
		try {
			String query1 = "";
			if (existQuery(utente, nomeQuery)){ 
				query1 = "DELETE * FROM query WHERE login = '" + utente + 
					"' AND nomeQuery = '" +	nomeQuery+"'";
				ps1 = con.prepareStatement(query1);
				ps1.executeUpdate();
				ps1.close();
				System.out.println("Eliminazione di "+nomeQuery);
			}
			String query2 = "INSERT INTO query VALUES ('" + utente + "', '" + nomeQuery +
					    "', '" + SQL.correggiApice(testoQuery) + "')";
			String query3 = "INSERT INTO tokenQuery (token, query, login) VALUES "+
			                "(?, ?, ?)";
			String query4 = "INSERT INTO attinenza (query, categoria, login) VALUES " +
					    "(?, ?, ?)";	 
			ps2 = con.prepareStatement(query2);
			ps3 = con.prepareStatement(query3);
			ps4 = con.prepareStatement(query4);
			ps2.executeUpdate();
			System.out.println("Inserimento di "+nomeQuery);  
			for (int i = 0;i < tokens.size();i++) {
				ps3.setString(3, utente);
				ps3.setString(2, nomeQuery);
				ps3.setString(1, ((HtmlWord)tokens.elementAt(i)).getText());
				ps3.executeUpdate();
			}
			System.out.println("Inserimento token");  
			for (int j = 0;j < categories.size();j++) {
				ps4.setString(1, nomeQuery);
				ps4.setString(2, (String)categories.elementAt(j));
				ps4.setString(3, utente);
				ps4.executeUpdate();
			}
			System.out.println("Inserimento categorie");  
			con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			//if (ps1 != null)
			//	ps1.close();
			if (ps2 != null)
				ps2.close();
			if (ps3 != null)
				ps3.close();
			if (ps4 != null)
				ps4.close();
		}
	}

	/**
	* Restituisce <li>true</li> se la Query <li>nomeQuery</li>.
	*/
	public boolean existQuery(String utente, String nomeQuery) throws SQLException {
		String query1 = "SELECT * FROM query WHERE login = '"+utente+"' AND "+
			"nomeQuery = '"+nomeQuery+"'";
		Statement stmt = con.createStatement();
		ResultSet rs = stmt.executeQuery(query1);
		boolean result = rs.next();
		rs.close();
		stmt.close();
		return result;
	}


	/**
	* Calcola la rilevanza, senza enfasi, di ciascun documento di un utente restituendo<br>
	* l'elenco dei documenti in ordine di rilevanza organizzati in un vettore.<br>
	* Come ultimo elemento del vettore  presente un numero reale che rappresenta la soglia.
	*/
	
	
	public synchronized Vector getSignificance(String utente, String nomeQuery,
			Vector categorie) throws SQLException {
		Vector result = new Vector();
		CallableStatement ril = null;
		ResultSet rs = null;
		Vector config = new Vector();
		config = getConfiguration(utente);
		int repType = ((Integer)config.elementAt(4)).intValue();
		try {
			String tavola = "";
			switch (repType) {
				case 0: tavola = "creaRilevanza2 ";
						break;
				case 1: tavola = "creaRilevanzaAdd ";
						break;
				case 2: tavola = "creaRilevanzaMol ";
						break;
			}
			con.setAutoCommit(false);
			ril = con.prepareCall("{ call " + tavola + utente + ", " + nomeQuery + "}");
			rs = ril.executeQuery();
			int h = 1;
			while (rs.next()) {
				int idDoc = rs.getInt(1);
				String url = rs.getString(2);
				double sig = rs.getDouble(3);
				String cat = rs.getString(4);
				result.addElement(idDoc + " " + url + " " + sig + " " + cat);
				h++;
			}
		} catch(SQLException ex) {
			System.out.println(ex.getMessage());
			ex.printStackTrace();
			con.rollback();
			throw ex;
		} finally {
			con.setAutoCommit(true);
			if (rs != null)
				rs.close();
			if (ril != null)
				ril.close();
			return result;
		}
	}

	/**
	* Determina la categoria del documento specificato.
	*/
	public synchronized String getCategory(int idDoc) throws SQLException {

		Statement st = null;
		ResultSet rs = null;
		try {
			String query = "SELECT nomeCategoria FROM appartenenza WHERE idDocumento = " + idDoc;
			st = con.createStatement();
			rs = st.executeQuery(query);
			rs.next();
			String result = rs.getString(1);
			return result;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (st != null)
				st.close();
		}
		
	}


	/**
	* Restituisce la classe come risultato di<br>
	* classificazione delle pagine HTML dell'utente specificato.
	*/
	public synchronized Vector NNTest(String utente, CategoriesManager cgm)
				throws SQLException, OperationException{
		CallableStatement creaTr = null;
		CallableStatement creaTs = null;
		CallableStatement nnTest = null;
		CallableStatement classify = null;
		CallableStatement dropTr = null;
		CallableStatement dropTs = null;
		CallableStatement dropNN = null;
		ResultSet rs = null;
		Vector config = new Vector();
		config = getConfiguration(utente);
		int repType = ((Integer)config.elementAt(4)).intValue();
		try {
			Vector pagine = new Vector();
			double tVector[] = new double[cgm.totalCategories()];
			for (int i = 0;i < cgm.totalCategories();i++) 
				tVector[i] = getThreshold(utente, cgm.categoryName(i));
			String result = null;
			con.setAutoCommit(false);
			switch (repType) {
				case 0:	{
					creaTr = con.prepareCall("{call createTraining " + utente + "}");
					creaTs = con.prepareCall("{call createTesting " + utente + "}");}
					break;
				case 1: {
					creaTr = con.prepareCall("{call createTrainingAdd " + utente + "}");
					creaTs = con.prepareCall("{call createTestingAdd " + utente + "}");}
					break;
				case 2: {
					creaTr = con.prepareCall("{call createTrainingMol " + utente + "}");
					creaTs = con.prepareCall("{call createTestingMol " + utente + "}");}
					break;
			}
			System.out.println("Sta per eseguire la creazione");
			creaTr.executeUpdate();
			System.out.println("Tavola Training creata");
			creaTs.executeUpdate();
			System.out.println("Tavola Testing creata");
			int inizioIntervallo = getMinIdDoc(utente);
			int fineIntervallo = 0;
			int ultimo = getMaxIdDoc(utente);
			while (inizioIntervallo <= ultimo) {
				fineIntervallo = inizioIntervallo + blocco;
				nnTest = con.prepareCall("{ call creaNNclassifica "  
											+ inizioIntervallo + ", " 
											+ fineIntervallo + "}");
				nnTest.executeUpdate();
				nnTest.close();
				classify = con.prepareCall("{ call risultato " + utente + ", "
											+ inizioIntervallo + ", " 
											+ fineIntervallo + "}");
	            	rs = classify.executeQuery();
System.out.println("Query eseguita da " + inizioIntervallo + " a " + fineIntervallo);
				int pred = -1,succ = -1; 
				while(rs.next()){
					succ = rs.getInt(1);
					if (succ != pred){
						String reale = rs.getString(2);
						double dis = rs.getDouble(4);
						result = rs.getString(3);
						if (rs.wasNull()) {
							result = "UNKNOWN";
						} else 
						if (!reale.equals("Reject")) {
							if (dis <= (double)tVector[cgm.categoryIndex(reale)]/100)
								result = "UNKNOWN";   
						}
						pagine.addElement(succ + " " + reale + " " + result);
						pred = succ;
					}
				}
				rs.close();
				classify.close();
				dropNN = con.prepareCall("{call DropNNclassifica}");
				dropNN.executeUpdate();
				dropNN.close();
				inizioIntervallo = fineIntervallo + 1;
			}
			dropTr = con.prepareCall("{call dropTraining}");
			dropTs = con.prepareCall("{call dropTesting}");
			dropTr.executeUpdate();
			dropTs.executeUpdate();
			con.commit();
			return pagine;
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			if (creaTr != null)
				creaTr.close();
			if (creaTs != null)
				creaTs.close();
			//if (classify != null)
			//	classify.close();
			if (dropTr != null)
				dropTr.close();
			if (dropTs != null)
				dropTs.close();
			//if (rs != null)
			//	rs.close();
			//if (classify != null)
			//	classify.close();
		}

	}


	/**
	* Restituisce la classe come risultato di<br>
	* classificazione delle pagine HTML dell'utente specificato.
	*/
	public synchronized Vector KNNTest2(int kFactor, String utente)
								throws SQLException {

		CallableStatement creaTr = null;
		CallableStatement creaTs = null;
		CallableStatement knn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		CallableStatement dropTr = null;
		CallableStatement dropTs = null;
		CallableStatement dropKNN = null;
		Vector config = new Vector();
		config = getConfiguration(utente);
		int repType = ((Integer)config.elementAt(4)).intValue();
		try {
			con.setAutoCommit(false);
			Vector test = new Vector();
			int idDoc = 0;
			String urlDoc = "";
System.out.println("Sta per eseguire la creazione");
			switch (repType) {
				case 0:	{
					creaTr = con.prepareCall("{call createTraining " + utente + "}");
					creaTs = con.prepareCall("{call createTesting " + utente + "}");}
					break;
				case 1: {
					creaTr = con.prepareCall("{call createTrainingAdd " + utente + "}");
					creaTs = con.prepareCall("{call createTestingAdd " + utente + "}");}
					break;
				case 2: {
					creaTr = con.prepareCall("{call createTrainingMol " + utente + "}");
					creaTs = con.prepareCall("{call createTestingMol " + utente + "}");}
					break;
			}
			creaTr.executeUpdate();
System.out.println("Tavola training creata");	
			creaTs.executeUpdate();
System.out.println("Tavola testing creata");
			CategoriesManager cgm = new CategoriesManager(this, utente);
			int inizioIntervallo = getMinIdDoc(utente);
			int fineIntervallo = 0;
			int ultimo = getMaxIdDoc(utente);
			while (inizioIntervallo <= ultimo) {
				fineIntervallo = inizioIntervallo + blocco; 
				knn = con.prepareCall("{ call creaKNNclassifica " + 
					inizioIntervallo + ", " + fineIntervallo + "}");
				knn.executeUpdate();
System.out.println("Tavola KNNclassifica da " + inizioIntervallo + " a " + 
	fineIntervallo + " creata");
				String query = "SELECT TOP " + kFactor + " risultato, distanza FROM " + 
					"KNNclassifica WHERE idDocTest = ?";
				ps = con.prepareStatement(query);
				Vector pages = new Vector();
				pages = getSortedTestingPages(utente, inizioIntervallo, fineIntervallo); 
				for (int j = 0;j < pages.size();j++){
					idDoc = ((Integer)pages.elementAt(j)).intValue();
					ps.setInt(1, idDoc);
					rs = ps.executeQuery();
					Vector page = new Vector();
					while (rs.next()){
						double d = rs.getDouble(2);
						String c = rs.getString(1);
						page.addElement(new kNNVectorElement(d, cgm.categoryIndex(c)));
					}
					test.addElement(page);
					rs.close();
				}
				ps.close();	
				knn.close();
				dropKNN = con.prepareCall("{call DropKNNclassifica}");
				dropKNN.executeUpdate();
				dropKNN.close();	
				inizioIntervallo = fineIntervallo + 1;
			}
			dropTr = con.prepareCall("{call dropTraining}");
			dropTs = con.prepareCall("{call dropTesting}");
			dropTr.executeUpdate();
			dropTs.executeUpdate();
			creaTr.close();
			creaTs.close();
			//knn.close();
			dropTr.close();
			dropTs.close();
			//dropKNN.close();
			con.commit();
			return test;
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			//if (creaTr != null)
			//	creaTr.close();
			//if (creaTs != null)
			//	creaTs.close();
			//if (knn != null)
			//	knn.close();
			//if (ps != null)
			//	ps.close();
			//if (rs != null)
			//	rs.close();
			//if (dropTr != null)
			//	dropTr.close();
			//if (dropTs != null)
			//	dropTs.close();
			//if (dropKNN != null)
			//	dropKNN.close();
		}
			
	}

	/**
	* Restituisce il vocabolario della categoria specificata.
	*/
	public synchronized Vector dictionary(String utente, String categoria) throws SQLException {

		CallableStatement dict = null;
		ResultSet rs = null;
		try {
			Vector result = new Vector();
			Vector config = new Vector();
			config = getConfiguration(utente);
			int maxSize = ((Integer)config.elementAt(5)).intValue() * 3;
			dict = con.prepareCall("{ call vocabolarioCategoria " + utente + 
                                                   ", " + categoria + "}");
            	rs = dict.executeQuery();
			result = new Vector();
  			for (int i = 0;(i < maxSize)&&(rs.next());i++)
				result.addElement(new DoubleWeightedWord(rs.getString(1), rs.getDouble(2)));
			return result;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (dict != null)
				dict.close();
		}
		
	}
		

		
	/**
	* Restituisce i vettori delle probabilit bayesiane per ogni categoria.
	*/
	public synchronized String makeBayesianProbabilities(String utente) throws SQLException, 
										OperationException {

		CallableStatement cs = null;
            ResultSet rs = null;
		try {
			con.setAutoCommit(false);
			Vector prob = new Vector();
			String cat;
			String fea;
			String result = "Features            Bayes Probabilities    Category\n";
			double probab = 0;
			cs = con.prepareCall("{ call makeBayesian " + utente + "}");
            	rs = cs.executeQuery();
			while (rs.next()){
				prob.addElement(new TermBayesianProbability(rs.getString(2),
										rs.getDouble(3),
										rs.getString(1)));
			}
			rs.close();
			cs.close();
			for (int i = 0;i < prob.size();i++) {
				cat = ((TermBayesianProbability)prob.elementAt(i)).getCat();
				fea = ((TermBayesianProbability)prob.elementAt(i)).getFea();
				probab = ((TermBayesianProbability)prob.elementAt(i)).getProb();
		   		String linea = fea + "                           ";
				linea = linea.substring(0,20) + probab + "                           ";
				linea = linea.substring(0,43) + cat;
		      	result += "\n" + linea;
			}
	      	putBayesianProbabilities(utente, prob); 
	      	result += "\n\n Bayesian Probabilities created";
			con.commit();
			return result;
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			//if (cs != null)
			//	cs.close();
			//if (rs != null)
			//	rs.close();
		}
	}


	/**
	* Il seguente metodo  utilizzato dal classificatore<br>
	* per velocizzare l'operazione di classificazione<br>
	* di tutte le pagine di testing.
	*/
	public synchronized Vector KNNTest(int kFactor, Enumeration testPage, String utente)
								throws SQLException {

		CallableStatement creaTr = null;
		CallableStatement creaTs = null;
		CallableStatement knn = null;
		PreparedStatement ps = null;
		ResultSet rs = null;
		CallableStatement dropTr = null;
		CallableStatement dropTs = null;
		CallableStatement dropKNN = null;
		try {
			con.setAutoCommit(false);
			Vector test = new Vector();
			int idDoc = 0;
			String urlDoc = "";
			creaTr = con.prepareCall("{call createTraining " + utente + "}");
			creaTs = con.prepareCall("{call createTesting " + utente + "}");
			creaTr.executeUpdate();
			creaTs.executeUpdate();
			CategoriesManager cgm = new CategoriesManager(this, utente);
			knn = con.prepareCall("{ call creaKNNclassifica}");
			knn.executeUpdate();
			String query = "SELECT TOP " + kFactor + " risultato, distanza FROM " + 
				"KNNclassifica WHERE idDocTest = ?";
			try {
				ps = con.prepareStatement(query);
				while (testPage.hasMoreElements()) {
					StringTokenizer st = new StringTokenizer((String)testPage.
						nextElement());
					idDoc = (new Integer(st.nextToken())).intValue();
					ps.setInt(1, idDoc);
					rs = ps.executeQuery();
					Vector page = new Vector();
					while (rs.next()){
						double d = rs.getDouble(2);
						String c = rs.getString(1);
						page.addElement(new kNNVectorElement(d, cgm.categoryIndex(c)));
					}
					test.addElement(page);
					rs.close();
				}
				ps.close();	
			} catch(SQLException sqle) {
				dropTr = con.prepareCall("{call dropTraining}");
				dropTs = con.prepareCall("{call dropTesting}");
				dropKNN = con.prepareCall("{call dropKNNclassifica}");
				dropTr.executeUpdate();
				dropTs.executeUpdate();
				dropKNN.executeUpdate();
				dropTr.close();
				dropTs.close();
				dropKNN.close();
				throw sqle;
			}
			dropTr = con.prepareCall("{call dropTraining}");
			dropTs = con.prepareCall("{call dropTesting}");
			dropKNN = con.prepareCall("{call dropKNNclassifica}");
			dropTr.executeUpdate();
			dropTs.executeUpdate();
			dropKNN.executeUpdate();
			creaTr.close();
			creaTs.close();
			knn.close();
			dropTr.close();
			dropTs.close();
			dropKNN.close();
			con.commit();
			return test;
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			//if (creaTr != null)
			//	creaTr.close();
			//if (creaTs != null)
			//	creaTs.close();
			//if (knn != null)
			//	knn.close();
			//if (ps != null)
			//	ps.close();
			//if (rs != null)
			//	rs.close();
			//if (dropTr != null)
			//	dropTr.close();
			//if (dropTs != null)
			//	dropTs.close();
			//if (dropKNN != null)
			//	dropKNN.close();
		}
			
	}

	/**
	 * Per ogni categoria c, e per ogni token t, restituisce
	 * il numero di documenti di categoria c che contengono
	 * il token t.
	 */
	public synchronized Hashtable getWordsInCategory(String utente, String categoria) throws SQLException {

		CallableStatement rep = null;
		ResultSet rs = null;
		try {
			rep = con.prepareCall("{ call occCategory " + utente + ", " + categoria + " }");
			rs = rep.executeQuery();
			Hashtable h = new Hashtable();
			while (rs.next()) 
				h.put(rs.getString("token"), new Integer(rs.getInt("numeroPagine")));
			return h;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (rep != null)
				rep.close();	
		}
		
		
	}

	/**
	* Controlla se la login di un nuovo utente  gi presente nel database.
	*/

	public synchronized boolean existUser(String login) throws SQLException {

		Statement stmt = null;
		ResultSet rs = null;
		try {
			String query = "SELECT * FROM utente WHERE login = '" + login + "'";
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
			boolean result = rs.next();
			return result;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
				stmt.close();
		}
	}

	/**
	* Estrae dal database le informazioni relative all'utente specificato.
      * <li>elemento <b>0</b> del vettore = <b>login</b></li>
      * <li>elemento <b>1</b> del vettore = <b>password</b></li>
      * <li>elemento <b>2</b> del vettore = <b>nome</b></li>
      * <li>elemento <b>3</b> del vettore = <b>cognome</b></li>
      * <li>elemento <b>4</b> del vettore = <b>e-mail</b></li>  
	*/

	public synchronized Vector getUser(String login) throws SQLException {

		Statement stmt = null;
		ResultSet rs = null;
		try {
			String query = "SELECT * FROM utente WHERE login = '" + login + "'";
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
			Vector result = new Vector();
			if (rs.next()) {
				for (int i = 1;i <= 5; i++)
					result.addElement(rs.getString(i));
			}
			return result;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
				stmt.close();
		}
		
	}

	/**
	* Inserisce un nuovo utente nel database con la sua configurazione.
	*/
	
	public synchronized void putUser(	String login, 
							String password, 
							String nome, 
							String cognome,
						  	String email,
							String userType,
							int minLT,
							int maxLT,	
							int maxWCF,
							int maxSCD,
							int repType,
							int kFactor,
							String localHost,
							String porta) throws SQLException {


		Statement stmt = null;
		ResultSet rs = null;
		Statement stmt1 = null;
		Statement stmt2 = null;
		Statement stmt3 = null;
		Statement stmt4 = null;
		Statement stmt5 = null;
		Statement s = null;
		try {
			con.setAutoCommit(false);
			String query = "SELECT * FROM utente WHERE login = '" + login + "'";
			String query1;
			String query2;
			String query3;
			String query4;
			String query5;
			boolean inesistente = true;
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
			if (rs.next()) {
				inesistente = false;
				if (email.equals("")) {
					query1 = 	"UPDATE utente SET " + 
							"nome = '" + nome + "', " + 
							"cognome = '" + cognome + "', " + 
							"password = '" + password + "' " + 
							"WHERE login = '" + login + "'"; 
				} else { 
					query1 = 	"UPDATE utente SET " + 
							"nome = '" + nome + "', " + 
							"cognome = '" + cognome + "', " + 
							"password = '" + password + "', " + 
							"email = '" + email + "' " + 
							"WHERE login = '" + login + "'"; 
				}
				query2 = 	"UPDATE configurazione SET " + 
							"minLenghtThreshold = " + minLT + ", " +
							"maxLenghtThreshold = " + maxLT + ", " +
							"maxWordsPerCategoryAsFeatures = " + maxWCF + ", " +
							"categoryDictionaryMaxSize = " + maxSCD + ", " + 
							"representationType = " + repType + ", " +
							"kFactor = " + kFactor + " WHERE " + 
							"login = '" + login + "'";
				
			} else {
				if (email.equals("")) {
					query1 = "INSERT INTO utente ("	+ "login, " 
											+ "password, "
											+ "nome, "
											+ "cognome) "
											+ "values ('" 
											+ login + "', '"
											+ password + "', '"
											+ nome + "', '"
											+ cognome + "')";
				} else {
					query1 = "INSERT INTO utente ("	+ "login, " 
											+ "password, "
											+ "nome, "
											+ "cognome, "
											+ "email) "
											+ "values ('" 
											+ login + "', '"
											+ password + "', '"
											+ nome + "', '"
											+ cognome + "', '"
											+ email + "')";

				}
				query2 = "INSERT INTO configurazione ("
									+ "login, "
									+ "minLenghtThreshold, "
									+ "maxLenghtThreshold, "
									+ "maxWordsPerCategoryAsFeatures, "
									+ "categoryDictionaryMaxSize, "
									+ "representationType, "
									+ "nroSessione, "
									+ "stato, "
									+ "kFactor) "
									+ "values('" 
									+ login + "', "
									+ minLT + ", "
									+ maxLT + ", " 
									+ maxWCF + ", "
									+ maxSCD + ", "
									+ repType + ", 1, 1,"
									+ kFactor + ")";
				
			}
			rs.close();
			stmt.close();
			stmt1 = con.createStatement();
			stmt1.executeUpdate(query1);
			stmt1.close();
			stmt2 = con.createStatement();
			stmt2.executeUpdate(query2);
			stmt2.close();
			if (inesistente){
				String q = "INSERT INTO categoria(login, nomeCategoria, threshold) " + 
					     "values('" + login + "', 'Reject',0)";
				s = con.createStatement();
				s.executeUpdate(q);
				s.close();
			}
			if ((!localHost.equals("")) && (localHost != null)) {
				query3 = "UPDATE configurazione SET localhost = '" + localHost + "'" +
					" WHERE login = '" + login + "'"; 
				stmt3 = con.createStatement();
				stmt3.executeUpdate(query3);
				stmt3.close();
			}
			if ((!porta.equals("")) && (porta != null)) {
				query4 = "UPDATE configurazione SET porta = '" + porta + "'" +
					" WHERE login = '" + login + "'";
				stmt4 = con.createStatement();
				stmt4.executeUpdate(query4);
				stmt4.close();
			}
			if ((!userType.equals("")) && (userType != null) && inesistente) {
				query5 = "UPDATE utente SET tipoUtente = '" + userType + "'" +
					" WHERE login = '" + login + "'";
				stmt5 = con.createStatement();
				stmt5.executeUpdate(query5);
				stmt5.close();
			}	
			con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			//if (stmt != null)
			//	stmt.close();
			//if (rs != null)
			//	rs.close();
			//if (stmt1 != null)
			//	stmt1.close();
			//if (stmt2 != null)
			//	stmt2.close();
			//if (stmt3 != null)
			//	stmt3.close();
			//if (stmt4 != null)
			//	stmt4.close();
			//if (stmt5 != null)
			//	stmt5.close();
			//if (s != null)
			//	s.close();
		}
	}



	/**
	 * Aggiorna la soglia di rigetto per <code>utente</code> e <code>categoria</code>.
	 */
	public synchronized void updateThreshold(String utente, String categoria, double soglia) throws SQLException {

		Statement stmt = null;
		try {
			String query = "UPDATE categoria SET threshold = " + soglia + 
			         " WHERE login = '" + utente + "' " +
				   "AND nomeCategoria = '" + categoria + "'";
			stmt = con.createStatement();
			stmt.executeUpdate(query);
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			if (stmt != null)	
				stmt.close();
		}
	}

	/**
       * Restituisce le pagine di rigetto di <code>utente</code>.
       * Gli elementi del vettore restituito sono stringhe del tipo
       * <code>idDocumento</code>  
       */
      public synchronized Vector getRejectPages(String utente) throws SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
             	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 			"SELECT idDocumento FROM appartenenza " +
                 			"WHERE login = '" + utente + "' AND tipo = " + "'REJECT'");
             	while (rs.next())
                 		ris.addElement(rs.getInt(1)+ "");
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
				rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
      }

	/**
	 * Restituisce il vettore contenente stringhe della forma:
	 * <i>categoria</i> + <i>soglia</i>.
	 */
	public synchronized Vector getThresholdes(String utente) throws SQLException {

		Statement stmt = null;
            ResultSet rs = null;
		try {
		 	Vector ris = new Vector();
             	stmt = con.createStatement();
             	rs = stmt.executeQuery(
                 			"SELECT nomeCategoria, threshold FROM categoria " +
                 			"WHERE login = '" + utente + "'");

             	while (rs.next()) {
		     		String cat = rs.getString(1);
		     		if (!cat.equalsIgnoreCase("REJECT"))	
                 			ris.addElement(cat + " " + rs.getDouble(2));
		 	}
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
             		rs.close();
			if (stmt != null)
             		stmt.close();
		}
             
	}
	

	/**
	* Elimina l'utente dal Database con tutti gli oggetti realtivi.
	*/

	public synchronized void delUser(String utente) throws Exception, SQLException {

		Statement stmt = null;
		ResultSet rs = null;
		Statement stmt1 = null;
		try {
			con.setAutoCommit(false);
			int rowCount = 0;
			String query = "SELECT * FROM appartenenza WHERE login = '" + utente + "'";
			stmt = con.createStatement();
			rs = stmt.executeQuery(query);
			if (rs.next()) {
				Exception ex = new Exception("Prima di eliminare l'utente " + utente + "\n" +
								     "occorre eliminare ogni suo documento.");
				throw ex;
			} else {
				stmt1 = con.createStatement();
				stmt1.executeUpdate("DELETE * FROM utente WHERE login = '" + utente + "'");
				stmt1.close();
			}
			con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {			
			con.setAutoCommit(true);
			if (rs != null)
				rs.close();
			if (stmt != null)
				stmt.close();
			//if (stmt1 != null)
			//	stmt1.close();
		}
	}

	

	/**
	 * Restituisce il tipo di un documento.
	 */
	public synchronized String getType(int idPage) throws SQLException {

		Statement text = null;
            ResultSet rs = null;
		try {
			text = con.createStatement();
            	rs = text.executeQuery("SELECT tipo FROM appartenenza " +
                                             "WHERE idDocumento = " + idPage );
			String result = null;
            	if (rs.next()) 
				result = rs.getString(1);
			return result;	
            } catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null)
            		rs.close();
			if (text != null)
            		text.close();	
		}

            
	}

	/**
	 * Aggiorna un documento.
	 */
	public synchronized void updateDocument(int idPage, String urlPage, String cat, String type) throws SQLException {

		Statement st1 = null;
		Statement st2 = null;
		try {
			con.setAutoCommit(false);
			st1 = con.createStatement();
			st2 = con.createStatement();
			String q1 = "UPDATE documento SET URL = '" + urlPage + "' WHERE idDocumento = " + idPage;
			String q2 = "UPDATE appartenenza SET nomeCategoria = '" + cat + "', tipo = '" + type + "' WHERE idDocumento = " + idPage;
			st1.executeUpdate(q1);
			st2.executeUpdate(q2);
			con.commit();
		} catch (SQLException se) {
			con.rollback();
			throw se;
		} finally {
			con.setAutoCommit(true);
			if (st1 != null)
				st1.close();
			if (st2 != null)
				st2.close();
		}
	}

	/**
	 * Rerstituisce l'URL di un documento.
	 */
	public synchronized String getUrl(int idDoc) throws SQLException {


		PreparedStatement st1 = null;
		ResultSet rs = null;
		try {
			String ris = null;
			st1 = con.prepareStatement("SELECT URL FROM documento WHERE idDocumento = ?");
			st1.setInt(1, idDoc);
			rs = st1.executeQuery();
			if (rs.next())
				ris = rs.getString(1);
			return ris;
		} catch (SQLException se) {
			throw se;
		} finally {
			if (rs != null) 
				rs.close();
			if (st1 != null)
				st1.close();
		}
		
	}


/**
 * Genera i vettori centroidi per ogni categoria e li memorizza nel database.
 * La stringa ritornata  un report da visualizzare per l'utente.	
 */


  public synchronized String makeCentroids(String utente, CategoriesManager cgm) throws SQLException,
						OperationException, Exception {
	
	int numeroFeature = getFeature(utente).size();
	int numeroCategorie = getCategories(utente).size();
	double value[] = new double[numeroFeature];
      double sum[][] = new double[numeroFeature][numeroCategorie];
	int i;
	int j;
	int totPageInCategory[] = new int[numeroCategorie];
	String result = "Making clusters...\n";
	for (j=0; j < numeroCategorie; j++) { totPageInCategory[j]=0; }
	for (i=0; i < numeroFeature; i++) {
	  value[i]=0;
	  for (j=0; j < numeroCategorie; j++) { sum[i][j]=0; }
	}
	String line;
	String extractedCategoryName;
	int currentCategory=0;
      Enumeration trainingPages = getTrainingPages(utente).elements();
	while (trainingPages.hasMoreElements()) {
         StringTokenizer stk = new StringTokenizer((String)trainingPages.nextElement());
	   int idDoc = (new Integer(stk.nextToken())).intValue();	
         extractedCategoryName = getCategory( idDoc );
	   if (!extractedCategoryName.equals("")) {
      	 value = getRepresentedPage(idDoc, utente);
		 currentCategory = cgm.categoryIndex(extractedCategoryName);
		 totPageInCategory[currentCategory]++;
		 for (i=0; i < numeroFeature; i++) 
      		   sum[i][currentCategory]+=value[i];
	   }
	}
	Vector features = new Vector();
	features = getFeature(utente);	
	for (j=0; j<numeroCategorie; j++) {
		result += "\n" + "Center of " + totPageInCategory[j] + " vectors in '" + 
			cgm.categoryName(j) + "' is: \n";
	      String vec = "";
		String res = "";
            String fea[] = new String[numeroFeature];
		double centroid[] = new double[numeroFeature];
		for (i=0; i<numeroFeature; i++) {
                centroid[i] = sum[i][j] / (double)totPageInCategory[j];
		    fea[i] = ((Feature)features.elementAt(i)).getFeature(); 
		    res = fea[i] + "                                 "; 	
                vec = vec + (String)res.substring(0,20) + centroid[i] + "\n";
		}
            putCentroid(utente, cgm.categoryName(j), fea, centroid);
	      result += vec + "\n";
	}
	result += "\n Clusters created";
	return result;
  } 


/**
* Genera le informazioni necessarie all'estrazione delle feature <br>
* secondo il metodo "Average Mutual Information".
*/ 
public void initAMI(String utente) throws SQLException {
	con.setAutoCommit(false);
	CallableStatement num = null;
	CallableStatement numc = null;
	
	try {
		num = con.prepareCall("{call creaNum " + utente + "}");
		num.executeUpdate();
		num.close();
		numc = con.prepareCall("{call creaNumc " + utente + "}");
		numc.executeUpdate();
		numc.close();
	} catch(SQLException ex) {
		throw ex;
	} finally {
		con.setAutoCommit(true);
		//if (num != null)
		//	num.close();
		//if (numc != null)
		//	numc.close();
	}
}


/**
* Estrae la "Average Mutual Information" delle parole presenti nei documenti<br>
* di training appartenenti alla categoria <li>cat</li>specificata.
*/ 
public Hashtable extractAMI(String utente, String cat, int totPag, int numDoc)
									throws SQLException {
	con.setAutoCommit(false);
	Hashtable result = new Hashtable();	
	int cdms = ((Integer)getConfiguration(utente).elementAt(5)).intValue();
	CallableStatement numc2 = null;
	CallableStatement ami = null;
	CallableStatement agg = null;
	CallableStatement Dnumc2 = null;
	ResultSet rs = null;
	try {
		int docPerCat = getTrainingPages(utente, cat).size();
		int totDoc = getTrainingPages(utente).size();	
		double probCat = (double)docPerCat/(double)totDoc;
System.out.println("Probabilit di Categoria: " + probCat); 		
		numc2 = con.prepareCall("{call creaNumc2 " + cat + ", " + utente + "}");
		numc2.executeUpdate();
		numc2.close();
		ami = con.prepareCall("{call ami " + probCat + ", " + totPag + ", " + utente + "}");
		rs = ami.executeQuery();
		int i = 0;
		while((rs.next()) && (i<cdms))
			result.put(rs.getString(2), new Double(rs.getDouble(1)));
		rs.close();
		ami.close();
		agg = con.prepareCall("{call aggiornaTokenVisti " + cat + "}");
		agg.executeUpdate();
		agg.close();	
		Dnumc2 = con.prepareCall("{call DropNumc2 }");
		Dnumc2.executeUpdate();
		Dnumc2.close(); 
		con.commit();
	} catch (SQLException ex) {
		con.rollback();
	} finally {
		//if (numc2 != null)
		//	numc2.close();
		//if (rs != null)
		//	rs.close();
		//if (ami != null)
		//	ami.close();
		//if (agg != null)
		//	agg.close();
		//if (Dnumc2 != null)
		//	Dnumc2.close();
		con.setAutoCommit(true);
		return result;
	}
}

/**
* Ripristina il database del sistema dopo l'estrazione della feature<br>
* secondo il metodo Average Mutual Information.
*/ 
public void commitAMI() throws SQLException {
	con.setAutoCommit(false);
	CallableStatement Dnum = null;
	CallableStatement	Dnumc = null;
	try {
		Dnum = con.prepareCall("{call DropNum }");
		Dnum.executeUpdate();
		Dnum.close();
		Dnumc = con.prepareCall("{call DropNumc }");
		Dnumc.executeUpdate();
		Dnumc.close();
	} catch(SQLException ex) {
		//if (Dnum != null)
		//	Dnum.close();
		//if (Dnumc != null)
		//	Dnumc.close();
		throw ex;
	} finally {
		con.setAutoCommit(true);
	}
}

private boolean existStopword(String utente, String parola) throws SQLException {
	boolean exist = false;
	String query = "SELECT parolaNonValida FROM stopword WHERE login = '" +
		utente + "' AND parolaNonValida = '" + parola + "'";
	Statement st = null;
	ResultSet rs = null;
	try {
		st = con.createStatement();
		rs = st.executeQuery(query);
		exist = rs.next();
	} catch(SQLException ex) {
		throw ex;
	} finally {
		if (rs != null)	
			rs.close();
		if (st != null)
			st.close();
		return exist;
	}
}

private void putStopword(String utente, String parola) throws SQLException{
	String query = "INSERT INTO stopword (login, parolaNonValida) VALUES ('" +
		utente + "', '" + parola + "')"; 
	Statement st = null;
	st = con.createStatement();
	try {
		st.executeUpdate(query);
	} catch(SQLException ex) {
		throw ex;
	} finally {
		if (st != null)
			st.close();
	}
}

/**
* Inserisce il vettore di stopword relativo all'utente specificato.
*/ 
public void insertStopwords(String utente, Vector stopwords) throws SQLException{
	con.setAutoCommit(false);
	try {
		for (int i = 0;i < stopwords.size();i++) {
			String parola = (String ) stopwords.elementAt(i);
			if (!existStopword(utente, parola)) 
				putStopword(utente, parola);
		}
	} catch(SQLException ex) {
		con.rollback();
		throw ex;
	} finally {
		con.setAutoCommit(true);
	}
}

/**
* Inserisce la stopword relativa all'utente specificato.
*/ 
public void insertStopword(String utente, String parola) throws SQLException{
	con.setAutoCommit(false);
	try {
		if (!existStopword(utente, parola)) 
			putStopword(utente, parola);
	} catch(SQLException ex) {
		con.rollback();
		throw ex;
	} finally {
		con.setAutoCommit(true);
	}
}

/** 
* Restituisce il tipo dell'utente specificato come parametro:<br>
* <li>AMMINISTRATORE</li> oppure <li>FINALE</li>.
*/

public String getUserType(String utente, String passw) throws SQLException {
	String result = "";
	String query = "SELECT tipoUtente FROM utente WHERE login = '" + utente + "' AND " +
		"password = '" + passw + "'";
	Statement stmt = con.createStatement();
	ResultSet rs = stmt.executeQuery(query);
	if (rs.next()) 
		result = rs.getString(1);
	rs.close();
	stmt.close();
	return result;
	
}

/**
 * Restituisce il titolo del documento identificato da <code>idDoc</code>.
 */
public String getTitle(int idDoc) throws SQLException {

	String result = "No Title";
	String query = "SELECT titolo FROM documento WHERE idDocumento = " + idDoc;
	Statement stmt = con.createStatement();
	ResultSet rs = stmt.executeQuery(query);
	if (rs.next()) 
		result = rs.getString(1);
	rs.close();
	stmt.close();
	String c = result.substring(0, 1);
	result = c.toUpperCase() + result.substring(1, result.length());
	return result.replace('_', ' ');
}

	/**
       * Restituisce un vettore contenete le categorie 
       * associate alla query.
       */
	public synchronized Vector getQueryCategories(String utente, String nomeQuery) throws SQLException {

		PreparedStatement ps = con.prepareStatement("SELECT categoria FROM attinenza WHERE login = ? AND query = ?");
		ps.setString(1, utente);
		ps.setString(2, nomeQuery);
		ResultSet rs = ps.executeQuery();
		Vector ris = new Vector();
		while (rs.next())
			ris.addElement(rs.getString(1));
		rs.close();
		ps.close();
		return ris;
	}

	/**
       * Restituisce tutti i documenti che contengono la parola
       * <code>token</code>.
       */
	public synchronized Vector getDocumentsWith(String token, String utente) throws SQLException {

		PreparedStatement ps = con.prepareStatement("SELECT appartenenza.idDocumento " +
													"FROM composizione INNER JOIN appartenenza ON (composizione.idDocumento = appartenenza.idDocumento) " + 
		                                            "WHERE composizione.token = ? AND appartenenza.login = ?");
		ps.setString(1, token);
		ps.setString(2, utente);
		ResultSet rs = ps.executeQuery();
		Vector ris = new Vector();
		while (rs.next())
			ris.addElement(""+rs.getInt(1));
		rs.close();
		ps.close();
		return ris;
	}


	/**
	* Sostituisce il nome <li>oldCategory</li> di un categoria con il nuovo nome 
	* <li>newCategory</li>.
	*/
	public void replaceCategory(String utente, String oldCategory, String newCategory)
			throws SQLException {
		String query = "UPDATE categoria SET nomeCategoria = '" + newCategory + "'" +
			" WHERE ((nomeCategoria = '" + oldCategory + "') AND (login = '" + 
			utente + "'))";
		String query1 = "UPDATE feature SET categoria = '" + newCategory + "'" +
			" WHERE ((categoria = '" + oldCategory + "') AND (login = '" + 
			utente + "'))";
		Statement stmt = null;
		Statement stmt1 = null;
		try {
			con.setAutoCommit(false);
			stmt = con.createStatement();
			stmt.executeUpdate(query);
			stmt1 = con.createStatement();
			stmt1.executeUpdate(query1);
		} catch(SQLException ex) {
			con.rollback();
			throw ex;
		} finally {
			stmt.close();
			stmt1.close();
			con.setAutoCommit(true);
		}
	}

	/**
	* Restituisce l'id massimo dei documenti di testing di un utente.
	*/	
	public int getMaxIdDoc(String utente) throws SQLException {
		String query = "SELECT Max(idDocumento) FROM appartenenza " + 
			"GROUP BY login, tipo " +
			"HAVING ((login = '" + utente + "') AND (tipo = 'TESTING'))";
		Statement stmt = con.createStatement();
		ResultSet rs = stmt.executeQuery(query);
		int result = -1;
		if (rs.next())
			result = rs.getInt(1);
		rs.close();
		stmt.close();
		return result;
	}
	
	/**
	* Restituisce l'id minimo dei documenti di testing di un utente.
	*/	
	public int getMinIdDoc(String utente) throws SQLException {
		String query = "SELECT Min(idDocumento) FROM appartenenza " + 
			"GROUP BY login, tipo " +
			"HAVING ((login = '" + utente + "') AND (tipo = 'TESTING'))";
		Statement stmt = con.createStatement();
		ResultSet rs = stmt.executeQuery(query);
		int result = -1;
		if (rs.next())
			result = rs.getInt(1);
		rs.close();
		stmt.close();
		return result;
	}
	
	
	public Hashtable getEnphasis() throws SQLException {
		
		String query = "SELECT tagHTML, coefficiente FROM tagHTML";
		Statement stmt = con.createStatement();
		ResultSet rs = stmt.executeQuery(query);
		Hashtable ht = new Hashtable();
		while (rs.next())
			ht.put(rs.getString(1), new Integer(rs.getInt(2)));
		rs.close();
		stmt.close();
		return ht;
	}
	
	public void setEnfRepresentation(String utente) throws SQLException {
		Vector allDoc = new Vector();
		Vector result = new Vector();
		allDoc = getAllDocuments(utente);
		CallableStatement enfcs = null;
		PreparedStatement ps = null;
		ResultSet rs2 = null;	
		String query = "UPDATE composizione SET enfasiAdditiva = ? " +
			", enfasiMoltiplicativa = ? WHERE idDocumento = ? AND token = ?";
		try {
			con.setAutoCommit(false);
			ps = con.prepareStatement(query);
			for (int i = 0; i < allDoc.size(); i++) {
				StringTokenizer st = new StringTokenizer((String)allDoc.elementAt(i));
				int idDoc = (new Integer(st.nextToken())).intValue();
				Enumeration en = getWords(idDoc);
				while (en.hasMoreElements()) {
					HtmlWord token = (HtmlWord)en.nextElement();	
					double tf = token.getTF();
					enfcs = con.prepareCall("{call EnfasiTagToken " + idDoc + ", '" + token.getText() + "'}");
					rs2 = enfcs.executeQuery();
					double add = 0;
					double mol = 1;
					double peso = 0;
					while (rs2.next()) {
						peso = rs2.getDouble(1);
						add += peso;
						mol += peso;
					}
					rs2.close();
					enfcs.close();	
					ps.setDouble(1, (tf + add));
					ps.setDouble(2, (tf*(1 + mol)));
					ps.setInt(3, idDoc);
					ps.setString(4, token.getText());
					System.out.println((i+1) + " " + idDoc + " " + token.getText() + " " + (tf + add) + " " + (tf*(1 + mol)));
					ps.executeUpdate();
				}
			}
		ps.close();
		} catch(SQLException ex) {
			con.rollback();
			throw ex;
		} finally {
			con.setAutoCommit(true);
		}
	}
}