package uniba.gui;

import java.io.*;
import java.awt.*;
import java.util.*;
import uniba.core.*;
import uniba.databanker.*;

/**
 * Permette la visualizzazione delle parole estratte
 * da una pagina HTML recuperata dalla rete o da file.
 */
public class BrowserDialog extends Dialog {

  Frame                   father;
  boolean                 isClearPage;
  TextField               urlName;
  FileDialog              openFile;
  TextArea                textPage;
  java.awt.List           wordList;
  Button                  connectButton;
  Button                  openFileButton;
  Button                  cancelButton;
  Button                  clearButton;
  Button			  viewButton;	
  Panel                   buttonPanel;
  Panel                   centerPanel;
  Panel			   urlPanel;
  Label                   urlLabel;
  Label                   wordsLabel;
  Vector                  conf = null;
  Vector                  stopword = null;
  Enumeration 		  parole = null;	
  HtmlPage                currentHtmlPage; 
  int 			  totParole;
  Databanker db = null;
    
  
  /** 
   Costruttore dell'oggetto BrowserDialog. Crea la GUI.
  */     
  
  public BrowserDialog(Frame f, Vector conf, Vector stopword, Databanker db) {
   super(f,"Browsing Web",true);
   isClearPage=true;
   setBackground(new Color(0).lightGray);  
   this.conf = conf;
   this.stopword = stopword;
   this.db = db;
   connectButton = new Button("Connect");
   cancelButton = new Button("Cancel");
   clearButton = new Button("Clear");
   viewButton = new Button("View HTML Page");
   openFileButton = new Button("Open file");
   openFile = new FileDialog(f,"Open File",openFile.LOAD);
   openFile.setFile("*.htm");
   urlLabel = new Label("URL:");
   wordsLabel = new Label("HTML Tagged Words");
   urlName = new TextField("http://", 60);
   urlName.setForeground(new Color(0).black);
   urlName.setBackground(new Color(0).white);
   textPage = new TextArea();
   textPage.setForeground(new Color(0).black);
   textPage.setBackground(new Color(0).white);
   textPage.setEditable(false);
   textPage.setFont(new Font("Courier",0,12));
   wordList = new java.awt.List(10,true);
   wordList.setForeground(new Color(0).black);
   wordList.setBackground(new Color(0).white);
   wordList.setMultipleSelections(false);
   wordList.setFont(new Font("Courier",0,12));
   buttonPanel= new Panel();
	buttonPanel.add(clearButton);
	buttonPanel.add(cancelButton);
	urlPanel = new Panel();
	urlPanel.add(openFileButton);
	urlPanel.add(urlLabel);
	urlPanel.add(urlName);
	urlPanel.add(connectButton);
      urlPanel.add(viewButton);      
   Panel wordListPanel = new Panel();
	wordListPanel.setLayout(new BorderLayout());
	wordListPanel.add("North",wordsLabel);
	wordListPanel.add("Center",wordList);
   centerPanel = new Panel();
	centerPanel.setLayout(new GridLayout(1,2));
	centerPanel.add(textPage);
	centerPanel.add(wordListPanel);
   add("North",urlPanel);
   add("Center",centerPanel);
   add("South",buttonPanel);
   viewButton.setEnabled(false);	
   connectButton.setEnabled(false);
   resize(800,400);
   father=f;
   Toolkit t = father.getToolkit();
   int scw   = t.getScreenSize().width;
   int sch   = t.getScreenSize().height; 
   int w     = this.size().width;
   int h     = this.size().height;
   setLocation((scw/2)-(w/2), (sch/2)-(h/2));
 }  


  public void clearPage() {
	urlName.setText("http://");
	textPage.setText("");
	wordList.clear();
	isClearPage=true;
  }  

//----------------------------------------------------------------------------------------

  public boolean handleEvent(Event evt) {
	if (evt.id==evt.WINDOW_DESTROY) {
	  clearPage();
	  hide();
	  return true;
	}
	else
	if (evt.id == evt.KEY_PRESS) {
		if (evt.target.equals(urlName)) {
			if (urlName.getText().length() > 10) {
				connectButton.setEnabled(true);
				viewButton.setEnabled(false);
			} else {
				connectButton.setEnabled(false);
				viewButton.setEnabled(false);
			}
			if (evt.key == evt.DELETE) {
				wordsLabel.setText("HTML Tagged Words");
				textPage.setText("");
			 	wordList.clear();
			 	isClearPage=true;
				connectButton.setEnabled(false);
				viewButton.setEnabled(false);
			}
		}
		return super.handleEvent(evt);
	} else
	if (evt.id==evt.ACTION_EVENT) {

	 if (evt.target instanceof Button) {
		if (evt.arg.equals("Connect")) {
		  try {
			 setCursor(new Cursor(Cursor.WAIT_CURSOR));	
			 loadPage(urlName.getText());
			 connectButton.setEnabled(false);
			 viewButton.setEnabled(true);
              } catch (Exception e) {  
                      new StampaErrore(father,
                                       "BrowseDialog",
				      	   "Exception:\n" + e.getMessage());
                      e.printStackTrace();
		  } finally {
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		  }
		   	
		  return true;
		}
		else
            if (evt.arg.equals("View HTML Page")) {
                try {
                    new Explorer(urlName.getText(), 4);
                    setEnabled(true);
                }
                catch (UnsatisfiedLinkError ule) {
                     new StampaErrore(father, 
                                      "BrowseDialog", 
                                      "Impossibile aprire Internet Explorer:\n" +
                                      "per utilizzare questa funzione, compilare\n" +
                                      "ed eseguire WebClass con i tools\n" +
                                      "della Microsoft per Java.");
                                           
                }            
                return true;
            }
            else   
		if (evt.arg.equals("Clear")) {
		  clearPage();
		  wordsLabel.setText("HTML Tagged Words");
		  connectButton.setEnabled(false);
		  viewButton.setEnabled(false);
		  return true;
		}
		else
		if (evt.arg.equals("Cancel")) {
		  clearPage();
		  hide();
		  father.enable();
		  return true;
		}
		else
		if (evt.arg.equals("Open file")) {
		  openFile.setDirectory("./html/");	
		  openFile.setFile("*.htm");
		  openFile.show();
		  if (openFile.getFile() != null) {
		   clearPage();
		   String fileName;
		   fileName=openFile.getDirectory()+openFile.getFile();
		   String fileUrl;
		   fileUrl="file://"+fileName.replace('\\','/');
		   urlName.setText(fileUrl);
		   wordList.clear();
		   wordsLabel.setText("HTML Tagged Words");  
		   connectButton.setEnabled(true);
		   viewButton.setEnabled(true);
		  
		  }
		  return true;
		}
	  }  
	  else  
	  if (evt.target instanceof java.awt.List) {  
		if  (evt.target.equals(wordList)) {
		
		}
	  }
	}
	return false;
  }
  
//--------------------------------------------------------------------------------------------

  public void loadPage(String url) throws Exception {
	System.out.println("URL = " + url);
	if (url != null) {
	  urlName.setText(url);
	  currentHtmlPage = new HtmlPage(url, conf, stopword, db); 
	  textPage.setText(currentHtmlPage.getSource());
	  isClearPage=false;
	  HtmlWord currentWord;
	  parole = currentHtmlPage.getWords();
        Enumeration e = currentHtmlPage.getWords();
        totParole = currentHtmlPage.getNumWords();
	  while (e.hasMoreElements()) {
		  currentWord = (HtmlWord)e.nextElement();
		  String att;
		  String spazi="                                  ";
		  String tagFlag=spazi;
              
		  if (currentWord.getAttributes() != null) {
			att=currentWord.getAttributes();
                  tagFlag=new String(att + spazi.substring(0,spazi.length()-att.length()));
              }
              	     
		  wordList.addItem(tagFlag+"   "+currentWord.getText() + "  " + currentWord.getOcc());
	  }
	  wordsLabel.setText("HTML Tagged Words ( Total Words: " + totParole + " )"); 
	  
     }  
  }

}