package uniba.gui;

import java.awt.*;
import java.sql.SQLException;
import java.awt.event.*;
import java.util.Vector;
import java.util.StringTokenizer;
import uniba.databanker.*;
import uniba.core.*;

/**
 * Permette la visualizzazione dei documenti
 * di un utente
 */
public class DocumentDialog extends Dialog implements ItemListener, MouseListener {

	private List documenti = new List(15, false);
	private Label numDoc = null;
	private Vector doc = null;
	private Button cancella = new Button("Cancel");
	private Button properties = new Button("Properties");
	private Button view = new Button("View Document");
	private String urlDoc[] = null;
	private int idDoc[] = null;
	private Frame parent = null;	
	private Databanker db = null;
	private TrainingPagesManager trpm = null;
	private TestingPagesManager tspm = null;
	private RejectPagesManager rjpm = null;
	private String utente = null;
		

	public DocumentDialog(Frame p, 
				    Vector doc, 
				    Databanker db, 
				    TrainingPagesManager trpm,
				    TestingPagesManager tspm,
				    RejectPagesManager rjpm,
				    String utente) {

		super(p, "View All Documents", true);
		this.doc = doc;
            this.parent = p;
		this.db = db;
		this.trpm = trpm;
		this.tspm = tspm;
		this.rjpm = rjpm;
		this.utente = utente;
		GridBagLayout gridbag = new GridBagLayout();
            GridBagConstraints constraints = new GridBagConstraints();
            setLayout(gridbag);
		setBackground(Color.lightGray);
		
		addWindowListener( new WindowAdapter() {
						public void windowClosing(WindowEvent we) {
                                          urlDoc = null;
							idDoc = null;
							setVisible(false);
						}
					 });
					
		for (int j=0; j<doc.size(); j++) 
			documenti.addItem((String)doc.elementAt(j));
		            
		buildConstraints(constraints, 0, 0, 3, 1, 100, 50, GridBagConstraints.BOTH, GridBagConstraints.CENTER);
            gridbag.setConstraints(documenti, constraints);
		add(documenti);
		documenti.addItemListener(this);

		numDoc = new Label("Number of Documents: " + doc.size());
		buildConstraints(constraints, 0, 1, 3, 1, 100, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
            gridbag.setConstraints(numDoc, constraints);
		add(numDoc);
		
		
		buildConstraints(constraints, 0, 2, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.CENTER);
            gridbag.setConstraints(view, constraints);
		add(view); 
		view.setEnabled(false);
		view.addMouseListener(this);

		buildConstraints(constraints, 1, 2, 1, 1, 40, 20, GridBagConstraints.NONE, GridBagConstraints.CENTER);
            gridbag.setConstraints(properties, constraints);
		add(properties); 
		properties.setEnabled(false);
		properties.addMouseListener(this);

		cancella.addMouseListener(new MouseAdapter() {
							public void mouseClicked(MouseEvent me) {
								urlDoc = null;
								idDoc = null;
								setVisible(false);
							}
						  });
		buildConstraints(constraints, 2, 2, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.CENTER);
            gridbag.setConstraints(cancella, constraints);
		add(cancella);

		setSize(600, 300);
            Toolkit t = parent.getToolkit();
            int scw   = t.getScreenSize().width;
	      int sch   = t.getScreenSize().height;
	      int w     = this.size().width;
	      int h     = this.size().height;
	      setLocation((scw/2)-(w/2), (sch/2)-(h/2));	
            setResizable(false);
            setVisible(true);

	}

	private void buildConstraints(GridBagConstraints gbc, int gx,
           int gy, int gw, int gh, int wx, int wy,
           int fill, int anchor) {
             
             gbc.gridx = gx;
             gbc.gridy = gy;
             gbc.gridwidth = gw;
             gbc.gridheight = gh;
             gbc.weightx = wx;
             gbc.weighty = wy;
             gbc.fill = fill;
             gbc.anchor = anchor;
        }

	  
	  public int[] getIdDoc() {
		return idDoc;
	  }	

	  public String[]  getUrlDoc() {
		return urlDoc;
	  }	

	  public void itemStateChanged(ItemEvent ie) {
		int sel[] = documenti.getSelectedIndexes();
		if (sel.length == 1) {
			view.setEnabled(true);
			properties.setEnabled(true);
            } else {
			view.setEnabled(false);
			properties.setEnabled(false);
		}
	  } 

	public void mouseClicked(MouseEvent e) {
		Button source = (Button)e.getSource();
		if (source.equals(view)) {
			String host = null;
		      String porta = null;		
		      String url = null;	
		      try {	
		     		host = (String)db.getConfiguration(utente).elementAt(12);	
		     		porta = (String)db.getConfiguration(utente).elementAt(13);	
				url = "http://" + host + ":" + porta + "/";
		      } catch (Exception se) {
				new StampaErrore(parent, "View All Documents", se.getMessage());
		      }	
			if (url != null) {
				url = url + documenti.getSelectedItem();
				try {
					new Explorer(url, 4);
            		} catch (UnsatisfiedLinkError ule) {
             			new StampaErrore(parent, "ViewDocuments", "Impossibile aprire Internet Explorer:\n" +
                           						"per utilizzare questa funzione, compilare\n" +
                           						"ed eseguire WebClass con i tools\n" +
                           						"della Microsoft per Java.");
				}
			}
		} else {
		   try {
			StringTokenizer st = new StringTokenizer(documenti.getSelectedItem());
			int id = new Integer(st.nextToken()).intValue();
			String cat = db.getCategory(id);
			String type = db.getType(id);
			Vector prop = new Vector();
			prop.addElement(new Integer(id));
			prop.addElement(db.getUrl(id));
			prop.addElement(cat);
			if (type != null)
				prop.addElement(type);
			else
				prop.addElement("");
			PropertiesDialog pd = new PropertiesDialog(parent, prop, db, trpm, tspm, rjpm);
		   } catch(Exception ee) {
			new StampaErrore(parent, "ViewDocuments", ee.getMessage());
			ee.printStackTrace();
		   }

		}
	}

		

	public void mousePressed(MouseEvent e) {}

        public void mouseReleased(MouseEvent e) {}

        public void mouseEntered(MouseEvent e) {}
 
        public void mouseExited(MouseEvent e) {}



}
		
		
		

	