package uniba.gui;

import java.awt.*;
import java.util.*;
import java.sql.SQLException;
import java.sql.Time;	
import uniba.core.*;
import uniba.databanker.*;

/**
 * Permette l'estrazione delle feature
 */
public class ExtractFeaturesDialog extends Dialog {

  private Frame parent;  //GV- Owner of dialog.
  private Button extractButton;
  private Button extractMIButton;
  private Button extractORButton;
  private Button dictButton;
  private Button clearButton;
  private Button cancelButton;
  private Checkbox enStemm;
  private Checkbox enStopwords;
  private Checkbox enHTMLEmphasis;
  private TextField totWords;
  private Checkbox TFIDF;
  private Checkbox mostCommon;
  private Checkbox lessCommon;
  private Checkbox random;
  private TextField totTFIDF;
  private TextField totMostCommon;
  private TextField totLessCommon;
  private TextField totRandom;
  private java.awt.List featuresList;
  private java.awt.List categoriesList;
  private java.awt.List dictionaryList;

  private Frame progressWindow;  
  private FeaturesManager ftm;
  private CategoriesManager cgm;
  private Databanker db;
  
  private String currentCategory;
  private boolean MI = false;
  private boolean OR = false;
  private StampaErrore2 se;
  private String utente = null;
  private String titolo = "Extract features from training pages";
  private String report = "";
  private Time inizio;
  private Time fine;			
  
  
  public ExtractFeaturesDialog (Frame parent, 
                                FeaturesManager ftm,
                                CategoriesManager cgm, 
                                Frame progressWindow, 
                                Databanker db,
                                String utente) {
	super(parent, "Extract features from training pages", true);
	this.parent = parent;
      this.db = db;
      this.utente = utente;
	try {
		String modo = db.getFeatureMode(utente);
		if (modo != null) {
			if (modo.equals(db.MAXTF))	
				modo = " - (Feature extracted by MaxTF-PF^2-ICF)";
			else if (modo.equals(db.AMI))
				modo = " - (Feature extracted by Average Mutal Information)";
			else if (modo.equals(db.ODDS))
				modo = " - (Feature extracted by Odds Ratio)";
			else
				modo = " (No Feature Extracted)";
		} else
			modo = " (No Feature Extracted)";
		setTitle(titolo + modo);
	} catch (SQLException se) {
		new StampaErrore(parent, "ExtractFeaturesDialog", se.getMessage());
	}
	
	setBackground(new Color(0).lightGray);
	extractButton=new Button("Extract features");
	extractButton.disable();  //GV- enabled after category words extraction.
	clearButton=new Button("Clear features"); 
      try { 
          clearButton.setEnabled(db.existFeature(utente));
      } catch (Exception e) {}
      
	extractMIButton=new Button("Extract features by AMI");
	extractORButton=new Button("Extract features by OR");
	dictButton=new Button("Extract category words");
	cancelButton=new Button("Cancel");

	Panel buttonPanel = new Panel();
	buttonPanel.add(dictButton); 
	buttonPanel.add(extractButton);
	buttonPanel.add(extractMIButton);
	buttonPanel.add(extractORButton);
	buttonPanel.add(clearButton);
	buttonPanel.add(cancelButton);
	
	featuresList = new java.awt.List(12,false);
	featuresList.setForeground(new Color(0).black);
	featuresList.setBackground(new Color(0).white);
	featuresList.setFont(new Font("Courier",0,12));
	
	Label featuresLabel = new Label("Features");
	Panel listPanel = new Panel();
	listPanel.setLayout(new BorderLayout());
	listPanel.add("North",featuresLabel);
	listPanel.add("Center",featuresList);
	
	enStemm =        new Checkbox("Enable stemming");
	enStopwords =    new Checkbox("Enable stopwords");
	enHTMLEmphasis = new Checkbox("Enable HTML emphasis");
	totWords = new TextField("100",4);
	TFIDF = new Checkbox("TFIDF ");
	TFIDF.setState(true);
	mostCommon = new Checkbox("Most common words ");
	lessCommon = new Checkbox("Less common words ");
	random     = new Checkbox("Random words ");
	totTFIDF   = new TextField("100");
	totMostCommon = new TextField("0");
	totMostCommon.disable();
	totLessCommon = new TextField("0");
	totLessCommon.disable();
	totRandom     = new TextField("0");
	totRandom.disable();
	
	Panel checkPanel = new Panel();
	checkPanel.setLayout(new GridLayout(8,1));
	checkPanel.add(enStemm);
	checkPanel.add(enStopwords);
	checkPanel.add(enHTMLEmphasis);     
	Panel totWordsPanel = new Panel();
	 totWordsPanel.setLayout(new GridLayout(1,2));
	 totWordsPanel.add(new Label("Total words: "));
	 totWordsPanel.add(totWords);
	checkPanel.add(totWordsPanel);     
	Panel tfidfPanel = new Panel();
	 tfidfPanel.setLayout(new GridLayout(1,2));
	 tfidfPanel.add(TFIDF);
	 tfidfPanel.add(totTFIDF);
	checkPanel.add(tfidfPanel);      
	Panel mostPanel = new Panel();
	 mostPanel.setLayout(new GridLayout(1,2));
	 mostPanel.add(mostCommon);
	 mostPanel.add(totMostCommon);
	checkPanel.add(mostPanel);      
	Panel lessPanel = new Panel();
	 lessPanel.setLayout(new GridLayout(1,2));
	 lessPanel.add(lessCommon);
	 lessPanel.add(totLessCommon);
	checkPanel.add(lessPanel);      
	Panel randomPanel = new Panel();
	 randomPanel.setLayout(new GridLayout(1,2));
	 randomPanel.add(random);
	 randomPanel.add(totRandom);
	checkPanel.add(randomPanel);      

	Panel categoriesPanel = new Panel();
	 categoriesPanel.setLayout(new BorderLayout());
	 categoriesList = new java.awt.List(10,false);
	 categoriesList.setForeground(new Color(0).black);
	 categoriesList.setBackground(new Color(0).white);
	 categoriesList.setFont(new Font("Courier",0,12));
	 categoriesPanel.add("North",new Label("Categories"));  
	 categoriesPanel.add("Center",categoriesList);
	
	Panel dictionaryPanel = new Panel();
	 dictionaryPanel.setLayout(new BorderLayout());
	 dictionaryList = new java.awt.List(10,false);
	 dictionaryList.setForeground(new Color(0).black);
	 dictionaryList.setBackground(new Color(0).white);
	 dictionaryList.setFont(new Font("Courier",0,12));
	 dictionaryPanel.add("North",new Label("Category words"));  
	 dictionaryPanel.add("Center",dictionaryList);
	
	Panel centerPanel = new Panel();
	centerPanel.setLayout(new GridLayout(1,4));

	//centerPanel.add(checkPanel);
	centerPanel.add(categoriesPanel);
	centerPanel.add(dictionaryPanel);
	centerPanel.add(listPanel);

	Panel northPanel = new Panel();
	northPanel.add(new Label(""));  


	add("North",northPanel);
	add("Center",centerPanel);
	add("South",buttonPanel);
	resize(800,550);
	
	
	this.ftm=ftm;
	this.cgm=cgm;
	  this.progressWindow=progressWindow;
	setupFeaturesList();
	setupCategoriesList();
      Toolkit t = parent.getToolkit();
      int scw   = t.getScreenSize().width;
      int sch   = t.getScreenSize().height;
      int w     = this.size().width;
      int h     = this.size().height;
      setLocation((scw/2)-(w/2), (sch/2)-(h/2));
      setResizable(false);
      setVisible(true);

  }  

  /**
   * Gestione degli eventi.
   */	
  public boolean handleEvent(Event evt) {
	if (evt.id==evt.WINDOW_DESTROY) {
	  hide();
	  return true;
	}
	else
	if (evt.id==evt.ACTION_EVENT) {
	  if (evt.target instanceof Button) {
		if (evt.arg.equals("Extract features")) {
		  progressWindow.show();
		  try {
			inizio = new Time(System.currentTimeMillis());
                  if (!clearButton.isEnabled())	{ 
		  		ftm.extract(progressWindow);
                        clearButton.enable();  
		      	setupFeaturesList();
				String modo = " - (Feature extracted by MaxTF-PF^2-ICF)";
				setTitle(titolo + modo);
		      } else { new StampaErrore(parent, "ExtractFeaturesDialog", "Cancellare prima le feature\nprecedentemente estratte"); }	
		  } catch (Exception s) {se = new StampaErrore2(parent, "extract", "Exception:			\n"+s.getMessage(), db);if (!se.validato()) {setVisible(false);}}
              finally {
			fine = new Time(System.currentTimeMillis());
			report = "Extract features\n\tinizio: " + inizio.toString() + " fine: " + 
				fine.toString();
			System.out.println(report);
                  progressWindow.hide();
              }
		  report += "\nExtract features\n" + 	"inizio: " + inizio.toString() +  
									"fine: " + fine.toString();	
		  return true;
		}
		else
		if (evt.arg.equals("Extract features by AMI")) {
		  progressWindow.show();
		  try {
			db.getAutoma().Transizione("ef2");
			inizio = new Time(System.currentTimeMillis());
                  if (!clearButton.isEnabled())	{ 
           	      	ftm.extractFeaturesMI2(progressWindow);
				MI = true;
		  		OR = false;
				clearButton.enable();  //GV- enable "Clear features" button
                        dictionaryList.clear();
		  		setupFeaturesList();
				String modo = " - (Feature extracted by Average Mutal Information)";
				setTitle(titolo + modo);
		  	} else {new StampaErrore(parent, "ExtractFeaturesDialog",
				"Cancellare prima le feature\nprecedentemente estratte"); }	
		  } catch (Exception s) {se = new StampaErrore2(parent, "extract",
			"Exception:\n"+s.getMessage(), db);if (!se.validato()) {setVisible(false);}}
                finally {
			fine = new Time(System.currentTimeMillis());
			report = "Extract features by AMI\n\tinizio: " + inizio.toString() + " fine: " + 
				fine.toString();
			System.out.println(report);
                  progressWindow.hide();
               } 

		  return true;
		}
		if (evt.arg.equals("Extract features by OR")) {
		  progressWindow.show();
		  try {
			db.getAutoma().Transizione("ef3");
			inizio = new Time(System.currentTimeMillis());
                  if (!clearButton.isEnabled())	{ 
	      		ftm.extractFeaturesOR(progressWindow);
				OR = true;
		  		MI = false;
				clearButton.enable();  //GV- enable "Clear features" button
                        dictionaryList.clear(); 
		  		setupFeaturesList();
				String modo = " - (Feature extracted by Odds Ratio)";
                        setTitle(titolo + modo);
		  	} else { new StampaErrore(parent, "ExtractFeaturesDialog", "Cancellare prima le feature\nprecedentemente estratte"); } 	
		  } catch (Exception s) {se = new StampaErrore2(parent, "extract", "Exception:			\n"+s.getMessage(), db);if (!se.validato()) {setVisible(false);}}
               finally {
			fine = new Time(System.currentTimeMillis());
			report = "Extract features by Odds Ratio\n\tinizio: " + 
				inizio.toString() + " fine: " + fine.toString();
			System.out.println(report);
                  progressWindow.hide();
               } 
		   return true;
		}
		else
		if (evt.arg.equals("Cancel")) {
		  hide();
		  return true;
		}
		if (evt.arg.equals("Clear features")) {
		  
              try {
		     progressWindow.show();	
                 new DeleteFeature(utente, db, parent, progressWindow);
                 clearButton.setEnabled(false);
                 ftm.removeFeature();
                 featuresList.clear();
                 dictionaryList.clear();
		     setTitle(titolo + " (No Feature Extracted)");	
               } catch (Exception e) {
                   new StampaErrore(parent, 
                                    "ExtractFeatureDialog",
                                    "Exception:\n" + e.getMessage());
              }
		  return true;
		}
		if (evt.arg.equals("Extract category words")) {
		   progressWindow.show();
		   try {
			 inizio = new Time(System.currentTimeMillis());
                   if (!clearButton.isEnabled()) { 
				db.getAutoma().Transizione("ef1");
		       	ftm.extractDictionaries(progressWindow);
                   	OR = false;
                   	MI = false;
		       	extractButton.enable();  //GV- enable "Extract features" button
                        featuresList.clear();
		            dictionaryList.clear();
              	 } else { new StampaErrore(parent, "ExtractFeaturesDialog", "Cancellare prima 					le feature\nprecedentemente estratte"); } 
		  } catch (Exception s) {se = new StampaErrore2(parent, "extract", "Exception:			\n"+s.getMessage(), db);s.printStackTrace();
							if (!se.validato()) {setVisible(false);}}
              finally {
			fine = new Time(System.currentTimeMillis());
			report = "Extract category words\n\tinizio: " + 
				inizio.toString() + " fine: " + fine.toString();
			System.out.println(report);
                  progressWindow.hide();
              } 
 
		  return true;
		}
		
	  }
	}
    if (evt.id==evt.LIST_SELECT) {
	  if (evt.target==categoriesList) {
		java.awt.List t = (java.awt.List) evt.target;
		currentCategory=t.getSelectedItem();

		setupDictionaryList(utente, currentCategory);

		// System.out.println("Current category is: '"+currentCategory+"'");
		return true;
	  }
	}
	return false;
  }  

    
  private void setupCategoriesList() {
	categoriesList.clear();
	Enumeration e = cgm.listCategories();
	String c;
	while (e.hasMoreElements())
	 {
		c=(String) e.nextElement();
		if (!c.equals("UNKNOWN")) categoriesList.addItem(c);
	 } 
  }  


  private void setupDictionaryList(String utente, String category) {
	dictionaryList.clear();
	try {
		Enumeration e = db.listDictionary(utente, category);
      	String w;
		String spazi = "                                                                 ";
		while(e.hasMoreElements()) {
			String line = "";
	   		w = (String)e.nextElement();
			StringTokenizer st = new StringTokenizer(w);
			line = (st.nextToken() + spazi).substring(0,15) + st.nextToken();
			dictionaryList.addItem(line);
      	}
	} catch(Exception e) {
		new StampaErrore(parent, 
                             "ExtractFeatureDialog",
                             "Exception:\n" + e.getMessage());
      }
		
  }  
  

  private void setupFeaturesList() {
	featuresList.clear();
	Enumeration e = ftm.listFeatures();
	String c;
	while (e.hasMoreElements())
	 {
		c = (String)e.nextElement();
		featuresList.addItem(c);
	 } 
  }  
  
 /**
  * Mostra questa finestra 
  */
 public void show(){
       super.show();
	 categoriesList.clear();
	 setupCategoriesList();
  }  

}
