package uniba.gui;

import java.awt.*;
import java.io.*;
import java.lang.Runtime;
import java.util.Vector;
import java.util.StringTokenizer;
import java.sql.Time;
import uniba.core.*;
import uniba.databanker.*;

/**
 * Costruisce un albero di decisione usando
 * il programma <code>mktree</code> di OC1.
 * Questa classe utilizza la tecnologia J/Direct.
 */
public class MakeDTDialog extends Dialog implements Runnable{

  private Checkbox paralCheck;
  private Checkbox obliqCheck;
  private Checkbox prunedCheck;
  private Checkbox unprunedCheck;
  private String utente = null;
  private Databanker db = null;
  private CategoriesManager cgm = null;
  private Button makeButton = new Button("Make");
  private Thread cThread = null;
  private Frame parent = null;
  private String FILEPRUNEDTREE = null;
  private String FILEUNPRUNEDTREE = null;
  private String FILELOG = null; 		
  private String FILETRAINING = null;
  private int mode = 0;
  private Time inizio, fine;
  private String report;	  

  public MakeDTDialog(Frame f, String utente, Databanker db, CategoriesManager cgm, int mode) {
	super(f,"Make Decision Tree",true);
	this.parent = f;
	this.db = db;
      this.utente = utente;
      this.cgm = cgm;
      this.mode = mode;
      FILEPRUNEDTREE = ".\\DT\\"+ utente +"\\training.dt";
  	  FILEUNPRUNEDTREE = ".\\DT\\"+ utente +"\\training.unp";
      FILELOG = ".\\DT\\"+ utente +"\\oc1.log"; 		
      FILETRAINING = ".\\DT\\"+ utente +"\\training";
      setBackground(new Color(0).lightGray);
	Button cancelButton = new Button("Cancel");
	Panel buttonPanel = new Panel();
	 buttonPanel.add(makeButton);
	 buttonPanel.add(cancelButton);
	
	CheckboxGroup hyperplanesCheckGroup = new CheckboxGroup();
	paralCheck= new Checkbox("Parallel",hyperplanesCheckGroup,true);
	obliqCheck= new Checkbox("Obliqus",hyperplanesCheckGroup,false);
	Panel hyperPlanesPanel = new Panel();
	 hyperPlanesPanel.setLayout(new GridLayout(3,1));
	 hyperPlanesPanel.add(new Label("Hyperplanes    "));
	 hyperPlanesPanel.add(paralCheck);
	 hyperPlanesPanel.add(obliqCheck);
	
	CheckboxGroup pruneCheckGroup = new CheckboxGroup();
	prunedCheck= new Checkbox("Pruned",pruneCheckGroup,false);
	unprunedCheck= new Checkbox("Unpruned",pruneCheckGroup,true);
	Panel prunePanel = new Panel();
	 prunePanel.setLayout(new GridLayout(3,1));
	 prunePanel.add(new Label("Tree            "));
	 prunePanel.add(prunedCheck);
	 prunePanel.add(unprunedCheck);
	   
	Panel centerPanel = new Panel();
	 centerPanel.add(hyperPlanesPanel);
	 centerPanel.add(prunePanel);
	
	add("Center",centerPanel);
	add("South",buttonPanel);
	resize(290,290);
	pack();
	setResizable(false);
  }  

  public boolean handleEvent(Event evt) {
	if (evt.id==evt.WINDOW_DESTROY) {
	  hide();
	  return true;
	}
	else
	if (evt.id==evt.ACTION_EVENT) {
	  if (evt.target instanceof Button) {
		if (evt.arg.equals("Make")) {
		    try {	
			    db.getAutoma().Transizione("mt");
			    cThread = new Thread(this);	
			    cThread.start();	
		    } catch(Exception oex) {
				new StampaErrore(parent,
                			"Make Decision tree",
				      "Exception:\n" + oex.getMessage());
		    }	
		    return true;
		}
		else
		if (evt.arg.equals("Cancel")) {
		  hide();
		  return true;
		}
	  }
	  else
	  if (evt.target instanceof Checkbox) {
		Checkbox c = (Checkbox) evt.target;
		return true;  
	  }
	  
	}
	return false;
  }  

	public void run() {
		
		makeButton.setEnabled(false);
		parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		setCursor(new Cursor(Cursor.WAIT_CURSOR));
		inizio = new Time(System.currentTimeMillis());
		//Elimina vecchi file
		java.io.File training = new java.io.File(FILETRAINING);
            java.io.File oc1log = new java.io.File(FILELOG);
		java.io.File pruned = new java.io.File(FILEPRUNEDTREE);
		java.io.File unpruned = new java.io.File(FILEUNPRUNEDTREE);
		
        training.delete();
		oc1log.delete();
		pruned.delete();
        unpruned.delete();
		
		try {
			Vector pagine = new Vector();
			pagine = db.getTrainingPages(utente);
			FileWriter fw = new FileWriter(FILETRAINING, false);
			for(int i=0; i<pagine.size(); i++) {
				StringTokenizer st = new StringTokenizer((String)pagine.elementAt(i));
				int idDoc = new Integer(st.nextToken()).intValue();
				String categoria = st.nextToken();
				double rep[] = db.getRepresentedPage(idDoc, utente);
				for (int k=0; k<rep.length; k++) {
					fw.write(rep[k] + ", ");
				}
				int cat = cgm.categoryIndex(categoria) + 1; 
				fw.write(cat + "\n");

			}
			
			pagine = db.getRejectPages(utente);
			for(int i=0; i<pagine.size(); i++) {
				int idDoc = new Integer((String)pagine.elementAt(i)).intValue();
				double rep[] = db.getRepresentedPage(idDoc, utente);
				for (int k=0; k<rep.length; k++) {
					fw.write(rep[k] + ", ");
				}
				int cat = cgm.categoryIndex("UNKNOWN") + 1; 
				fw.write(cat + "\n");

			}
			
			fw.close();
			try {
				if ((paralCheck.getState()) && (unprunedCheck.getState())) {
					new MkTree(".\\DT\\"+ utente +"\\mktree.bat", mode);
					System.out.println("Parall. Unpruned Decision Tree created");
				} else if ((obliqCheck.getState()) && (unprunedCheck.getState())) {
					new MkTree(".\\DT\\"+ utente +"\\mktree1.bat", mode);
					System.out.println("Obliq. Unpruned Decision Tree created");
				} else if ((paralCheck.getState()) && (prunedCheck.getState())) {
					new MkTree(".\\DT\\"+ utente +"\\mktree2.bat", mode);
					System.out.println("Parall. pruned Decision Tree created");
				} else if ((obliqCheck.getState()) && (prunedCheck.getState())) {
					new MkTree(".\\DT\\"+ utente +"\\mktree3.bat", mode);
					System.out.println("Obliq. pruned Decision Tree created");
				}
                  } catch (UnsatisfiedLinkError ule) {
					parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		    			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		    			setVisible(false);
					new StampaErrore(parent, 
                                      	     "Make Decision Tree", 
                                      	     "Impossibile continuare:\n" +
                                               "per utilizzare questa funzione, compilare\n" +
                                               "ed eseguire WebClass con i tools\n" +
                                               "della Microsoft per Java.");
					return;
			}

			boolean fine = false;
			while (!fine) {
				try {
					FileReader fr = new FileReader(FILELOG);
					if (fr.ready()) {
						fine = true;
						StringBuffer buf = new StringBuffer();
						int c = 0;
						do {
							c = fr.read();
							buf.append((char)c);
						} while (c != -1);
						fr.close();
						System.out.println(buf.toString());
					} 
				} catch (FileNotFoundException e) {}
			}
		    } catch (Exception e) {
				new StampaErrore(parent,
                                         "Make Decision tree",
				      	     "Exception:\n" + e.getMessage());
                        e.printStackTrace();
		    }  
		    parent.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		    fine = new Time(System.currentTimeMillis());
		    report = "Make Decision Tree\n\tinizio: " + 
			inizio.toString() + " fine: " + fine.toString();
		    System.out.println(report);			
		    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		    setVisible(false);	
	}

}