package uniba.gui;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.sql.Time;

import uniba.core.*;
import uniba.databanker.*;

/**
 * Calcola le soglie di rigetto per la varie categorie
 * di un utente. C' una eccezione quando non ci sono
 * pagine di rigetto.
 */
public class MakeRejectThresholdForCategoriesDialog extends Dialog {

   private Frame parent;  
   private TextArea result;
   private boolean rejectThresholdsMade;   
   private CategoriesManager cgm;
   private TrainingPagesManager trpm;
   private RejectPagesManager rjpm;
   private Frame progressWindow;
   private Databanker db = null;
   private String utente = null;
   private Time inizio, fine;	

  public MakeRejectThresholdForCategoriesDialog(Frame parent, 
						            CategoriesManager cgm,
								TrainingPagesManager trpm,
								RejectPagesManager rjpm,
								Frame progressWindow,
								Databanker db,
								String utente) {					

	super(parent,"Make Reject Thresholds For Categories",true);
	this.parent = parent;
	setBackground(new Color(0).lightGray);
	Button makeButton = new Button("Make");
	Button cancelButton = new Button("Cancel");
	Panel buttonPanel = new Panel();
	 buttonPanel.add(makeButton);
	 buttonPanel.add(cancelButton);
	result = new TextArea(20,40); 
	result.setBackground(new Color(0).white);
	result.setForeground(new Color(0).black);
	result.setFont(new Font("Curier",0,12));
	result.setText(" ");
	Panel centerPanel = new Panel();
	centerPanel.setLayout(new GridLayout(1,1));
	centerPanel.add(result);
	add("Center",centerPanel);
	add("South",buttonPanel);
	resize(600,400);
	// pack();
	setResizable(false);
	this.cgm=cgm;
	this.db = db;
	this.trpm=trpm;
	this.rjpm=rjpm;
	this.utente = utente;
	
	
	this.progressWindow = progressWindow;
	rejectThresholdsMade = false;
  }  


  public boolean handleEvent(Event evt) {
	if (evt.id==evt.WINDOW_DESTROY) {
	  hide();
	  return true;
	}
	else
	if (evt.id==evt.ACTION_EVENT) {
	  if (evt.target instanceof Button) {
		if (evt.arg.equals("Make")) {
		  inizio = new Time(System.currentTimeMillis());
		  if (rjpm.getNumPages() > 0) {	
		  	progressWindow.show();
		  	makeRejectThresholds(progressWindow);
		  } else 
			result.setText("No Reject Pages Found");
		  fine = new Time(System.currentTimeMillis());
		  System.out.println("Make Reject\ninizio: " + inizio.toString() + " fine: " +
			fine.toString());
		  return true;
		}
		else
		if (evt.arg.equals("Cancel")) {
		  result.setText(" ");
		  hide();
		  return true;
		}
	  }
	}
	return false;
  }  

private void makeRejectThresholds(Frame progressWindow){
  String line="";
  String currentCategory="";
  int idDoc;
  String urlDoc = "";
  String catOfPage="";
  Vector vectOfThresholds=new Vector(); 
  Vector vectOfSimCurrentCateg =new Vector();	 // vettore contenente le similarit 
                                             // tra le pagine di training della
											 // categoria corrente e il centroide
                                             // della categoria correte.	
  Vector vectOfSimilarityOtherCateg =new Vector();
  Vector vectOfSimilarityReject =new Vector();
  Vector vectOfCentroids= new Vector();
  
  try {
  	for (int i=0; i<cgm.totalCategories(); i++)			
	    vectOfCentroids.addElement(db.getCentroid(utente, cgm.categoryName(i)));    
	  
	for (int i=0; i< cgm.totalCategories(); i++) {
		vectOfSimCurrentCateg.removeAllElements();
		vectOfSimilarityOtherCateg.removeAllElements();
		vectOfSimilarityReject.removeAllElements();
		double greater = 0;
		currentCategory = cgm.categoryName(i);

		Enumeration trainPages=trpm.listPages();
		while(trainPages.hasMoreElements()) {
			StringTokenizer page = new StringTokenizer((String)trainPages.nextElement());	
                  idDoc = (new Integer(page.nextToken())).intValue();
			catOfPage = page.nextToken();
			urlDoc = page.nextToken();
                  double rep[] = db.getRepresentedPage(idDoc, utente);
			double centr[] = (double[])vectOfCentroids.elementAt(i);
			double similarity = Vettori.distanza(rep, centr);
			Double s = new Double(similarity);
		   	if(catOfPage.equalsIgnoreCase(currentCategory)){
		      	System.out.println("\n" + "pageName:" + urlDoc);
		      	System.out.println("sim di cat:" + similarity); 
		      	vectOfSimCurrentCateg.addElement(s);	
			} else {
		     		System.out.println("\n" + "pageName:" + urlDoc);
		     		System.out.println("Sim.di altre cat.:"+similarity);
		      	vectOfSimilarityOtherCateg.addElement(s);
			}
		} //end while
		Vector orderedVectOfSimCurrentCateg=ordina(vectOfSimCurrentCateg);
	      Vector orderedVectOfSimilarityOtherCateg=ordina(vectOfSimilarityOtherCateg);

		// calcola il vettore delle similarit tra le pagine di rigetto 
		// e il centroide della categoria corrente.
		Enumeration rejectPages = rjpm.listPages();
		while(rejectPages.hasMoreElements()){
			StringTokenizer pg = new StringTokenizer((String)rejectPages.nextElement());	
                  int iden = (new Integer(pg.nextToken())).intValue();
			double rep[] = db.getRepresentedPage(iden, utente);
			double centr[] = (double[])vectOfCentroids.elementAt(i);
		 	double sim = Vettori.distanza(rep, centr);
			Double simil = new Double(sim);
			System.out.println("\n"+"pageName:" + db.getUrl(iden));
		      System.out.println("sim di rjpag:"+sim);
			vectOfSimilarityReject.addElement(simil);
		}
		Vector orderedVectOfSimilarityReject=ordina(vectOfSimilarityReject);

		
		double lastOc =((Double)orderedVectOfSimilarityOtherCateg.lastElement()).
			doubleValue();
		double lastRj=((Double)orderedVectOfSimilarityReject.lastElement()).
			doubleValue();
		
    	  	if(lastRj>lastOc)
			greater = lastRj;
		else 
			greater=lastOc;
		if(greater<((Double)orderedVectOfSimCurrentCateg.firstElement()).doubleValue()){
			greater *= 100;
		      Double gr=new Double((double)(greater));
			vectOfThresholds.addElement(gr);
		} else {
		      double gr=(double)bestThreshold
		      	(orderedVectOfSimCurrentCateg,
		      	orderedVectOfSimilarityOtherCateg,
		      	orderedVectOfSimilarityReject);
			gr *=100;
			Double Gr =new Double(gr);
			vectOfThresholds.addElement(Gr);
		}
      }// end for(int i=0; ....)

	String ris = "\t\t\tReject Thresholds\n";
	for (int k=0; k<vectOfThresholds.size(); k++) {
		double soglia = ((Double)vectOfThresholds.elementAt(k)).doubleValue();
		db.updateThreshold(utente, cgm.categoryName(k), soglia);
		ris = ris + " reject threshold for " + cgm.categoryName(k) + 
			":\t\t" + soglia + "%\n";
	}
	result.setForeground(new Color(0).blue);
	result.setText(result.getText() + ris + "\n");	
  } catch(Exception  e) {
		new StampaErrore(parent,
                             "Make Reject Thresholds",
  		      	     "Exception:\n" + e.getMessage());
            e.printStackTrace();

  } finally {
	 progressWindow.hide();
  }
  rejectThresholdsMade = true;	
} // end method makeRejectThresholds()



private Vector ordina(Vector v){
 Double temp;
 Double temp1;
 for(int t=0;t<(v.size())-1;t++){
   for(int y=t+1;y<v.size();y++){
    if(((Double)v.elementAt(t)).doubleValue()>((Double)v.elementAt(y)).doubleValue()){
	 temp=(Double)v.elementAt(t);
	 temp1=(Double)v.elementAt(y);
	 v.setElementAt(temp1,t);
	 v.setElementAt(temp,y);
    }
   }
 }
 return v;
}

private Vector eliminateDuplicate(Vector v){
 Vector newVector=new Vector();
 newVector.addElement((Double)v.elementAt(0)); 
 for (int t=1;t<v.size();t++){
  if(!((Double)v.elementAt(t)).equals((Double)v.elementAt(t-1)))
     newVector.addElement((Double)v.elementAt(t));
 }
return newVector;
} 


private double bestThreshold(Vector V1,Vector V2,Vector V3){
  Vector globalVector= new Vector();
  int n=0;
  int n1=0;
  double bestTh = 0;
  double relativeOmissionError=0;  // errore relativo di omissione = recall
  double relativeCommissionError=0;  // errore relativo di commissione = precision
  double currentFScore=0;
  double maxFScore=0;

  for(int d=0;d< V1.size();d++){
    globalVector.addElement((Double)V1.elementAt(d));
  }
  for(int d=0;d< V2.size();d++){
    globalVector.addElement((Double)V2.elementAt(d));
  }	
  for(int d=0;d< V3.size();d++){
    globalVector.addElement((Double)V3.elementAt(d));
  }
  Vector orderedGlobalVector=ordina(globalVector);
  System.out.print("Vettore ordinato globale: ");
  for(int g=0;g<orderedGlobalVector.size();g++){
    System.out.print(((Double)orderedGlobalVector.elementAt(g)).doubleValue()+"  ");
  }	
  System.out.println();	
  Vector orderedGlobalVectorWithoutDuplicate=eliminateDuplicate(orderedGlobalVector); 
  System.out.print("Vettore ordinato globale senza duplicati: ");
  for(int g=0;g<orderedGlobalVectorWithoutDuplicate.size();g++){
    System.out.print(((Double)orderedGlobalVectorWithoutDuplicate.elementAt(g)).
    doubleValue()+"  ");
  }	
  System.out.println();	
  for(int h=0;h<((orderedGlobalVectorWithoutDuplicate.size())-1);h++){
   double previous = ((Double)orderedGlobalVectorWithoutDuplicate.elementAt(h)).
   	doubleValue();
   double following = ((Double)orderedGlobalVectorWithoutDuplicate.elementAt(h+1)).
   	doubleValue();
   double middle= (previous+following);
   middle /= 2;
   System.out.println();
   System.out.println("previous: "+previous);
   System.out.println("following: "+following);
   System.out.println("middle: "+middle);
   System.out.println();
   //calcola l'errore di omissione relativo che si commetterebbe considerando
   //  middle come bestTh (best threshold). Tale errore rappresenta la recall.
   if(middle<((Double)V1.firstElement()).doubleValue())
     relativeOmissionError=0; 
   else{
     n=0;
     for(int h1=0;h1<V1.size();h1++){
        if(middle >= ((Double)V1.elementAt(h1)).doubleValue())
	      n += 1;
	 }
	 int totalPosExamples  = V1.size();
     relativeOmissionError=(((double)n)/totalPosExamples);
     System.out.println("n: "+n);
	} //end else
	
	System.out.println("relativeOmissionError: "+relativeOmissionError);
	// calcola l'errore di commissione relativo che si commetterebbe considerando
	// middle come bestTh (best threshold). Tale errore rappresenta la precision.
	if((middle > ((Double)V2.lastElement()).doubleValue()) && 
	  (middle > ((Double)V3.lastElement()).doubleValue()))
	   relativeCommissionError=0;
	else{
	     n1=0;
		 for(int h2=0;h2<V2.size();h2++){
		   if(middle <= ((Double)V2.elementAt(h2)).doubleValue()){
		    n1 +=1;
		   }
		 }
		 for(int h3=0;h3<V3.size();h3++){
		   if(middle <= ((Double)V3.elementAt(h3)).doubleValue()){
              n1 +=1;
		   }
	     }
		 System.out.println("n1: "+n1);
		 int totalNegExamples=(V2.size() + V3.size());
		 System.out.println("totalNegExamples: "+totalNegExamples);
		 relativeCommissionError = (((double)n1)/totalNegExamples);
    	
	 }  //end else
	System.out.println("relativeCommissionError: "+relativeCommissionError);
	
	// Si sceglier come bestThreshold il middle che permetter di commettere
	// meno errori di omissione e meno errori di commissione, ovvero il middle 
	// che  massimizzer l' F-Score dove
	// F-Score = (2* recall*precision)/(recall+precision).
	double totalErrors=((double)(relativeOmissionError + relativeCommissionError));
	if(totalErrors==0) 
	    totalErrors=1;
	currentFScore =  ((double)(2 * relativeOmissionError * relativeCommissionError))/
		totalErrors;
	System.out.println("currentFScore: "+ (double)currentFScore);
	if(currentFScore>maxFScore){
	  maxFScore = currentFScore;
	  bestTh = middle;
	  System.out.println("bestTh: "+bestTh);
	}
  } // end for(int h=0;h<(orderedGlobalVectorWithoutDuplicate.size())-1;h++).......
  return bestTh;
 } // end method bestThreshold.

} // end class 



         

