package uniba.gui;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import java.sql.SQLException;
import uniba.databanker.*;
import uniba.core.*;

/**
* La classe implementa l'interfaccia per l'utente<br>
* che consente di inserire il testo di una nuova<br>
* query in linguaggio naturale.
*/

public class NewQueryDialog extends Dialog implements MouseListener,
									TextListener{

	Frame parent = null;
	TextArea queryText;
	TextArea nameText;
	Label queryLabel;
	Label nameLabel; 
	Button saveBut, closeBut;
	String nomeQuery;
	String testoQuery;
	String utente;
	Databanker db;
	
	public NewQueryDialog(Frame parent, String utente, Databanker db) {
		
		super(parent, "", true);
		this.parent = parent;
		this.db = db;
		this.utente = utente;
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets = new Insets(0,0,0,15);
		setLayout(gridbag);

		// Etichetta di query
		buildConstraints(constraints, 0, 0, 1, 1, 70, 25);
		constraints.fill = GridBagConstraints.NONE;
		constraints.anchor = GridBagConstraints.WEST;
		queryLabel = new Label("Query :", Label.LEFT);
		gridbag.setConstraints(queryLabel, constraints);
		add(queryLabel);
		queryLabel.setBackground(Color.lightGray);

		// Etichetta di nome
		buildConstraints(constraints, 1, 0, 1, 1, 30, 0);
		constraints.fill = GridBagConstraints.BOTH;
		nameLabel = new Label("Name :", Label.LEFT);
		gridbag.setConstraints(nameLabel, constraints);
		add(nameLabel);
		nameLabel.setBackground(Color.lightGray);

		// Area di testo della query
		buildConstraints(constraints, 0, 1, 1, 3, 0, 0);
		constraints.fill = GridBagConstraints.BOTH;
		queryText = new TextArea("", 1, 40, (new TextArea()).SCROLLBARS_BOTH);
		gridbag.setConstraints(queryText, constraints);
		queryText.setFont(new Font("Courier",0,12));
		queryText.addTextListener(this);
		add(queryText);

		// Area di testo del nome della query
		buildConstraints(constraints, 1, 1, 1, 1, 0, 25);
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.anchor = GridBagConstraints.NORTH;
		nameText = new TextArea("", 2, 20, (new TextArea()).SCROLLBARS_HORIZONTAL_ONLY);
		gridbag.setConstraints(nameText, constraints);
		nameText.addTextListener(this);
		add(nameText);

		// Pulsante saveBut
		buildConstraints(constraints, 1, 2, 1, 1, 0, 25);
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.anchor = GridBagConstraints.CENTER;
		saveBut = new Button("Save Query");
		gridbag.setConstraints(saveBut, constraints);
		saveBut.addMouseListener(this);
		add(saveBut);
		saveBut.disable();

		// Pulsante closeBut
		buildConstraints(constraints, 1, 3, 1, 1, 0, 25);
		constraints.fill = GridBagConstraints.HORIZONTAL;
		constraints.anchor = GridBagConstraints.CENTER;
		closeBut = new Button("Close");
		gridbag.setConstraints(closeBut, constraints);
		closeBut.addMouseListener(this);
		add(closeBut);

		this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					hide();}
				});

		
		setSize(400, 200); 
            Toolkit t = parent.getToolkit();
	      int scw   = t.getScreenSize().width;
	      int sch   = t.getScreenSize().height;
	      int w     = this.size().width;
	      int h     = this.size().height;
	      setLocation((scw/2)-(w/2), (sch/2)-(h/2)+23);	
            setResizable(false);
	
            
	}

//-----------------------------------------------------------------------------
		

private void buildConstraints(GridBagConstraints gbc, int gx, int gy, int gw, int gh,
	int wx, int wy) {
           
      gbc.gridx = gx;
      gbc.gridy = gy;
      gbc.gridwidth = gw;
      gbc.gridheight = gh;
      gbc.weightx = wx;
      gbc.weighty = wy;
}

//----------------------------------------------------------------------------

public void mousePressed(MouseEvent e) {}

public void mouseReleased(MouseEvent e) {}

public void mouseEntered(MouseEvent e) {}

public void mouseExited(MouseEvent e) {}

public void mouseClicked(MouseEvent e) {
	if (((Button)e.getSource()).getLabel().equals("New")) { 
		setTitle("WebClass - Qtf^2-Tf*Idf - New");
		setVisible(true);
	} else
	if (((Button)e.getSource()).getLabel().equals("View")) {
		setTitle("WebClass - Qtf^2-Tf*Idf - View");
		setVisible(true);
	} else	
	if (((Button)e.getSource()).getLabel().equals("Close")) {
		setVisible(false); 
	} else
	if (((Button)e.getSource()).getLabel().equals("Save Query")) {
		testoQuery = queryText.getText();
		nomeQuery = nameText.getText();
		if ((testoQuery == "") || (nomeQuery == ""))
			new StampaErrore(this.parent, "Save Query", 
                  	"Valori nelle aree di testo non validi.");	
		else {
			testoQuery.replace('\n', ' ');
			try {
				Vector conf = db.getConfiguration(utente);
				Vector stop = db.getStopword(utente);
				Vector tokenQueryHW = 
					(new QueryTokenizer(testoQuery, conf, stop)).getTokenQuery();
				db.setQuery(utente, nomeQuery, testoQuery, tokenQueryHW);
			} catch(Exception ex) {new StampaErrore(this.parent, 
				"Save Query", "Exception:\n" + ex.getMessage());ex.printStackTrace();}
		}
		setVisible(false);
	}
}

//-------------------------------------------------------

/**
* Aggiorna le variabili di istanza con il testo ed il nome della query.
*/

public void setText(String testo, String nome){
	String result = "";
	StringTokenizer st = new StringTokenizer(testo);
	testo = "";
	String token = "";
	while (st.hasMoreTokens()) {
		token = st.nextToken();
		if ((result+token).length() > 30) {		
			testo = testo + result + "\n" ;
			result = token + " "; }
		else
			result = result + token + " " ;
	}
	testo = testo + result;
	queryText.setText(testo);
	nameText.setText(nome);
	saveBut.disable();
}

//-----------------------------------------------------

public void textValueChanged(TextEvent e) {
	if (e.getID() == TextEvent.TEXT_VALUE_CHANGED)
		saveBut.enable();
}
	
//-------------------------------------------------------
	
}
