package uniba.gui;

import java.io.*;
import java.awt.*;
import java.util.*;
import uniba.core.*;
import uniba.databanker.*;

/**
 * Permette la rappresentazione di una pagina web recuperata da disco o
 * da Internet
 */
public class PageRepresentDialog extends BrowserDialog {
  
  private Button representButton;
  private FeaturesManager ftm;
  private RepresentManager rpm;
  private PannelloValori pv = new PannelloValori();
  private Databanker db = null;
	
  /**
   * Costruttore dell'oggetto PageClassifyDialog. Crea la GUI.
   */     
  public PageRepresentDialog(Frame f, FeaturesManager ftm, RepresentManager rpm,
								           Vector conf, Vector stopword, Databanker db) {
   super(f, conf, stopword, db);
   this.db = db;
   pv.setList(wordList);
   setTitle("Represent a Web Page");
   representButton = new Button("Represent");
   representButton.setEnabled(false);	
   buttonPanel.add(representButton, 0);
   remove(centerPanel);
   Panel newCenterPanel = new Panel();
   newCenterPanel.setLayout(new BorderLayout());
   newCenterPanel.add("Center",centerPanel);
   newCenterPanel.add("South", pv);
   add("Center",newCenterPanel);

   resize(800,600);
   Toolkit t = f.getToolkit();
   int scw   = t.getScreenSize().width;
   int sch   = t.getScreenSize().height;
   int w     = this.size().width;
   int h     = this.size().height;
   setLocation((scw/2)-(w/2), (sch/2)-(h/2));

   setResizable(false);

   this.ftm=ftm;
   this.rpm=rpm; 
  
  }  


  public boolean handleEvent(Event evt) {
	if (evt.id == evt.KEY_PRESS) {
		pv.clear();
		return super.handleEvent(evt);
	} else
	if (evt.id==evt.ACTION_EVENT) {
	  if (evt.target instanceof Button) {
		if (evt.arg.equals("Represent")) {
		  if (isClearPage==false) {
			  representButton.setEnabled(false);
			  rpm.represent(parole, totParole);
			  setupFeaturesList();
			return true;
		  }
		}
		else if (evt.arg.equals("Open file")) {
			openFile.setDirectory("./html/");
			openFile.setFile("*.htm");
		  	openFile.show();
		  	if (openFile.getFile() != null) {
		   		clearPage();
		   		String fileName;
		   		fileName=openFile.getDirectory()+openFile.getFile();
		   		String fileUrl;
		   		fileUrl="file://"+fileName.replace('\\','/');
		   		urlName.setText(fileUrl);
		   		wordList.clear();
		   		wordsLabel.setText("HTML Tagged Words");  
				pv.clear();
                        representButton.setEnabled(false); 
				connectButton.setEnabled(true);
				viewButton.setEnabled(true);
			}
			return true;
		}
		else if (evt.arg.equals("Connect")) {
			super.handleEvent(evt);
			if (!urlName.getText().equalsIgnoreCase("HTTP://"))
				representButton.setEnabled(true);
			return true;
		}
		else if (evt.arg.equals("Clear")) {
			pv.clear();
                  representButton.setEnabled(false);
			wordsLabel.setText("HTML Tagged Words");  
			return super.handleEvent(evt);
		}
	  }  
	}
      else if (evt.id==evt.LIST_SELECT) {
		if (evt.target instanceof java.awt.List) {
			java.awt.List l = (java.awt.List)evt.target;
			if (l.equals(wordList)) {
				StringTokenizer st = new StringTokenizer(wordList.getSelectedItem());
				int numToc = st.countTokens();
				for (int i=0; i<numToc-2; i++)
					st.nextToken();
				String t = st.nextToken();
				pv.select( t );
			}
            }
      } 
 
	return super.handleEvent(evt);
	
  } 

 
  public void setupFeaturesList() {
      pv.clear();
	Enumeration e = rpm.listFeatures();
      int ntok;
      while (e.hasMoreElements()) {
		StringTokenizer st = new StringTokenizer((String)e.nextElement());
            pv.setFeature(st.nextToken());
            pv.setFrequency(st.nextToken());
            pv.setHtmlWeight(st.nextToken());
            pv.setTotWeight(st.nextToken());
            String context = "";
            ntok = st.countTokens();
            while (ntok > 0) {
                  context = context + st.nextToken() + "  ";
                  ntok = st.countTokens();
            }
            pv.setHtmlContext(context); 
      }
     
  }  

}