package uniba.gui;

import java.awt.*;
import java.io.*;
import java.awt.event.*;
import java.util.*;
import java.sql.SQLException;
import uniba.databanker.*;

/**
* La classe implementa l'interfaccia per l'utente<tf>
* che consente di selezionare il file dal quale estrarre le stopword.
*/

public class StopwordsDialog extends Dialog implements MouseListener {

	Frame parent = null;
	Button br;
	TextField tf;
	Button cancellBut, insertBut;
	FileDialog fd;
	String utente;
	Databanker db;
	Vector stopwords;
	StreamTokenizer st;
	
	public StopwordsDialog(Frame parent, String utente, Databanker db) {
		
		super(parent, "Insert Stopwords", true);
		this.parent = parent;
		this.db = db;
		this.utente = utente;
		stopwords = new Vector();
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets = new Insets(0,0,0,0);
		setLayout(gridbag);

		// Bottone Browse
		buildConstraints(constraints, 0, 0, 1, 1, 50, 50);
		constraints.fill = GridBagConstraints.NONE;
		br = new Button("Browse File...");
		gridbag.setConstraints(br, constraints);
		br.addMouseListener(this);
		add(br);

		// campo di testo
		buildConstraints(constraints, 1, 0, 1, 1, 50, 50);
		constraints.fill = GridBagConstraints.HORIZONTAL;
		tf = new TextField("", 10);
		gridbag.setConstraints(tf, constraints);
		tf.setFont(new Font("Courier",0,12));
		add(tf);

		// Pulsante cancellBut
		buildConstraints(constraints, 0, 1, 1, 1, 0, 0);
		constraints.fill = GridBagConstraints.NONE;
		constraints.anchor = GridBagConstraints.EAST;
		cancellBut = new Button("Cancell");
		gridbag.setConstraints(cancellBut, constraints);
		cancellBut.addMouseListener(this);
		add(cancellBut);

		// Pulsante insertBut
		buildConstraints(constraints, 1, 1, 1, 1, 0, 0);
		constraints.fill = GridBagConstraints.NONE;
		constraints.anchor = GridBagConstraints.WEST;
		insertBut = new Button("Insert");
		gridbag.setConstraints(insertBut, constraints);
		insertBut.addMouseListener(this);
		add(insertBut);

		this.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					hide();}
				});

		fd = new FileDialog(new Frame(), "Open file", FileDialog.LOAD);
		fd.setDirectory("./");
		fd.setFile("*.stop");
		
		setSize(300, 100); 
            Toolkit t = parent.getToolkit();
	      int scw   = t.getScreenSize().width;
	      int sch   = t.getScreenSize().height;
	      int w     = this.size().width;
	      int h     = this.size().height;
	      setLocation((scw/2)-(w/2), (sch/2)-(h/2)+23);	
            setResizable(false);
		setVisible(true);


	
            
	}

//-----------------------------------------------------------------------------
		

private void buildConstraints(GridBagConstraints gbc, int gx, int gy, int gw, int gh,
	int wx, int wy) {
           
      gbc.gridx = gx;
      gbc.gridy = gy;
      gbc.gridwidth = gw;
      gbc.gridheight = gh;
      gbc.weightx = wx;
      gbc.weighty = wy;
}

//----------------------------------------------------------------------------

public void mousePressed(MouseEvent e) {
	if (((Button)e.getSource()).equals(insertBut)) {
		parent.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		setVisible(false);
	}
}

public void mouseReleased(MouseEvent e) {}

public void mouseEntered(MouseEvent e) {}

public void mouseExited(MouseEvent e) {}

public void mouseClicked(MouseEvent e) {
	if (((Button)e.getSource()).equals(cancellBut)) {
		setVisible(false); 
	} else
	if (((Button)e.getSource()).equals(insertBut)) {
		setCursor(new Cursor(Cursor.WAIT_CURSOR));
		String fileName = tf.getText();
		try {
			if (fileName.equals("")) {
				throw new Exception("Nome di file non valido per il caricamento."); 
			} else {
				stopwords = loadStopword(fileName); 
				db.insertStopwords(utente, stopwords);
			}
		} catch(Exception ex) {
                   new StampaErrore(parent, 
                       	"Load Stopword", 
                        "Exception:\n" + ex.getMessage());ex.printStackTrace();
           	}
	} else
	if (((Button)e.getSource()).equals(br)) {
		fd.setFile("*.stop");	
		fd.setVisible(true);
		if (fd.getFile() != null) {
			tf.setText(fd.getDirectory() + fd.getFile());
			fd.setVisible(false);
		}
	}
}

//-------------------------------------------------------

private Vector loadStopword(String fileName) throws Exception {
	Vector stopwords = new Vector();	
	BufferedReader in = new BufferedReader(new FileReader(fileName));
	st = new StreamTokenizer(in);
	int tipo = st.nextToken();
	while (tipo != st.TT_EOF) {
		String token = "";
		switch (tipo) {
			case st.TT_WORD: {token = st.sval;} break;	
			case st.TT_NUMBER: {token = "" + st.nval + "";} break;
		}
		if (!token.equals(""))
			stopwords.addElement(token);
		tipo = st.nextToken();
	}
	return stopwords;
}

//------------------------------------------------------------------------

}
