package uniba.gui;

import java.io.*;
import java.awt.*;
import java.lang.Runtime;
import java.util.*;
import java.sql.SQLException;
import java.sql.Time;


import uniba.core.*;
import uniba.databanker.*;


public class TestingPagesClassifyDialog extends Dialog {
  
  Button classify1Button;
  Button classify2Button;
  Button classify3Button;
  Button classify4Button;
  Button classify5Button;
  Button clearButton;
  Button cancelButton;
  public TextArea resultText;
  Frame f;
  
  CategoriesManager cgm;
  TestingPagesManager tspm;
  TrainingPagesManager trpm;
  FeaturesManager ftm;
  Vector tVector=new Vector();
  Classifier cls;
  Databanker db;
  String utente;
  int kFactor = 0;	
  int mode = 0;
  private Time inizio, fine;
  private String report;	
  
  /**
	Costruttore dell'oggetto TestingPagesClassifyDialog. Crea la GUI.
  */     
  public TestingPagesClassifyDialog(Frame f, CategoriesManager cgm, 
		 TestingPagesManager tspm, TrainingPagesManager trpm,
		 FeaturesManager ftm, String utente, Databanker db, int mode)
			 throws SQLException, OperationException {	
   super(f,"Classify all Testing Pages",true );
   this.f=f;
   this.mode = mode; 
   setBackground(new Color(0).lightGray);
   classify1Button = new Button("Classify by DT");
   classify2Button = new Button("Classify by Centroids");
   classify3Button = new Button("Classify by NN");
   classify4Button = new Button("Classify by k-NN");
   classify5Button = new Button("Classify by naive Bayesian ");
   clearButton = new Button("Clear");
   cancelButton = new Button("Cancel");
   Panel buttonPanel = new Panel();
   buttonPanel.add(classify1Button);
   buttonPanel.add(classify2Button);
   buttonPanel.add(classify3Button);
   buttonPanel.add(classify4Button);
   buttonPanel.add(classify5Button);
   buttonPanel.add(clearButton);
   buttonPanel.add(cancelButton);
   Panel resultPanel = new Panel();
   resultPanel.setLayout(new BorderLayout());
   resultPanel.add("North",new Label("Results:"));
   resultText = new TextArea(20,40);
   resultText.setBackground(new Color(0).white);
   resultText.setForeground(new Color(0).blue);
   resultText.setFont(new Font("Courier",0,12));
   resultText.setText("");
   resultPanel.add("Center",resultText);

   add("Center",resultPanel);
   add("South",buttonPanel);
   resize(800,450);
   setResizable(false);
   this.cgm=cgm;
   this.tspm=tspm;
   this.trpm=trpm;
   this.ftm=ftm;
   this.db = db;
   this.utente = utente;
   kFactor = ((Integer)((Vector)db.getConfiguration(utente)).elementAt(7)).intValue();
   cls = new Classifier(cgm, trpm, utente, db);
   show();	
  }  

//------------------------------------------------------------------------------------------

   /** Classify all testing pages using centroids method.
   */     
   void classifyCentroidAllTestingPages()throws SQLException, OperationException {
	  int idDoc = 0;	
	  String urlDoc = "";
	  String category = "";
	  int totPage=0;
	  int okPage=0;
	  int accuracy=0;
	  ConfusionMatrix confMatr = new ConfusionMatrix(cgm);
	
		 inizio = new Time(System.currentTimeMillis());
		 // get list of testing pages
		 Enumeration testPages = tspm.listPages();
	  
		 while (testPages.hasMoreElements()) {
			StringTokenizer sT = new StringTokenizer((String)testPages.nextElement());
   
			idDoc = (new Integer(sT.nextToken())).intValue();
			category = sT.nextToken();
			if (category.equals("Reject"))
				category = "UNKNOWN";
			urlDoc = sT.nextToken();
						  
			totPage++;
			// Print progressive number, name and category of the page
		System.out.println("Documento n.ro " + totPage);
			
			int i = urlDoc.lastIndexOf("/");
                  String temp = urlDoc.substring(i+1, urlDoc.length()) + " of '" + category + 				"'                               " ;
//			resultText.setText(resultText.getText() +"\n"+ temp.substring(0,30));

			String result = cls.classifyCentroidPage(db.getRepresentedPage(idDoc, utente));
			String ok;

			if (result.equals(category)) {
			   ok = "True"; 
			   okPage++;
			} else {
			   ok = "False";
			}
			confMatr.write(category, result);
			temp = "classified as '" + result + "'                             ";
//			resultText.setText(resultText.getText()+temp.substring(0,30)+"'"+ok+"'");
		 }
		 resultText.setText(resultText.getText() + "\n\n\n" + confMatr.toString() + "\n");
System.out.println("\n" + confMatr.toString() + "\n");
 		 accuracy = (100*okPage) / totPage;
		 fine = new Time(System.currentTimeMillis());
		 report = "Classify by Centroids\n\tinizio: " + inizio.toString() +
			" fine: " + fine.toString();
		 String finalReport = "\n\nMicro-average (accuracy): " + accuracy + "%" +
			   confMatr.allPrecision() + confMatr.allRecall() + 
			   confMatr.allFScore() + "\n" + report;
		 resultText.setText(resultText.getText() + finalReport);
System.out.println(resultText.getText() + finalReport);
   }  // end classify2AllTestingPages                  

//-------------------------------------------------------------------------------------------

   /** Classify all testing pages using k-NN method.
   */ 
   void classifyKNNAllTestingPages2()throws SQLException, OperationException {
	  Vector test = new Vector();
	  Vector pages = new Vector();	
	  int idDoc = 0;
	  String urlDoc = "";
	  String category = "";
	  int totPage=0;
	  int okPage=0;
	  ConfusionMatrix confMatr = new ConfusionMatrix(cgm);
	
		 inizio = new Time(System.currentTimeMillis());
		 test = db.KNNTest2(kFactor, utente);
		 pages = db.getSortedTestingPages(utente);
		 for (int k = 0;k < pages.size();k++){
			StringTokenizer st = new StringTokenizer((String)pages.elementAt(k));	
			idDoc = (new Integer(st.nextToken())).intValue();
			category = st.nextToken();
			if (category.equals("Reject"))
				category = "UNKNOWN";
			urlDoc = st.nextToken();
			Vector page = new Vector();
			page = (Vector)test.elementAt(totPage);	
			String result = "";
			if (page.size() > 0) {
				result = cls.kNNEvalFunction(page);}
			else {
				result = "UNKNOWN";}	
			// Print progressive number, name and category of the page
			totPage++;
			
			int i = urlDoc.lastIndexOf("/");
                  String temp = urlDoc.substring(i+1, urlDoc.length()) + " of '" + category + 				"'                               " ;
//			resultText.setText(resultText.getText() + "\n" + temp.substring(0,30));
			String ok;
			if (result.equals(category)) {
			   ok = "True"; 
			   okPage++;
			} else {
			   ok = "False";
			}
			confMatr.write(category, result);
			temp = "classified as '" + result + "'                             ";
//			resultText.setText(resultText.getText()+temp.substring(0,30)+"'"+ok+"'");
		 }        
		 resultText.setText(resultText.getText() + "\n\n\n" + confMatr.toString() + "\n");
System.out.println("\n" + confMatr.toString() + "\n");
		 int accuracy = (100*okPage) / totPage;
		 fine = new Time(System.currentTimeMillis());
		 report = "Classify by K-NN\n\tinizio: " + inizio.toString() + 
		 	" fine: " + fine.toString();
		 String finalReport = "\n\nMicro-average (Accuracy): " + accuracy + "%" +
			confMatr.allPrecision() + confMatr.allRecall() + 
			confMatr.allFScore() + "\n" + report;
		 resultText.setText(resultText.getText()+finalReport);
System.out.println(resultText.getText()+finalReport);
	  
   } // end classifyKNNAllTestingPages2  

//-------------------------------------------------------------------------------------------

/**
* Classify all testing pages using naive Bayesian metod.
*/
    void classifyNaiveBayesianAllTestingPages()
		throws SQLException, OperationException {
	  int idDoc = 0;
	  String urlDoc = "";
	  String category = "";
	  int totPage=0;
	  int okPage=0;
	  int accuracy=0;
	  ConfusionMatrix confMatr = new ConfusionMatrix(cgm);
		 inizio = new Time(System.currentTimeMillis());
		 // get list of testing pages
		 Enumeration testPages = tspm.listPages();
	  
		 while (testPages.hasMoreElements()) {
			StringTokenizer sT = new StringTokenizer( (String)testPages.nextElement() );
   
			idDoc = (new Integer(sT.nextToken())).intValue();
			category = sT.nextToken();
			if (category.equals("Reject"))
				category = "UNKNOWN";
			urlDoc = sT.nextToken();
		 
			totPage++;
			// Print progressive number, name and category of the page
	   		System.out.println("Documento n.ro " + totPage);

			int i = urlDoc.lastIndexOf("/");
                  String temp = urlDoc.substring(i+1, urlDoc.length()) + " of '" + category + 				"'                               " ;
//			resultText.setText(resultText.getText() + "\n" + temp.substring(0,30));
			String result = cls.classifyNaiveBayesianPage(db.getRepresentedPage(idDoc, utente));
			String ok;

			if (result.equals(category)) {
			   ok = "True"; 
			   okPage++;
			} else {
			   ok = "False";
			}
			confMatr.write(category, result);
			temp = "classified as '" + result + "'                             ";
//			resultText.setText(resultText.getText()+temp.substring(0,30)+"'"+ok+"'");
		 }
 		 resultText.setText(resultText.getText() + "\n\n\n" + confMatr.toString() + "\n");
System.out.println("\n" + confMatr.toString() + "\n");
		 accuracy = (100*okPage) / totPage;
		 fine = new Time(System.currentTimeMillis());
		 report = "Classify by Naive Bayes\n\tinizio: " + inizio.toString() +
			" fine: " + fine.toString();
		 String finalReport = "\n\nMicro-average (accuracy): " + accuracy + "%" +
			   confMatr.allPrecision() + confMatr.allRecall() + 
			   confMatr.allFScore() + report;
		 resultText.setText(resultText.getText() + finalReport);
System.out.println(resultText.getText() + finalReport);
   }  // end classifyNaiveBayesianAllTestingPages                   
     
//----------------------------------------------------------------------------------------

   /** 
	Classify all testing pages using NN method.
   */ 
   void classifyNNAllTestingPages()throws SQLException, OperationException {
	int totPage=0;
	int okPage=0;
	ConfusionMatrix confMatr = new ConfusionMatrix(cgm);
	Vector pagine = new Vector();
	inizio = new Time(System.currentTimeMillis());
	pagine = db.NNTest(utente, cgm);
	for(totPage = 0;totPage < pagine.size();totPage++){
		StringTokenizer st = new StringTokenizer((String)pagine.elementAt(totPage));
		String idDoc = st.nextToken();
		String urlDoc = db.getUrl(new Integer(idDoc).intValue());
		System.out.println("Documento n.ro " + totPage);
		String reale = st.nextToken();
		if (reale.equals("Reject"))
			reale = "UNKNOWN";
		String risultato = st.nextToken();
		int i = urlDoc.lastIndexOf("/");
            String temp = urlDoc.substring(i+1, urlDoc.length()) + " of '" + reale + 				"'                               " ;
//		resultText.setText(resultText.getText() + "\n" + temp.substring(0,30));
		temp = "classified as '" + risultato + "'                             ";
		System.out.println(idDoc + " " + reale + " " + risultato);
		String ok;
		if (risultato.equals(reale)){
			ok = "True";
			okPage++;
 		} else {
			ok = "False";}
//		resultText.setText(resultText.getText()+temp.substring(0,25)+"'" + ok + "'");
		confMatr.write(reale, risultato);
	} 
	resultText.setText(resultText.getText() + "\n\n\n" + confMatr.toString() + "\n");
	int accuracy = (100*okPage) / totPage;
	fine = new Time(System.currentTimeMillis());
	report = "Classify by NN\n\tinizio: " + inizio.toString() +
		" fine: " + fine.toString();
	String finalReport = "\n\nMicro-average (Accuracy): " + accuracy + "%" +
		confMatr.allPrecision() + confMatr.allRecall() +
		confMatr.allFScore() + report;
	resultText.setText(resultText.getText() + finalReport);
System.out.println(resultText.getText() + finalReport);
   } // end classifyNNAllTestingPages         

//---------------------------------------------------------------------------------------------

  public boolean handleEvent(Event evt) {
	if (evt.id == evt.WINDOW_DESTROY) {
	  dispose();
	  return true;
	}
	if (evt.id==evt.ACTION_EVENT) {
	  if (evt.target instanceof Button) {
		if 	((evt.arg.equals("Classify by DT")) &&
			(classify1Button.isEnabled())) {
			classify1Button.disable();
			classify2Button.disable();
			classify3Button.disable();
			classify4Button.disable();
			classify5Button.disable();
			clearButton.disable();
			cancelButton.disable();
//-------------------------------------------------------------------------------------------
/*********************************  DECISION TREE  *****************************************/
			try {
				resultText.setText("");
				db.getAutoma().Transizione("cst");	
				setCursor(new Cursor(Cursor.WAIT_CURSOR));
				resultText.setText("Classifing by Decision Tree...");
				resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				DTClassifier decTree = new DTClassifier(this, utente, db, cgm, mode);
				decTree.start();
			}catch(Exception e) {
				new StampaErrore(f, 
						     "Classify all testing pages",
						     "Exception:\n"+e.getMessage());
				e.printStackTrace();
			} finally {
				classify1Button.enable();
				classify2Button.enable();
				classify3Button.enable();
				classify4Button.enable();
				classify5Button.enable();
				clearButton.enable();
				cancelButton.enable();
			}
//-------------------------------------------------------------------------------------------
			return true;
		}
		else
		if 	((evt.arg.equals("Classify by Centroids")) &&
			(classify2Button.isEnabled())) {
			 resultText.setText("");
			classify1Button.disable();
			classify2Button.disable();
			classify3Button.disable();
			classify4Button.disable();
			classify5Button.disable();
			clearButton.disable();
			cancelButton.disable();
			 setCursor(new Cursor(Cursor.WAIT_CURSOR));
			 resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			 resultText.setText("Classifing by Centroids...");
			 try {	
				 db.getAutoma().Transizione("csc");	
				 classifyCentroidAllTestingPages();
			 }catch(Exception e) {new StampaErrore(f,"Classify alla testing pages",
							"Exception:\n"+e.getMessage());e.printStackTrace();} 
			 finally {
				 setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				 resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				classify1Button.enable();
				classify2Button.enable();
				classify3Button.enable();
				classify4Button.enable();
				classify5Button.enable();
				clearButton.enable();
				cancelButton.enable();
			 }
			 return true;
		}
		else
		if 	((evt.arg.equals("Classify by NN")) &&
			(classify3Button.isEnabled())) {
			 resultText.setText("");
			classify1Button.disable();
			classify2Button.disable();
			classify3Button.disable();
			classify4Button.disable();
			classify5Button.disable();
			clearButton.disable();
			cancelButton.disable();
			 setCursor(new Cursor(Cursor.WAIT_CURSOR));
			 resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		  	 resultText.setText("Classifing by Nearest Neighbour...");
			 try {	
				 db.getAutoma().Transizione("csnn");
				 classifyNNAllTestingPages();
			 }catch(Exception e) {new StampaErrore(f,"Classify all testing pages",
							"Exception:\n"+e.getMessage());e.printStackTrace();} 
			 finally {
				 setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				 resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				classify1Button.enable();
				classify2Button.enable();
				classify3Button.enable();
				classify4Button.enable();
				classify5Button.enable();
				clearButton.enable();
				cancelButton.enable();
			 }
		   return true;
		}
		else
		if 	((evt.arg.equals("Classify by k-NN")) &&
			(classify4Button.isEnabled())) {

			 resultText.setText("");
			classify1Button.disable();
			classify2Button.disable();
			classify3Button.disable();
			classify4Button.disable();
			classify5Button.disable();
			clearButton.disable();
			cancelButton.disable();
			 setCursor(new Cursor(Cursor.WAIT_CURSOR));
			 resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			 resultText.setText("Classifing by " + kFactor + " -Nearest Neighbour...");
			 try {	
				 db.getAutoma().Transizione("csnn");
				 classifyKNNAllTestingPages2();
			 }catch(Exception e) {new StampaErrore(f,"Classify all testing pages",
							"Exception:\n"+e.getMessage());e.printStackTrace();} 
			 finally {
				 setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				 resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				classify1Button.enable();
				classify2Button.enable();
				classify3Button.enable();
				classify4Button.enable();
				classify5Button.enable();
				clearButton.enable();
				cancelButton.enable();
			 }
		   return true;
		}
		else
		if 	((evt.arg.equals("Classify by naive Bayesian ")) &&
			(classify5Button.isEnabled())) {
			 resultText.setText("");
			classify1Button.disable();
			classify2Button.disable();
			classify3Button.disable();
			classify4Button.disable();
			classify5Button.disable();
			clearButton.disable();
			cancelButton.disable();
			 setCursor(new Cursor(Cursor.WAIT_CURSOR));
			 resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		   	 resultText.setText("Classifing by naive Bayesian ");
			 try {	
				 db.getAutoma().Transizione("csb");
				 classifyNaiveBayesianAllTestingPages();
			 }catch(Exception e) {new StampaErrore(f,"Classify all testing pages",
							"Exception:\n"+e.getMessage());e.printStackTrace();} 
			 finally {
				 setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				 resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				classify1Button.enable();
				classify2Button.enable();
				classify3Button.enable();
				classify4Button.enable();
				classify5Button.enable();
				clearButton.enable();
				cancelButton.enable();
			 }
		   return true;
		}
		else
		if (evt.arg.equals("Clear")) {
			classify1Button.disable();
			classify2Button.disable();
			classify3Button.disable();
			classify4Button.disable();
			classify5Button.disable();
			clearButton.disable();
			cancelButton.disable();
			setCursor(new Cursor(Cursor.WAIT_CURSOR));
		 	resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		   	resultText.setText("");
		 	setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		 	resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			classify1Button.enable();
			classify2Button.enable();
			classify3Button.enable();
			classify4Button.enable();
			classify5Button.enable();
			clearButton.enable();
			cancelButton.enable();
		   	return true;
		}
		else
		if (evt.arg.equals("Cancel")) {
		 setCursor(new Cursor(Cursor.WAIT_CURSOR));
		 resultText.setCursor(new Cursor(Cursor.WAIT_CURSOR));
		   resultText.setText("");
		 setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		 resultText.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		   hide();
		   return true;
		}
	  }  
	}
	super.handleEvent(evt);
	return true;
  } 

} 
