package uniba.gui;

import java.awt.*;
import java.util.*;
import uniba.core.*;
import uniba.databanker.*;

/** 
 * Browse category centroids.
 * @author Giorgio Vespucci (GV) 1998
 */
public class ViewCentroidsDialog extends Dialog {

   private Panel buttonsPanel;
   private Button okButton;
   private java.awt.List categoriesList;
   private java.awt.List centroidsList;
   private String currentCategory;
   private FeaturesManager ftm;
   private CategoriesManager cgm;
   private Databanker db = null;
   private String utente = null;
   private Frame parent = null;
   
   public ViewCentroidsDialog(Frame parent, 
                              FeaturesManager ftm, 
					CategoriesManager cgm, 
                              String utente,
                              Databanker db) {

	  super(parent, "Centroids browser -" + 
					" Click on a category to view its centroid", true);
	  move(100, 100);
	  resize(500, 450);
	  setResizable(false);
	  setBackground(new Color(0).lightGray);
	  
	  okButton = new Button("Ok");
	  buttonsPanel = new Panel();
	  buttonsPanel.add(okButton);
	  add("South", buttonsPanel);
			
	  // Object that shows categories.
	  categoriesList = new java.awt.List(10, false);
		 categoriesList.setForeground(new Color(0).black);
		 categoriesList.setBackground(new Color(0).white);
		 categoriesList.setFont(new Font("Courier", Font.PLAIN, 12));
	  Panel categoriesPanel = new Panel();
		 categoriesPanel.setLayout(new BorderLayout());
		 categoriesPanel.add("North", new Label("Categories"));
		 categoriesPanel.add("Center", categoriesList);

	  // Object that shows centroids.
	  centroidsList = new java.awt.List(25, false);
		 centroidsList.setForeground(new Color(0).blue);           
		 centroidsList.setBackground(new Color(0).white);           
		 centroidsList.setFont(new Font("Courier", Font.PLAIN, 12));   
	  Panel centroidsPanel = new Panel();
		 centroidsPanel.setLayout(new BorderLayout());
		 centroidsPanel.add("North", new Label("Features and values"));
		 centroidsPanel.add("Center", centroidsList);  
	  
	  // Object that shows both.
	  Panel centerPanel = new Panel();
		 centerPanel.setLayout(new GridLayout(1,2));
		 centerPanel.add(categoriesPanel);
		 centerPanel.add(centroidsPanel);
	  add("Center", centerPanel);
	  
	  // Catch parameters.
	  this.ftm = ftm;
	  this.cgm = cgm;   
        this.db = db;
        this.utente = utente;
        this.parent = parent;
	 
	  // Fill list and show.
	  showCategoriesList();   
   }   
   /** Watch for okButton.
   */
   public boolean action(Event evt, Object arg) {
	  if(evt.target instanceof Button) {
		 if(arg.equals("Ok")) {
			dispose();
		 }
		 return true;
	  }
	  return false;
   }   
   /** Remove dialog when terminated by user.
   */
   public boolean handleEvent(Event evt) {
	  if(evt.id == evt.WINDOW_DESTROY) {
		 dispose();
		 return true;
	  }
	  if(evt.id == evt.LIST_SELECT) {
		 if(evt.target == categoriesList) {
			java.awt.List t = (java.awt.List)evt.target;
			currentCategory = t.getSelectedItem();
			showCategoryCentroid(currentCategory);
			return true;
		 }
	  } 
	  return super.handleEvent(evt);
   }   


   public void show() {
	 showCategoriesList();   
	 super.show();
   }   


   /*
    * Show categories in list.
    */
   private void showCategoriesList() {
	  categoriesList.clear();
	  Enumeration enum = cgm.listTrainingCategories();
	  while(enum.hasMoreElements()) 
			categoriesList.addItem((String)enum.nextElement());
   }

   
   /*
    * Show the centroid of the category.
    */
   private void showCategoryCentroid(String category) {
	  
	 try {
	     centroidsList.clear();
           double centroid[] = db.getCentroid(utente, category); 
           
           // Send feature + value to centroids list.        
	     Enumeration enum = ftm.listFeatures();
	     String spazi = "                              ";
	     String fea = "";	 
	     for(int j=0; enum.hasMoreElements() && j<ftm.getTotalFeatures();j++) {
                fea = (String)enum.nextElement()+spazi;
	          centroidsList.addItem( fea.substring(0,15) + centroid[j]);
	     }	
	 } 
	 catch(Exception e) {
		 new StampaErrore(parent,
 				      "ViewCentroidsDialog",
                              "Exception:\n" + e.getMessage());               
	 }
 
   }   
} // end class